@@ -47,7 +47,7 @@
 #define	TEXT_FONT		("Serif")
 #define	FONT_HEIGHT		((gdouble) BST_GCONFIG (snet_font_size))
 #define RGBA_BLACK		(0x000000ff)
-
+#define RGBA_INTERNAL           (0x0000ffff)
 
 /* --- signals --- */
 enum
@@ -60,10 +60,6 @@ typedef void    (*SignalUpdateLinks)            (BstCanvasSource       *source,
 
 
 /* --- prototypes --- */
-static void	bst_canvas_source_build		(BstCanvasSource	*csource);
-static void	bst_canvas_source_destroy	(GtkObject		*object);
-static gboolean bst_canvas_source_event		(GnomeCanvasItem        *item,
-						 GdkEvent               *event);
 static gboolean bst_canvas_source_child_event	(BstCanvasSource	*csource,
 						 GdkEvent               *event,
 						 GnomeCanvasItem        *child);
@@ -72,38 +68,16 @@ static void	bst_canvas_icon_set		(GnomeCanvasItem	*item,
 						 BseIcon         	*icon,
                                                  const gchar            *module_type);
 static void	csource_info_update		(BstCanvasSource	*csource);
+static void     bst_canvas_source_build         (BstCanvasSource        *csource);
 
 
 /* --- static variables --- */
-static BstCanvasSourceClass *bst_canvas_source_class = NULL;
 static guint                 csource_signals[SIGNAL_LAST] = { 0 };
 
 
 /* --- functions --- */
 G_DEFINE_TYPE (BstCanvasSource, bst_canvas_source, GNOME_TYPE_CANVAS_GROUP);
 
-static void
-bst_canvas_source_class_init (BstCanvasSourceClass *class)
-{
-  GtkObjectClass *object_class = GTK_OBJECT_CLASS (class);
-  GnomeCanvasItemClass *canvas_item_class = GNOME_CANVAS_ITEM_CLASS (class);
-  /* GnomeCanvasGroupClass *canvas_group_class = GNOME_CANVAS_GROUP_CLASS (class); */
-  
-  bst_canvas_source_class = class;
-  
-  object_class->destroy = bst_canvas_source_destroy;
-  canvas_item_class->event = bst_canvas_source_event;
-  class->update_links = NULL;
-
-  csource_signals[SIGNAL_UPDATE_LINKS] =
-    gtk_signal_new ("update-links",
-		    GTK_RUN_LAST,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (BstCanvasSourceClass, update_links),
-		    gtk_signal_default_marshaller,
-		    GTK_TYPE_NONE, 0);
-}
-
 static void
 bst_canvas_source_init (BstCanvasSource *csource)
 {
@@ -216,7 +190,8 @@ source_icon_changed (BstCanvasSource *csource)
   /* update icon in group, revert to a stock icon if none is available
    */
   icon = bse_item_get_icon (csource->source);
-  bst_canvas_icon_set (csource->icon_item, icon, bse_item_get_type (csource->source));
+  if (csource->icon_item)
+    bst_canvas_icon_set (csource->icon_item, icon, bse_item_get_type (csource->source));
 }
 
 static void
@@ -225,6 +200,12 @@ bst_canvas_source_destroy (GtkObject *object)
   BstCanvasSource *csource = BST_CANVAS_SOURCE (object);
   GnomeCanvasGroup *group = GNOME_CANVAS_GROUP (object);
 
+  if (csource->in_move)
+    {
+      csource->in_move = FALSE;
+      bse_item_ungroup_undo (csource->source);
+    }
+
   while (csource->channel_hints)
     gtk_object_destroy (csource->channel_hints->data);
   while (group->item_list)
@@ -283,7 +264,7 @@ bst_canvas_source_new (GnomeCanvasGroup *group,
 		     NULL);
 
   canvas_source_set_position (csource);
-  bst_canvas_source_build (BST_CANVAS_SOURCE (item));
+  bst_canvas_source_build (csource);
   
   GNOME_CANVAS_NOTIFY (item);
   
@@ -674,7 +655,9 @@ bst_canvas_source_build_channels (BstCanvasSource *csource,
 				  gint             color1,
 				  gint		   color1_fade,
 				  gint		   color2,
-				  gint		   color2_fade)
+				  gint		   color2_fade,
+                                  gboolean         build_channel_items,
+                                  gboolean         build_channel_hints)
 {
   GnomeCanvasGroup *group = GNOME_CANVAS_GROUP (csource);
   const guint alpha = 0xa0;
@@ -719,20 +702,22 @@ bst_canvas_source_build_channels (BstCanvasSource *csource,
   else if (n_channels == 0)
     {
       GnomeCanvasItem *item;
-
-      item = g_object_connect (gnome_canvas_item_new (group,
-						      GNOME_TYPE_CANVAS_RECT,
-						      "fill_color_rgba", (0xc3c3c3 << 8) | alpha,
-						      "outline_color_rgba", RGBA_BLACK,
-						      "x1", x1,
-						      "y1", y1,
-						      "x2", x2,
-						      "y2", y2,
-						      NULL),
-			       "swapped_signal::destroy", channel_item_remove, csource,
-			       "swapped_signal::event", bst_canvas_source_child_event, csource,
-			       NULL);
-      csource->channel_items = g_slist_prepend (csource->channel_items, item);
+      if (build_channel_items)
+        {
+          item = g_object_connect (gnome_canvas_item_new (group,
+                                                          GNOME_TYPE_CANVAS_RECT,
+                                                          "fill_color_rgba", (0xc3c3c3 << 8) | alpha,
+                                                          "outline_color_rgba", RGBA_BLACK,
+                                                          "x1", x1,
+                                                          "y1", y1,
+                                                          "x2", x2,
+                                                          "y2", y2,
+                                                          NULL),
+                                   "swapped_signal::destroy", channel_item_remove, csource,
+                                   "swapped_signal::event", bst_canvas_source_child_event, csource,
+                                   NULL);
+          csource->channel_items = g_slist_prepend (csource->channel_items, item);
+        }
     }
   
   for (i = 0; i < n_channels; i++)
@@ -743,157 +728,183 @@ bst_canvas_source_build_channels (BstCanvasSource *csource,
       guint tmp_color = is_jchannel ? color2 : color1;
 
       y2 = y1 + d_y;
-      item = g_object_connect (gnome_canvas_item_new (group,
-						      GNOME_TYPE_CANVAS_RECT,
-						      "fill_color_rgba", (tmp_color << 8) | alpha,
-						      "outline_color_rgba", RGBA_BLACK,
-						      "x1", x1,
-						      "y1", y1,
-						      "x2", x2,
-						      "y2", y2,
-						      NULL),
-			       "swapped_signal::destroy", channel_item_remove, csource,
-			       "swapped_signal::event", bst_canvas_source_child_event, csource,
-			       NULL);
-      csource->channel_items = g_slist_prepend (csource->channel_items, item);
+      if (build_channel_items)
+        {
+          item = gnome_canvas_item_new (group,
+                                        GNOME_TYPE_CANVAS_RECT,
+                                        "fill_color_rgba", (tmp_color << 8) | alpha,
+                                        "outline_color_rgba", RGBA_BLACK,
+                                        "x1", x1,
+                                        "y1", y1,
+                                        "x2", x2,
+                                        "y2", y2,
+                                        NULL);
+          g_object_connect (item,
+                            "swapped_signal::destroy", channel_item_remove, csource,
+                            "swapped_signal::event", bst_canvas_source_child_event, csource,
+                            NULL);
+          csource->channel_items = g_slist_prepend (csource->channel_items, item);
+        }
+
+      if (build_channel_hints)
+        {
+          item = gnome_canvas_item_new (group,
+                                        GNOME_TYPE_CANVAS_TEXT,
+                                        "fill_color_rgba", (0x000000 << 8) | 0x80,
+                                        "anchor", east_channel ? GTK_ANCHOR_WEST : GTK_ANCHOR_EAST,
+                                        "justification", GTK_JUSTIFY_RIGHT,
+                                        "x", east_channel ? TOTAL_WIDTH (csource) + BORDER_PAD * 2. : -BORDER_PAD,
+                                        "y", (y1 + y2) / 2.,
+                                        "font", CHANNEL_FONT,
+                                        "text", csource->show_hints ? label : "",
+                                        NULL);
+          g_object_connect (item,
+                            "swapped_signal::destroy", channel_name_remove, csource,
+                            NULL);
+          gnome_canvas_text_set_zoom_size (GNOME_CANVAS_TEXT (item), FONT_HEIGHT);
+          g_object_set_data_full (G_OBJECT (item), "hint_text", g_strdup (label), g_free);
+          csource->channel_hints = g_slist_prepend (csource->channel_hints, item);
+        }
 
-      item = gnome_canvas_item_new (group,
-				    GNOME_TYPE_CANVAS_TEXT,
-				    "fill_color_rgba", (0x000000 << 8) | 0x80,
-				    "anchor", east_channel ? GTK_ANCHOR_WEST : GTK_ANCHOR_EAST,
-				    "justification", GTK_JUSTIFY_RIGHT,
-				    "x", east_channel ? TOTAL_WIDTH (csource) + BORDER_PAD * 2. : -BORDER_PAD,
-				    "y", (y1 + y2) / 2.,
-				    "font", CHANNEL_FONT,
-				    "text", csource->show_hints ? label : "",
-				    NULL);
-      item = g_object_connect (item,
-			       "swapped_signal::destroy", channel_name_remove, csource,
-			       NULL);
-      gnome_canvas_text_set_zoom_size (GNOME_CANVAS_TEXT (item), FONT_HEIGHT);
-      g_object_set_data_full (G_OBJECT (item), "hint_text", g_strdup (label), g_free);
-      csource->channel_hints = g_slist_prepend (csource->channel_hints, item);
-      
       color1 += color1_delta;
       color2 += color2_delta;
       y1 = y2;
     }
 }
 
+static gboolean
+bst_canvas_source_build_async (gpointer data)
+{
+  GnomeCanvasItem *item = GNOME_CANVAS_ITEM (data);
+  if (gnome_canvas_item_check_undisposed (item))
+    {
+      BstCanvasSource *csource = BST_CANVAS_SOURCE (item);
+      GnomeCanvasGroup *group = GNOME_CANVAS_GROUP (csource);
+      
+      /* keep in mind, that creation order affects stacking */
+
+      /* add input and output channel items */
+      if (!csource->built_ichannels)
+        {
+          csource->built_ichannels = TRUE;
+          bst_canvas_source_build_channels (csource,
+                                            TRUE,               /* input channels */
+                                            0xffff00, 0x808000,	/* ichannels */
+                                            0x00afff, 0x005880, /* jchannels */
+                                            TRUE, FALSE);
+          return TRUE;
+        }
+      if (!csource->built_ochannels)
+        {
+          csource->built_ochannels = TRUE;
+          bst_canvas_source_build_channels (csource,
+                                            FALSE,              /* output channels */
+                                            0xff0000, 0x800000, /* ochannels */
+                                            0, 0,               /* unused */
+                                            TRUE, FALSE);
+          return TRUE;
+        }
+
+      /* add icon to group */
+      if (!csource->icon_item)
+        {
+          csource->icon_item = g_object_connect (gnome_canvas_item_new (group,
+                                                                        GNOME_TYPE_CANVAS_PIXBUF,
+                                                                        "x", ICON_X (csource),
+                                                                        "y", ICON_Y (csource),
+                                                                        "width", ICON_WIDTH (csource),
+                                                                        "height", ICON_HEIGHT (csource),
+                                                                        NULL),
+                                                 "signal::destroy", gtk_widget_destroyed, &csource->icon_item,
+                                                 "swapped_signal::event", bst_canvas_source_child_event, csource,
+                                                 NULL);
+          source_icon_changed (csource);
+          return TRUE;
+        }
+      
+      if (!csource->text)
+        {
+          /* add text item, invoke name_changed callback to setup the text value */
+          guint ocolor = csource->source && bse_item_internal (csource->source) ? RGBA_INTERNAL : RGBA_BLACK;
+          csource->text = gnome_canvas_item_new (group,
+                                                 GNOME_TYPE_CANVAS_TEXT,
+                                                 "fill_color_rgba", ocolor,
+                                                 "anchor", GTK_ANCHOR_NORTH,
+                                                 "justification", GTK_JUSTIFY_CENTER,
+                                                 "x", TEXT_X (csource),
+                                                 "y", TEXT_Y (csource),
+                                                 "font", TEXT_FONT,
+                                                 NULL);
+          g_object_connect (csource->text,
+                            "signal::destroy", gtk_widget_destroyed, &csource->text,
+                            "swapped_signal::event", bst_canvas_source_child_event, csource,
+                            NULL);
+          gnome_canvas_text_set_zoom_size (GNOME_CANVAS_TEXT (csource->text), FONT_HEIGHT);
+          source_name_changed (csource);
+          return TRUE;
+        }
+
+      /* add input and output channel hints */
+      if (!csource->built_ihints)
+        {
+          csource->built_ihints = TRUE;
+          bst_canvas_source_build_channels (csource,
+                                            TRUE,               /* input channels */
+                                            0xffff00, 0x808000,	/* ichannels */
+                                            0x00afff, 0x005880, /* jchannels */
+                                            FALSE, TRUE);
+          return TRUE;
+        }
+      if (!csource->built_ohints)
+        {
+          csource->built_ohints = TRUE;
+          bst_canvas_source_build_channels (csource,
+                                            FALSE,              /* output channels */
+                                            0xff0000, 0x800000, /* ochannels */
+                                            0, 0,               /* unused */
+                                            FALSE, TRUE);
+          return TRUE;
+        }
+    }
+  GnomeCanvas *canvas = item->canvas;
+  g_object_unref (item);
+  g_object_unref (canvas);      /* canvases don't properly protect their items */
+  return FALSE;
+}
+
 static void
 bst_canvas_source_build (BstCanvasSource *csource)
 {
   GnomeCanvasGroup *group = GNOME_CANVAS_GROUP (csource);
-  GnomeCanvasPoints *gpoints;
-  GnomeCanvasItem *item;
-
-  /* order of creation is important to enforce stacking */
-
-  /* add icon to group
-   */
-  csource->icon_item = g_object_connect (gnome_canvas_item_new (group,
-								GNOME_TYPE_CANVAS_PIXBUF,
-								"x", ICON_X (csource),
-								"y", ICON_Y (csource),
-								"width", ICON_WIDTH (csource),
-								"height", ICON_HEIGHT (csource),
-								NULL),
-					 "signal::destroy", gtk_widget_destroyed, &csource->icon_item,
-					 "swapped_signal::event", bst_canvas_source_child_event, csource,
-					 NULL);
-  source_icon_changed (csource);
-
-  /* add text item, invoke name_changed callback to setup the text value
-   */
-  csource->text = gnome_canvas_item_new (group,
-					 GNOME_TYPE_CANVAS_TEXT,
-					 "fill_color", "black",
-					 "anchor", GTK_ANCHOR_NORTH,
-					 "justification", GTK_JUSTIFY_CENTER,
-					 "x", TEXT_X (csource),
-					 "y", TEXT_Y (csource),
-					 "font", TEXT_FONT,
-					 NULL);
-  g_object_connect (csource->text,
-		    "signal::destroy", gtk_widget_destroyed, &csource->text,
-		    "swapped_signal::event", bst_canvas_source_child_event, csource,
-		    NULL);
-  gnome_canvas_text_set_zoom_size (GNOME_CANVAS_TEXT (csource->text), FONT_HEIGHT);
-  source_name_changed (csource);
-
-  /* add input and output channel items
+  /* put an outer rectangle, make it transparent in aa mode,
+   * so we can receive mouse events everywhere
    */
+  GnomeCanvasItem *rect = gnome_canvas_item_new (group,
+                                                 GNOME_TYPE_CANVAS_RECT,
+                                                 "outline_color_rgba", RGBA_BLACK, /* covers buggy canvas lines */
+                                                 "x1", 0.0,
+                                                 "y1", 0.0,
+                                                 "x2", TOTAL_WIDTH (csource),
+                                                 "y2", TOTAL_HEIGHT (csource),
+                                                 (GNOME_CANVAS_ITEM (csource)->canvas->aa
+                                                  ? "fill_color_rgba"
+                                                  : NULL), 0x00000000,
+                                                 NULL);
+  g_object_connect (rect,
+                    "swapped_signal::event", bst_canvas_source_child_event, csource,
+                    NULL);
+  /* make sure no items are left over */
   while (csource->channel_items)
     gtk_object_destroy (csource->channel_items->data);
   while (csource->channel_hints)
     gtk_object_destroy (csource->channel_hints->data);
-  bst_canvas_source_build_channels (csource,
-				    TRUE, /* input channels */
-				    0xffff00, 0x808000,	  /* ichannels */
-				    0x00afff, 0x005880);  /* jchannels */
-  bst_canvas_source_build_channels (csource,
-				    FALSE, /* output channels */
-				    0xff0000, 0x800000,
-				    0, 0); /* unused */
-
-  /* put line above and below the icon
-   */
-  if (0)
-    {
-      gpoints = gnome_canvas_points_newv (2, ICON_X (csource), ICON_Y (csource), ICON_X (csource) + ICON_WIDTH (csource), ICON_Y (csource));
-      item = g_object_connect (gnome_canvas_item_new (group,
-						      GNOME_TYPE_CANVAS_LINE,
-						      "fill_color_rgba", RGBA_BLACK,
-						      "points", gpoints,
-						      NULL),
-			       "swapped_signal::event", bst_canvas_source_child_event, csource,
-			       NULL);
-      gpoints->coords[1] += ICON_HEIGHT (csource);
-      gpoints->coords[3] += ICON_HEIGHT (csource);
-      item = g_object_connect (gnome_canvas_item_new (group,
-						      GNOME_TYPE_CANVAS_LINE,
-						      "fill_color_rgba", RGBA_BLACK,
-						      "points", gpoints,
-						      NULL),
-			       "swapped_signal::event", bst_canvas_source_child_event, csource,
-			       NULL);
-      gnome_canvas_points_free (gpoints);
-    }
-  
-#if 0
-#define ICON_BOTTOM             ((gdouble) ICON_Y + ICON_HEIGHT)
-  /* put a line at the bottom (ontop of the text) to close
-   * text bounding rectangle
-   */
-  gpoints = gnome_canvas_points_newv (2,
-				      CHANNEL_WIDTH (csource), ICON_BOTTOM,
-				      CHANNEL_WIDTH (csource) + ICON_WIDTH, ICON_BOTTOM);
-  item = g_object_connect (gnome_canvas_item_new (group,
-						  GNOME_TYPE_CANVAS_LINE,
-						  "fill_color_rgba", RGBA_BLACK,
-						  "points", gpoints,
-						  NULL),
-			   "swapped_signal::event", bst_canvas_source_child_event, csource,
-			   NULL);
-  gnome_canvas_points_free (gpoints);
-#endif
-
-  /* put an outer rectangle, make it transparent in aa mode,
-   * so we can receive mouse events everywhere
-   */
-  item = g_object_connect (gnome_canvas_item_new (group,
-						  GNOME_TYPE_CANVAS_RECT,
-						  "outline_color_rgba", RGBA_BLACK, /* covers buggy canvas lines */
-						  "x1", 0.0,
-						  "y1", 0.0,
-						  "x2", TOTAL_WIDTH (csource),
-						  "y2", TOTAL_HEIGHT (csource),
-						  (GNOME_CANVAS_ITEM (csource)->canvas->aa
-						   ? "fill_color_rgba"
-						   : NULL), 0x00000000,
-						  NULL),
-			   "swapped_signal::event", bst_canvas_source_child_event, csource,
-			   NULL);
+  csource->built_ichannels = FALSE;
+  csource->built_ochannels = FALSE;
+  csource->built_ihints = FALSE;
+  csource->built_ohints = FALSE;
+  /* asyncronously rebuild contents */
+  g_object_ref (GNOME_CANVAS_ITEM (csource)->canvas);   /* GnomeCanvasItem doesn't properly clear its pointers */
+  bst_background_handler2_add (bst_canvas_source_build_async, g_object_ref (csource), NULL);
 }
 
 static void
@@ -903,7 +914,6 @@ bst_canvas_source_changed (BstCanvasSource *csource)
     {
       GnomeCanvasItem *item = GNOME_CANVAS_ITEM (csource);
       gdouble x = 0, y = 0;
-
       gnome_canvas_item_i2w (item, &x, &y);
       bse_object_set_parasite_coords (csource->source, x, y);
     }
@@ -919,21 +929,21 @@ bst_canvas_source_event (GnomeCanvasItem *item,
   switch (event->type)
     {
     case GDK_BUTTON_PRESS:
-      if (event->button.button == 2)
+      if (!csource->in_move && event->button.button == 2)
 	{
-	  GdkCursor *fleur;
-	  gdouble x = event->button.x, y = event->button.y;
-	  
-	  gnome_canvas_item_w2i (item, &x, &y);
-	  csource->move_dx = x;
-	  csource->move_dy = y;
-	  csource->in_move = TRUE;
-	  
-	  fleur = gdk_cursor_new (GDK_FLEUR);
-	  gnome_canvas_item_grab (item,
-				  GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
-				  fleur,
-				  event->button.time);
+	  GdkCursor *fleur = gdk_cursor_new (GDK_FLEUR);
+	  if (gnome_canvas_item_grab (item,
+                                      GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
+                                      fleur,
+                                      event->button.time) == 0)
+            {
+              gdouble x = event->button.x, y = event->button.y;
+              gnome_canvas_item_w2i (item, &x, &y);
+              csource->move_dx = x;
+              csource->move_dy = y;
+              csource->in_move = TRUE;
+              bse_item_group_undo (csource->source, "Move");
+            }
 	  gdk_cursor_destroy (fleur);
 	  handled = TRUE;
 	}
@@ -980,6 +990,7 @@ bst_canvas_source_event (GnomeCanvasItem *item,
     case GDK_BUTTON_RELEASE:
       if (event->button.button == 2 && csource->in_move)
 	{
+          bse_item_ungroup_undo (csource->source);
 	  csource->in_move = FALSE;
 	  gnome_canvas_item_ungrab (item, event->button.time);
 	  handled = TRUE;
@@ -1041,3 +1052,23 @@ bst_canvas_source_child_event (BstCanvasSource *csource,
   
   return handled;
 }
+
+static void
+bst_canvas_source_class_init (BstCanvasSourceClass *class)
+{
+  GtkObjectClass *object_class = GTK_OBJECT_CLASS (class);
+  GnomeCanvasItemClass *canvas_item_class = GNOME_CANVAS_ITEM_CLASS (class);
+  /* GnomeCanvasGroupClass *canvas_group_class = GNOME_CANVAS_GROUP_CLASS (class); */
+  
+  object_class->destroy = bst_canvas_source_destroy;
+  canvas_item_class->event = bst_canvas_source_event;
+  class->update_links = NULL;
+
+  csource_signals[SIGNAL_UPDATE_LINKS] =
+    gtk_signal_new ("update-links",
+		    GTK_RUN_LAST,
+		    GTK_CLASS_TYPE (object_class),
+		    GTK_SIGNAL_OFFSET (BstCanvasSourceClass, update_links),
+		    gtk_signal_default_marshaller,
+		    GTK_TYPE_NONE, 0);
+}
