@@ -248,14 +248,15 @@ main (int   argc,
 	}
     }
 
-  /* listen to BseServer notification
-   */
+  /* listen to BseServer notification */
   bst_splash_update_entity (beast_splash, _("Dialogs"));
   bst_catch_scripts_and_msgs ();
   _bst_init_radgets ();
 
-  /* open files given on command line
-   */
+  /* install message dialog handler */
+  sfi_log_set_thread_handler (bst_user_message_log_handler);
+
+  /* open files given on command line */
   if (argc > 1)
     bst_splash_update_entity (beast_splash, _("Loading..."));
   for (i = 1; i < argc; i++)
@@ -306,7 +307,7 @@ main (int   argc,
       bse_item_unuse (project);
       
       if (error)
-	bst_status_eprintf (error, _("Loading project \"%s\""), argv[i]);
+        sfi_error (_("Failed to load project \"%s\": %s"), argv[i], bse_error_blurb (error));
     }
 
   /* open default app window
@@ -333,10 +334,9 @@ main (int   argc,
       bst_gconfig_set_rc_version (BST_VERSION);
     }
 
-  /* release splash grab, install message dialog handler */
+  /* release splash grab */
   gtk_widget_hide (beast_splash);
   bst_splash_release_grab (beast_splash);
-  sfi_log_set_thread_handler (bst_user_message_log_handler);
   /* away into the main loop */
   while (bst_main_loop_running)
     {
@@ -345,9 +345,9 @@ main (int   argc,
       g_main_iteration (TRUE);
       GDK_THREADS_ENTER ();
     }
-  sfi_log_set_thread_handler (NULL);
   
-  /* take down GUI leftovers */
+  /* take down GUI */
+  sfi_log_set_thread_handler (NULL);
   bst_user_messages_kill ();
   
   /* perform necessary cleanup cycles */
@@ -414,7 +414,12 @@ bst_early_parse_args (int    *argc_p,
   gboolean initialize_bse_and_exit = FALSE;
   for (i = 1; i < argc; i++)
     {
-      if (strncmp (argv[i], "-:", 2) == 0)
+      if (strcmp (argv[i], "--") == 0)
+        {
+          argv[i] = NULL;
+          break;
+        }
+      else if (strncmp (argv[i], "-:", 2) == 0)
 	{
 	  const gchar *flags = argv[i] + 2;
 	  g_printerr ("BEAST(%s): pid = %u\n", BST_VERSION, getpid ());
@@ -500,6 +505,17 @@ bst_early_parse_args (int    *argc_p,
 	  arg_force_xkb = TRUE;
           argv[i] = NULL;
 	}
+      else if (strcmp (argv[i], "-n") == 0 && i + 1 < argc)
+        { /* handled by priviledged launcher */
+          argv[i++] = NULL;
+          argv[i] = NULL;
+        }
+      else if (strncmp (argv[i], "-n=", 3) == 0 ||
+               (strncmp (argv[i], "-n", 2) == 0 && argv[i][3] >= '0' && argv[i][3] <= '9') ||
+               strcmp (argv[i], "-N") == 0)
+        { /* handled by priviledged launcher */
+          argv[i] = NULL;
+        }
       else if (strcmp ("-h", argv[i]) == 0 ||
 	       strcmp ("--help", argv[i]) == 0)
 	{
@@ -683,6 +699,8 @@ bst_print_blurb (void)
   g_print ("                          enable unstable plugins and experimental code\n");
   g_print ("  -h, --help              show this help message\n");
   g_print ("  -v, --version           print version and file paths\n");
+  g_print ("  -n NICELEVEL            run with priority NICELEVEL (for suid wrapper beast)\n");
+  g_print ("  -N                      disable renicing\n");
   g_print ("  --display=DISPLAY       X server for the GUI; see X(1)\n");
   g_print ("  --bse-latency=USECONDS  specify synthesis latency in milliseconds\n");
   g_print ("  --bse-mixing-freq=FREQ  specify synthesis mixing frequency in Hz \n");
@@ -758,6 +776,7 @@ beast_show_about_box (void)
     /* translations */
     "Adam Weinberger",
     "Alexandre Prokoudine",
+    "Amanpreet Singh Alam",
     //"Artem Popov",
     "Christian Neumair",
     "Christian Rose",
