@@ -139,12 +139,13 @@ pattern_column_note_draw_cell (BstPatternColumn       *column,
                                guint                   tick,
                                guint                   duration,
                                GdkRectangle           *crect,
-                               GdkRectangle           *expose_area)
+                               GdkRectangle           *expose_area,
+                               GdkGC                  *gcs[BST_PATTERN_COLUMN_GC_LAST])
 {
   BstPatternColumnNote *self = (BstPatternColumnNote*) column;
   PangoRectangle prect = { 0 };
-  GdkGC *dark_gc = STYLE (pview)->dark_gc[GTK_STATE_NORMAL];
-  GdkGC *black_gc = STYLE (pview)->black_gc;
+  GdkGC *inactive_gc = gcs[BST_PATTERN_COLUMN_GC_TEXT0];
+  GdkGC *text_gc = gcs[BST_PATTERN_COLUMN_GC_TEXT1];
   GdkGC *draw_gc;
   SfiProxy proxy = pview->proxy;
   BsePartNoteSeq *pseq = proxy ? bse_part_list_notes_within (proxy, column->num, tick, duration) : NULL;
@@ -153,7 +154,7 @@ pattern_column_note_draw_cell (BstPatternColumn       *column,
 
   accu = crect->x + FOCUS_WIDTH (pview);
   ch = pattern_column_note_char (self, pseq, 0);
-  draw_gc = ch == '-' ? dark_gc : black_gc;      /* choose gc dependant on note letter */
+  draw_gc = ch == '-' ? inactive_gc : text_gc;      /* choose gc dependant on note letter */
   pango_layout_set_text (pango_layout, &ch, 1);
   pango_layout_get_pixel_extents (pango_layout, NULL, &prect);
   yline = crect->y + (crect->height - prect.height) / 2;
@@ -439,16 +440,17 @@ pattern_column_event_draw_cell (BstPatternColumn       *column,
                                 guint                   tick,
                                 guint                   duration,
                                 GdkRectangle           *crect,
-                                GdkRectangle           *expose_area)
+                                GdkRectangle           *expose_area,
+                                GdkGC                  *gcs[BST_PATTERN_COLUMN_GC_LAST])
 {
   BstPatternColumnEvent *self = (BstPatternColumnEvent*) column;
-  GdkGC *dark_gc = STYLE (pview)->dark_gc[GTK_STATE_NORMAL];
-  GdkGC *black_gc = STYLE (pview)->black_gc;
+  GdkGC *inactive_gc = gcs[BST_PATTERN_COLUMN_GC_TEXT0];
+  GdkGC *text_gc = gcs[BST_PATTERN_COLUMN_GC_TEXT1];
   gchar placeholder;
   BsePartControl *pctrl = pattern_column_event_lookup (column, pview, tick, duration, NULL, &placeholder);
   gchar buffer[64] = { 0, };
   guint n = pattern_column_event_to_string (column, buffer, pctrl, placeholder, NULL);
-  GdkGC *draw_gc = placeholder == '-' ? dark_gc : black_gc;
+  GdkGC *draw_gc = placeholder == '-' ? inactive_gc : text_gc;
   gint i, yline, accu = crect->x + FOCUS_WIDTH (pview) + 1;
 
   for (i = 0; i < n; i++)
@@ -622,9 +624,10 @@ pattern_column_vbar_draw_cell (BstPatternColumn       *self,
                                guint                   tick,
                                guint                   duration,
                                GdkRectangle           *crect,
-                               GdkRectangle           *expose_area)
+                               GdkRectangle           *expose_area,
+                               GdkGC                  *gcs[BST_PATTERN_COLUMN_GC_LAST])
 {
-  GdkGC *draw_gc = STYLE (pview)->dark_gc[GTK_STATE_NORMAL];
+  GdkGC *draw_gc = gcs[BST_PATTERN_COLUMN_GC_VBAR];
   gint dlen, line_width = 0; /* line widths != 0 interfere with dash-settings on some X servers */
   if (self->num < 0)
     {
@@ -666,7 +669,7 @@ static void
 pattern_column_layouter_apply (GtkWidget *dialog)
 {
   BstPatternView *pview = g_object_get_data (dialog, "user_data");
-  GtkEntry *entry = gxk_gadget_find (dialog, "layout-entry");
+  GtkEntry *entry = gxk_radget_find (dialog, "layout-entry");
   const gchar *layout = gtk_entry_get_text (entry);
   guint l = bst_pattern_view_set_layout (pview, layout);
   if (l < strlen (layout))
@@ -733,11 +736,11 @@ bst_pattern_column_layouter_popup (BstPatternView *pview)
     {
       GtkWidget *sctext, *w;
       guint i;
-      dialog = gxk_dialog_new_gadget (NULL, GTK_OBJECT (pview),
+      dialog = gxk_dialog_new_radget (NULL, GTK_OBJECT (pview),
                                       GXK_DIALOG_HIDE_ON_DELETE | GXK_DIALOG_POPUP_POS, // GXK_DIALOG_MODAL
                                       _("Pattern Editor Layout"),
                                       "beast", "pattern-editor-layout-box");
-      sctext = gxk_scroll_text_create_for (GXK_SCROLL_TEXT_WIDGET_LOOK, gxk_gadget_find (dialog, "sctext-box"));
+      sctext = gxk_scroll_text_create_for (GXK_SCROLL_TEXT_WIDGET_LOOK, gxk_radget_find (dialog, "sctext-box"));
       gxk_scroll_text_clear (sctext);
       for (i = 0; i < G_N_ELEMENTS (help_text); i++)
         {
@@ -750,15 +753,15 @@ bst_pattern_column_layouter_popup (BstPatternView *pview)
           if (p > tx)
             gxk_scroll_text_pop_indent (sctext);
         }
-      g_signal_connect (gxk_gadget_find (dialog, "cancel-button"), "clicked",
+      g_signal_connect (gxk_radget_find (dialog, "cancel-button"), "clicked",
                         G_CALLBACK (gxk_toplevel_delete), NULL);
-      g_signal_connect_swapped (w = gxk_gadget_find (dialog, "apply-button"), "clicked",
+      g_signal_connect_swapped (w = gxk_radget_find (dialog, "apply-button"), "clicked",
                                 G_CALLBACK (pattern_column_layouter_apply), dialog);
       gxk_dialog_set_default (GXK_DIALOG (dialog), w);
       g_object_set_data (dialog, "user_data", pview);
       g_object_set_data (pview, "BstPattern-layouter", dialog);
     }
-  entry = gxk_gadget_find (dialog, "layout-entry");
+  entry = gxk_radget_find (dialog, "layout-entry");
   gtk_entry_set_text (entry, bst_pattern_view_get_layout (pview));
   gxk_widget_showraise (dialog);
 }
