@@ -23,6 +23,7 @@
 #include "bstmenus.h"
 #include "bstgconfig.h"
 #include "bstprocedure.h"
+#include "bstscrollgraph.h"
 #include <gdk/gdkkeysyms.h>
 
 
@@ -685,8 +686,16 @@ bst_snet_router_root_event (BstSNetRouter   *self,
                                                 ": ",
                                                 bse_item_get_name (csource->source),
                                                 NULL);
-              guint i, has_inputs = 0;
-              
+              /* create popup sumenu */
+              guint i, has_inputs = 0, monitor_ids = 1000000;
+              choice = bst_choice_menu_createv ("<BEAST-SNetRouter>/ModuleChannelPopup", NULL);
+              for (i = 0; i < bse_source_n_ochannels (csource->source); i++)
+                {
+                  gchar *name = g_strdup_printf ("%d: %s", i + 1, bse_source_ochannel_label (csource->source, i));
+                  bst_choice_menu_add_choice_and_free (choice, BST_CHOICE (monitor_ids + i, name, NONE));
+                  g_free (name);
+                }
+              /* create popup */
               for (i = 0; has_inputs == 0 && i < bse_source_n_ichannels (csource->source); i++)
                 has_inputs += bse_source_ichannel_get_n_joints (csource->source, i);
               choice = bst_choice_menu_createv ("<BEAST-SNetRouter>/ModulePopup",
@@ -696,7 +705,9 @@ bst_snet_router_root_event (BstSNetRouter   *self,
                                                 BST_CHOICE (6, _("Reset Properties"), PROPERTIES_RESET),
                                                 BST_CHOICE_S (3, _("Delete Inputs"), NO_ILINK, has_inputs),
                                                 BST_CHOICE_S (4, _("Delete Outputs"), NO_OLINK, bse_source_has_outputs (csource->source)),
+                                                BST_CHOICE_SEPERATOR,
                                                 BST_CHOICE (5, _("Show Info"), INFO),
+                                                BST_CHOICE_SUBMENU (_("Output Signal Monitor"), choice, SIGNAL),
                                                 BST_CHOICE_SEPERATOR,
                                                 BST_CHOICE_S (1, _("Delete"), DELETE, csource->source != self->snet),
                                                 BST_CHOICE_END);
@@ -704,6 +715,7 @@ bst_snet_router_root_event (BstSNetRouter   *self,
               i = bst_choice_modal (choice, event->button.button, event->button.time);
               switch (i)
                 {
+                  GtkWidget *dialog;
                   BseErrorType error;
                 case 2:
                   bst_canvas_source_popup_params (csource);
@@ -724,6 +736,11 @@ bst_snet_router_root_event (BstSNetRouter   *self,
                   error = bse_snet_remove_source (self->snet, csource->source);
                   bst_status_eprintf (error, _("Remove Module"));
                   break;
+                case 0: break;
+                default:
+                  dialog = bst_scrollgraph_build_dialog (GTK_WIDGET (self), csource->source, i - monitor_ids);
+                  gtk_widget_show (dialog);
+                  break;
                 }
               bst_choice_destroy (choice);
               /* FIXME: get rid of artifacts left behind removal (mostly rect-ellipse) */
@@ -1067,14 +1084,17 @@ bst_snet_router_init (BstSNetRouter      *self)
   palette_modules = gxk_action_list_create_grouped (self->canvas_tool);
   toolbar_modules = gxk_action_list_create_grouped (self->canvas_tool);
   cseq = bse_categories_match ("/Modules/*");
+  /* toolbar module types */
+  static struct { gchar *type, *name, *tip; } toolbar_types[] = {
+    { "BsePcmInput",        N_("Input"),      N_("PCM Input module") },
+    { "BseStandardOsc",     N_("Oscillator"), N_("Standard oscillator module") },
+    { "BseSimpleADSR",      N_("ADSR"),       N_("ADSR Envelope Generator") },
+    { "BseAmplifier",       N_("DCA"),        N_("Standard amplifier module") },
+    { "BsePcmOutput",       N_("Output"),     N_("PCM Output module") },
+  };
+  BseCategory *tbcats[G_N_ELEMENTS (toolbar_types)] = { 0, };
   for (i = 0; i < cseq->n_cats; i++)
     {
-      static struct { gchar *type, *name, *tip; } toolbar_types[] = {
-        { "BsePcmOutput",       N_("Output"),   N_("PCM Output module") },
-        { "BseAmplifier",       N_("DCA"),      N_("Standard amplifier module") },
-        { "BseSnooper",         N_("Snooper"),  N_("Signal debugging module") },
-        { "BsePcmInput",        N_("Input"),    N_("PCM Input module") },
-      };
       const gchar *stock_fallback = NULL;
       BseCategory *cat = cseq->cats[i];
       /* filter inappropriate modules */
@@ -1087,30 +1107,39 @@ bst_snet_router_init (BstSNetRouter      *self)
       /* provide selected modules in the palette */
       if (cat->icon && (cat->icon->width + cat->icon->height) > 0)
         bst_action_list_add_cat (palette_modules, cat, 1, stock_fallback, NULL, bst_router_popup_select, self);
-      /* provide certain variants in the toolbar */
+      /* remember toolbar types */
       for (n = 0; n < G_N_ELEMENTS (toolbar_types); n++)
         if (strcmp (toolbar_types[n].type, cat->type) == 0)
-          {
-            const gchar *stock_id;
-            if (cat->icon)
-              {
-                bst_stock_register_icon (cat->category, cat->icon->bytes_per_pixel,
-                                         cat->icon->width, cat->icon->height,
-                                         cat->icon->width * cat->icon->bytes_per_pixel,
-                                         cat->icon->pixels->bytes);
-                stock_id = cat->category;
-              }
-            else
-              stock_id = stock_fallback;
-            gxk_action_list_add_translated (toolbar_modules, cat->type,
-                                            _(toolbar_types[n].name), NULL,
-                                            _(toolbar_types[n].tip),
-                                            cat->category_id,
-                                            stock_id,
-                                            NULL, bst_router_popup_select, self);
-            break;
-          }
+          tbcats[n] = cat;
     }
+  /* provide certain variants in the toolbar */
+  for (n = 0; n < G_N_ELEMENTS (toolbar_types); n++)
+    if (tbcats[n])
+      {
+        BseCategory *cat = tbcats[n];
+        const gchar *stock_fallback = NULL;
+        if (strncmp (cat->type, "BseLadspaModule_", 16) == 0)
+          stock_fallback = BST_STOCK_LADSPA;
+        const gchar *stock_id;
+        
+        if (cat->icon)
+          {
+            bst_stock_register_icon (cat->category, cat->icon->bytes_per_pixel,
+                                     cat->icon->width, cat->icon->height,
+                                     cat->icon->width * cat->icon->bytes_per_pixel,
+                                     cat->icon->pixels->bytes);
+            stock_id = cat->category;
+          }
+        else
+          stock_id = stock_fallback;
+        gxk_action_list_add_translated (toolbar_modules, cat->type,
+                                        _(toolbar_types[n].name), NULL,
+                                        _(toolbar_types[n].tip),
+                                        cat->category_id,
+                                        stock_id,
+                                        NULL, bst_router_popup_select, self);
+      }
+  /* polish and publish */
   gxk_action_list_sort (canvas_modules);
   gxk_widget_publish_action_list (self, "router-module-actions", canvas_modules);
   gxk_action_list_sort (palette_modules);
