@@ -45,7 +45,7 @@ bst_track_roll_controller_canvas_actions (BstTrackRollController *self)
     { N_("Insert"),             "I",    N_("Insert/edit/move parts (mouse button 1 and 2)"),
       BST_GENERIC_ROLL_TOOL_INSERT,     BST_STOCK_PART_EDITOR },
     { N_("Link"),               "K",    N_("Link or move parts (mouse button 1 and 2)"),
-      BST_GENERIC_ROLL_TOOL_LINK,       BST_STOCK_PART_COPY },
+      BST_GENERIC_ROLL_TOOL_LINK,       BST_STOCK_PART_LINK },
     { N_("Rename"),             "E",    N_("Rename parts"),
       BST_GENERIC_ROLL_TOOL_RENAME,     BST_STOCK_PART_TEXT },
     { N_("Delete"),             "D",    N_("Delete parts"),
@@ -82,7 +82,7 @@ bst_track_roll_controller_quant_actions (BstTrackRollController *self)
       BST_QUANTIZE_TACT,      BST_STOCK_QTACT, },
     { N_("Q: None"),          "<ctrl>0",      N_("No quantization selected"),
       BST_QUANTIZE_NONE,      BST_STOCK_QNOTE_NONE, },
-    { N_("Q: 1\\/1"),         "<ctrl>1",      N_("Quantize to full note boundaries"),
+    { N_("Q: 1\\/1"),         "<ctrl>1",      N_("Quantize to whole note boundaries"),
       BST_QUANTIZE_NOTE_1,    BST_STOCK_QNOTE_1, },
     { N_("Q: 1\\/2"),         "<ctrl>2",      N_("Quantize to half note boundaries"),
       BST_QUANTIZE_NOTE_2,    BST_STOCK_QNOTE_2, },
@@ -92,6 +92,12 @@ bst_track_roll_controller_quant_actions (BstTrackRollController *self)
       BST_QUANTIZE_NOTE_8,    BST_STOCK_QNOTE_8, },
     { N_("Q: 1\\/16"),        "<ctrl>6",      N_("Quantize to sixteenth note boundaries"),
       BST_QUANTIZE_NOTE_16,   BST_STOCK_QNOTE_16, },
+    { N_("Q: 1\\/32"),        "<ctrl>3",      N_("Quantize to thirty-second note boundaries"),
+      BST_QUANTIZE_NOTE_32,   BST_STOCK_QNOTE_32, },
+    { N_("Q: 1\\/64"),        "<ctrl>5",      N_("Quantize to sixty-fourth note boundaries"),
+      BST_QUANTIZE_NOTE_64,   BST_STOCK_QNOTE_64, },
+    { N_("Q: 1\\/128"),       "<ctrl>7",      N_("Quantize to hundred twenty-eighth note boundaries"),
+      BST_QUANTIZE_NOTE_128,  BST_STOCK_QNOTE_128, },
   };
   gxk_action_list_add_actions (alist, G_N_ELEMENTS (actions), actions,
                                NULL /*i18n_domain*/, NULL /*acheck*/, NULL /*aexec*/, NULL);
@@ -463,7 +469,11 @@ delete_start (BstTrackRollController *self,
 {
   if (self->obj_part)	/* got part to delete */
     {
+      bse_item_group_undo (self->song, "Delete Part");
       bse_track_remove_tick (self->obj_track, self->obj_tick);
+      if (!bse_song_find_any_track_for_part (self->song, self->obj_part))
+        bse_song_remove_part (self->song, self->obj_part);
+      bse_item_ungroup_undo (self->song);
       gxk_status_set (GXK_STATUS_DONE, _("Delete Part"), NULL);
     }
   else
