@@ -24,14 +24,8 @@ G_BEGIN_DECLS
 
 /* --- prototypes --- */
 void	   bst_catch_scripts_and_msgs		(void);
-GtkWidget* bst_user_message_popup		(BseUserMsgType msg_type,
-						 const gchar   *message);
 void	   bst_user_messages_kill		(void);
-void       bst_user_message_log_handler         (const char             *log_domain,
-                                                 unsigned char           level,
-                                                 const SfiLogContext    *lcontext,
-                                                 const char             *message);
-
+void       bst_user_message_log_handler         (SfiLogMessage  *message);
 
 G_END_DECLS
 
