@@ -30,6 +30,7 @@
 #include "bsteventroll.h"
 #include "bstgrowbar.h"
 #include "bstdbmeter.h"
+#include "bstscrollgraph.h"
 #include <fcntl.h>
 #include <errno.h>
 #include <unistd.h>
@@ -125,6 +126,7 @@ _bst_init_radgets (void)
   gxk_radget_define_widget_type (BST_TYPE_DB_BEAM);
   gxk_radget_define_widget_type (BST_TYPE_DB_LABELING);
   gxk_radget_define_widget_type (BST_TYPE_DB_METER);
+  gxk_radget_define_widget_type (BST_TYPE_SCROLLGRAPH);
   gxk_radget_define_widget_type (BST_TYPE_PATTERN_VIEW);
   gxk_radget_define_widget_type (BST_TYPE_ZOOMED_WINDOW);
   text = gxk_zfile_uncompress (BST_RADGETS_STANDARD_SIZE, BST_RADGETS_STANDARD_DATA, G_N_ELEMENTS (BST_RADGETS_STANDARD_DATA));
@@ -189,29 +191,6 @@ bst_stock_register_icon (const gchar    *stock_id,
 }
 
 
-/* --- BSE utilities --- */
-BseErrorType
-bst_project_restore_from_file (SfiProxy         project,
-                               const gchar     *file_name)
-{
-  BseErrorType error = bse_project_restore_from_file (project, file_name);
-  /* regardless of how good the restoration worked, try to
-   * keep the resulting project in a GUI usable state.
-   */
-  BseItemSeq *iseq = bse_container_list_children (project);
-  guint i;
-  for (i = 0; i < iseq->n_items; i++)
-    if (BSE_IS_SONG (iseq->items[i]))
-      {
-        /* fixup orphaned parts */
-        bse_song_ensure_track_links (iseq->items[i]);
-        /* songs always need a master bus */
-        bse_song_ensure_master_bus (iseq->items[i]);
-      }
-  return error;
-}
-
-
 /* --- beast/bse specific extensions --- */
 void
 bst_status_eprintf (BseErrorType error,
@@ -640,7 +619,6 @@ gmask_form (GtkWidget   *parent,
   gmask->parent = g_object_ref (parent);
   gtk_object_sink (GTK_OBJECT (parent));
   gmask->action = action;
-  gpack = CLAMP (gpack, BST_GMASK_FIT, BST_GMASK_CENTER);
   gmask->gpack = gpack;
   
   return action;
@@ -701,6 +679,7 @@ bst_gmask_container_create (guint    border_width,
  * columns up to the prompt,
  @* %BST_GMASK_CENTER - center the action widget within space across all possible
  * columns up to the prompt.
+ @* %BST_GMASK_MULTI_SPAN - span aux2 widget across multiple gmask columns.
  */
 BstGMask*
 bst_gmask_form (GtkWidget   *gmask_container,
@@ -1012,8 +991,8 @@ get_toplevel_and_set_tip (GtkWidget   *widget,
 
 static guint
 table_max_bottom_row (GtkTable *table,
-                      guint     min_col,
-                      guint     max_col)
+                      guint     left_col,
+                      guint     right_col)
 {
   guint max_bottom = 0;
   GList *list;
@@ -1021,8 +1000,8 @@ table_max_bottom_row (GtkTable *table,
   for (list = table->children; list; list = list->next)
     {
       GtkTableChild *child = list->data;
-      
-      if (child->left_attach >= min_col && child->right_attach <= max_col)
+
+      if (child->left_attach < right_col && child->right_attach > left_col)
         max_bottom = MAX (max_bottom, child->bottom_attach);
     }
   return max_bottom;
@@ -1084,8 +1063,9 @@ bst_gmask_pack (BstGMask *mask)
     }
   
   /* pack gmask children, options: GTK_EXPAND, GTK_SHRINK, GTK_FILL */
-  c = 6 * gmask->column;
-  row = table_max_bottom_row (table, c, c + 5);
+  gboolean span_multi_columns = aux2 && gmask->gpack == BST_GMASK_MULTI_SPAN;
+  c = span_multi_columns ? 0 : 6 * gmask->column;
+  row = table_max_bottom_row (table, c, 6 * gmask->column + 6);
   if (prompt)
     {
       gtk_table_attach (table, prompt, c, c + 1, row, row + 1, GTK_FILL, GTK_FILL, 0, 0);
@@ -1119,8 +1099,11 @@ bst_gmask_pack (BstGMask *mask)
     }
   if (aux2)
     {
+      guint left_col = c;
+      if (span_multi_columns)
+        c += 6 * gmask->column;
       gtk_table_attach (table, aux2,
-                        c, c + 1,
+                        left_col, c + 1,
                         row, row + 1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
       if (dummy_aux2)
         aux2 = NULL;
@@ -1316,7 +1299,108 @@ bst_xpm_view_create (const gchar **xpm,
 }
 
 
-/* --- file utils --- */
+/* --- misc utils --- */
+gint
+bst_fft_size_to_int (BstFFTSize fft_size)
+{
+  switch (fft_size)
+    {
+#if 0
+    case BST_FFT_SIZE_1:          return 1;
+#endif
+    case BST_FFT_SIZE_2:          return 2;
+    case BST_FFT_SIZE_4:          return 4;
+    case BST_FFT_SIZE_8:          return 8;
+    case BST_FFT_SIZE_16:         return 16;
+    case BST_FFT_SIZE_32:         return 32;
+    case BST_FFT_SIZE_64:         return 64;
+    case BST_FFT_SIZE_128:        return 128;
+    case BST_FFT_SIZE_256:        return 256;
+    case BST_FFT_SIZE_512:        return 512;
+    case BST_FFT_SIZE_1024:       return 1024;
+    case BST_FFT_SIZE_2048:       return 2048;
+    case BST_FFT_SIZE_4096:       return 4096;
+    case BST_FFT_SIZE_8192:       return 8192;
+    case BST_FFT_SIZE_16384:      return 16384;
+    case BST_FFT_SIZE_32768:      return 32768;
+    case BST_FFT_SIZE_65536:      return 65536;
+#if 0
+    case BST_FFT_SIZE_131072:     return 131072;
+    case BST_FFT_SIZE_262144:     return 262144;
+    case BST_FFT_SIZE_524288:     return 524288;
+    case BST_FFT_SIZE_1048576:    return 1048576;
+    case BST_FFT_SIZE_2097152:    return 2097152;
+    case BST_FFT_SIZE_4194304:    return 4194304;
+    case BST_FFT_SIZE_8388608:    return 8388608;
+    case BST_FFT_SIZE_16777216:   return 16777216;
+    case BST_FFT_SIZE_33554432:   return 33554432;
+    case BST_FFT_SIZE_67108864:   return 67108864;
+    case BST_FFT_SIZE_134217728:  return 134217728;
+    case BST_FFT_SIZE_268435456:  return 268435456;
+    case BST_FFT_SIZE_536870912:  return 536870912;
+    case BST_FFT_SIZE_1073741824: return 1073741824;
+    case BST_FFT_SIZE_2147483648: return 2147483648;
+    case BST_FFT_SIZE_4294967296: return 4294967296;
+#endif
+    default:                      return 0;
+    }
+}
+
+BstFFTSize
+bst_fft_size_from_int (guint sz)
+{
+  const struct { BstFFTSize fft_size; guint sz; } sizes[] = {
+#if 0
+    { BST_FFT_SIZE_1, 1 },
+#endif
+    { BST_FFT_SIZE_2, 2 },
+    { BST_FFT_SIZE_4, 4 },
+    { BST_FFT_SIZE_8, 8 },
+    { BST_FFT_SIZE_16,    16 },
+    { BST_FFT_SIZE_32,    32 },
+    { BST_FFT_SIZE_64,    64 },
+    { BST_FFT_SIZE_128,   128 },
+    { BST_FFT_SIZE_256,   256 },
+    { BST_FFT_SIZE_512,   512 },
+    { BST_FFT_SIZE_1024,  1024 },
+    { BST_FFT_SIZE_2048,  2048 },
+    { BST_FFT_SIZE_4096,  4096 },
+    { BST_FFT_SIZE_8192,  8192 },
+    { BST_FFT_SIZE_16384, 16384 },
+    { BST_FFT_SIZE_32768, 32768 },
+    { BST_FFT_SIZE_65536, 65536 },
+#if 0
+    { BST_FFT_SIZE_131072,     131072 },
+    { BST_FFT_SIZE_262144,     262144 },
+    { BST_FFT_SIZE_524288,     524288 },
+    { BST_FFT_SIZE_1048576,    1048576 },
+    { BST_FFT_SIZE_2097152,    2097152 },
+    { BST_FFT_SIZE_4194304,    4194304 },
+    { BST_FFT_SIZE_8388608,    8388608 },
+    { BST_FFT_SIZE_16777216,   16777216 },
+    { BST_FFT_SIZE_33554432,   33554432 },
+    { BST_FFT_SIZE_67108864,   67108864 },
+    { BST_FFT_SIZE_134217728,  134217728 },
+    { BST_FFT_SIZE_268435456,  268435456 },
+    { BST_FFT_SIZE_536870912,  536870912 },
+    { BST_FFT_SIZE_1073741824, 1073741824 },
+    { BST_FFT_SIZE_2147483648, 2147483648 },
+    { BST_FFT_SIZE_4294967296, 4294967296 },
+#endif
+  };
+  /* find size via bisection */
+  guint offset = 0, n = G_N_ELEMENTS (sizes);
+  while (offset + 1 < n)
+    {
+      guint i = (offset + n) >> 1;
+      if (sz < sizes[i].sz)
+        n = i;
+      else
+        offset = i;
+    }
+  return sizes[offset].fft_size;
+}
+
 #include <sfi/sfistore.h>
 
 gchar*
