@@ -19,18 +19,15 @@
 #ifndef __BST_UTILS_H__
 #define __BST_UTILS_H__
 
-#include <bsw/bsw.h>
+#include "bstbseutils.h"
 #include <gtk/gtk.h>
 #include "bstdefs.h"
 #include "bstcluehunter.h"
+/* generated type IDs, idl types */
+#include "bstgentypes.h"
 
 G_BEGIN_DECLS
 
-/* --- BSE utilities --- */
-BseErrorType   bst_project_restore_from_file  (SfiProxy         project,
-                                               const gchar     *file_name);
-
-
 /* --- GUI utilities --- */
 void           bst_status_eprintf             (BseErrorType     error,
                                                const gchar     *message_fmt,
@@ -100,8 +97,10 @@ void       bst_stock_register_icon      (const gchar  *stock_id,
 #define bst_pixbuf_knob()       gxk_stock_fallback_pixbuf (BST_STOCK_KNOB)
 
 
-/* --- file utils --- */
-gchar*  bst_file_scan_find_key          (const gchar  *file,
+/* --- misc utils --- */
+gint            bst_fft_size_to_int     (BstFFTSize    fft_size);
+BstFFTSize      bst_fft_size_from_int   (guint         sz);
+gchar*          bst_file_scan_find_key  (const gchar  *file,
                                          const gchar  *key,
                                          const gchar  *value_prefix);
 
@@ -116,7 +115,8 @@ typedef enum /*< skip >*/
   BST_GMASK_FILL,
   BST_GMASK_INTERLEAVE, /* stretch */
   BST_GMASK_BIG,
-  BST_GMASK_CENTER
+  BST_GMASK_CENTER,
+  BST_GMASK_MULTI_SPAN
 } BstGMaskPack;
 BstGMask*       bst_gmask_form          (GtkWidget     *gmask_container,
                                          GtkWidget     *action,
@@ -158,6 +158,10 @@ BstGMask*       bst_gmask_quick         (GtkWidget     *gmask_container,
 #define bst_gmask_unref         g_object_unref
 
 
+/* --- object utils --- */
+#define bst_object_class_install_property(oclass, group, property_id, pspec) \
+  g_object_class_install_property (oclass, property_id, sfi_pspec_set_group (pspec, group))
+
 /* --- stock icon aliases --- */
 #define BST_STOCK_QUESTION              GTK_STOCK_DIALOG_QUESTION
 #define BST_STOCK_WARNING               GTK_STOCK_DIALOG_WARNING
@@ -210,8 +214,6 @@ BstGMask*       bst_gmask_quick         (GtkWidget     *gmask_container,
 
 
 /* --- generated includes --- */
-/* type IDs, idl types */
-#include "bstgentypes.h"
 /* marshallers */
 #include "bstmarshal.h"
 /* icon stock IDs */
