@@ -478,7 +478,7 @@ parse_formula (const gchar *c,
     }
   args = g_slist_prepend (args, g_strndup (last, c - last));
   if (!*c)
-    g_printerr ("malformed formula: $(%s", start);
+    g_printerr ("malformed formula: $(%s\n", start);
   else
     c++;        /* ')' */
   if (args)
@@ -946,7 +946,7 @@ gxk_radget_parse (const gchar    *domain_name,
   close (fd);
   if (myerror)
     {
-      g_warning ("GxkRadget: while parsing \"%s\": %s", file_name, myerror->message);
+      g_warning ("GxkRadget: parsing error in \"%s\": %s", file_name, myerror->message);
       g_error_free (myerror);
     }
 }
@@ -971,7 +971,7 @@ gxk_radget_parse_text (const gchar    *domain_name,
   radget_parser (domain, i18n_domain, -1, text, text_len < 0 ? strlen (text) : text_len, error ? error : &myerror);
   if (myerror)
     {
-      g_warning ("GxkRadget: while parsing: %s", myerror->message);
+      g_warning ("GxkRadget: parsing error: %s", myerror->message);
       g_error_free (myerror);
     }
 }
@@ -1797,7 +1797,7 @@ _gxk_init_radget_types (void)
   standard_domain->domain = g_intern_string ("standard");
   g_datalist_set_data (&domains, standard_domain->domain, standard_domain);
   *t++ = GTK_TYPE_WINDOW;       *t++ = GTK_TYPE_ARROW;          *t++ = GTK_TYPE_SCROLLED_WINDOW;
-  *t++ = GTK_TYPE_VIEWPORT;
+  *t++ = GTK_TYPE_VIEWPORT;     *t++ = GTK_TYPE_HRULER;         *t++ = GTK_TYPE_VRULER;
   *t++ = GTK_TYPE_TABLE;        *t++ = GTK_TYPE_FRAME;          *t++ = GTK_TYPE_ALIGNMENT;
   *t++ = GTK_TYPE_NOTEBOOK;     *t++ = GTK_TYPE_BUTTON;         *t++ = GTK_TYPE_MENU_BAR;
   *t++ = GTK_TYPE_TREE_VIEW;    *t++ = GTK_TYPE_LABEL;          *t++ = GTK_TYPE_PROGRESS_BAR;
