@@ -21,7 +21,7 @@
 #include "bseproject.h"
 #include "bsestorage.h"
 #include "bsemain.h"
-#include "gslengine.h"
+#include "bseengine.h"
 #include <stdlib.h>
 #include <string.h>
 
@@ -55,13 +55,13 @@ static BseItem*     bse_container_real_retrieve_child   (BseContainer
 static void         bse_container_prepare               (BseSource              *source);
 static void         bse_container_context_create        (BseSource              *source,
                                                          guint                    context_handle,
-                                                         GslTrans               *trans);
+                                                         BseTrans               *trans);
 static void         bse_container_context_connect       (BseSource              *source,
                                                          guint                   context_handle,
-                                                         GslTrans               *trans);
+                                                         BseTrans               *trans);
 static void         bse_container_context_dismiss       (BseSource              *source,
                                                          guint                   context_handle,
-                                                         GslTrans               *trans);
+                                                         BseTrans               *trans);
 static void         bse_container_reset                 (BseSource              *source);
 static GSList*      container_context_children          (BseContainer           *container);
 static void         container_release_children          (BseContainer           *container);
@@ -210,7 +210,7 @@ bse_container_do_add_item (BseContainer *container,
   
   if (BSE_IS_SOURCE (item) && BSE_SOURCE_PREPARED (container))
     {
-      GslTrans *trans = gsl_trans_open ();
+      BseTrans *trans = bse_trans_open ();
       guint *cids, n, c;
       
       g_return_if_fail (BSE_SOURCE_PREPARED (item) == FALSE);
@@ -224,7 +224,7 @@ bse_container_do_add_item (BseContainer *container,
       for (c = 0; c < n; c++)
         bse_source_connect_context (BSE_SOURCE (item), cids[c], trans);
       g_free (cids);
-      gsl_trans_commit (trans);
+      bse_trans_commit (trans);
     }
 }
 
@@ -256,13 +256,17 @@ bse_container_add_item (BseContainer *container,
       gchar *buffer, *p;
       guint i = 0, l;
       
+      if (!uname)
+        uname = g_object_get_data (container, "BseContainer-base-name");
       if (!uname)
         {
           uname = BSE_OBJECT_TYPE_NAME (item);
-          if (strncmp (uname, "Bse", 3) == 0 && uname[3])
-            uname += 3; /* strip namespace for convenient naming */
+          if (strncmp (uname, "BseContrib", 10) == 0 && uname[10] >= 'A' && uname[10] <= 'Z')
+            uname += 10;                /* strip BseContrib namespace for convenient naming */
+          else if (strncmp (uname, "Bse", 3) == 0 && uname[3] >= 'A' && uname[3] <= 'Z')
+            uname += 3;                 /* strip Bse namespace for convenient naming */
         }
-      
+
       l = strlen (uname);
       buffer = g_new (gchar, l + 12);
       strcpy (buffer, uname);
@@ -274,7 +278,8 @@ bse_container_add_item (BseContainer *container,
       g_object_set (item, "uname", buffer, NULL); /* no undo */
       g_free (buffer);
     }
-  
+  g_object_set_data (container, "BseContainer-base-name", NULL);
+
   BSE_CONTAINER_GET_CLASS (container)->add_item (container, item);
   if (item->parent != NULL)
     g_signal_emit (container, container_signals[SIGNAL_ITEM_ADDED], 0, item);
@@ -288,10 +293,11 @@ bse_container_add_item (BseContainer *container,
 }
 
 gpointer
-bse_container_new_child (BseContainer *container,
-                         GType         child_type,
-                         const gchar  *first_param_name,
-                         ...)
+bse_container_new_child_bname (BseContainer *container,
+                               GType         child_type,
+                               const gchar  *base_name,
+                               const gchar  *first_param_name,
+                               ...)
 {
   gpointer child;
   va_list var_args;
@@ -299,10 +305,13 @@ bse_container_new_child (BseContainer *container,
   g_return_val_if_fail (BSE_IS_CONTAINER (container), NULL);
   g_return_val_if_fail (g_type_is_a (child_type, BSE_TYPE_ITEM), NULL);
   g_return_val_if_fail (!G_TYPE_IS_ABSTRACT (child_type), NULL);
-  
+
+  g_object_set_data_full (container, "BseContainer-base-name", g_strdup (base_name), g_free);
   va_start (var_args, first_param_name);
   child = g_object_new_valist (child_type, first_param_name, var_args);
   va_end (var_args);
+  if (base_name)
+    g_object_set (child, "uname", NULL, NULL); /* no undo */
   bse_container_add_item (container, child);
   g_object_unref (child);
   
@@ -410,7 +419,7 @@ list_items (BseItem *item,
 }
 
 BseItemSeq*
-bse_container_list_items (BseContainer *container)
+bse_container_list_children (BseContainer *container)
 {
   BseItemSeq *iseq;
   
@@ -1110,7 +1119,7 @@ container_context_children (BseContainer *container)
 static void
 bse_container_context_create (BseSource *source,
                               guint      context_handle,
-                              GslTrans  *trans)
+                              BseTrans  *trans)
 {
   BseContainer *container = BSE_CONTAINER (source);
   
@@ -1151,7 +1160,7 @@ forall_context_connect (BseItem *item,
 static void
 bse_container_context_connect (BseSource *source,
                                guint      context_handle,
-                               GslTrans  *trans)
+                               BseTrans  *trans)
 {
   BseContainer *container = BSE_CONTAINER (source);
   
@@ -1192,7 +1201,7 @@ forall_context_dismiss (BseItem *item,
 static void
 bse_container_context_dismiss (BseSource *source,
                                guint      context_handle,
-                               GslTrans  *trans)
+                               BseTrans  *trans)
 {
   BseContainer *container = BSE_CONTAINER (source);
   
