@@ -21,8 +21,6 @@ extern GType BSE_TYPE_ID (BseInterpolType);
 extern GType BSE_TYPE_ID (BseLoopType);
 #define BSE_TYPE_MAGIC_FLAGS	    (BSE_TYPE_ID (BseMagicFlags))
 extern GType BSE_TYPE_ID (BseMagicFlags);
-#define BSE_TYPE_USER_MSG_TYPE	    (BSE_TYPE_ID (BseUserMsgType))
-extern GType BSE_TYPE_ID (BseUserMsgType);
 #define BSE_TYPE_ERROR_TYPE	    (BSE_TYPE_ID (BseErrorType))
 extern GType BSE_TYPE_ID (BseErrorType);
 
@@ -38,10 +36,6 @@ extern GType BSE_TYPE_ID (BseMidiEventType);
 #define BSE_TYPE_PROJECT_STATE	    (BSE_TYPE_ID (BseProjectState))
 extern GType BSE_TYPE_ID (BseProjectState);
 
-/* --- bsessequencer.h --- */
-#define BSE_TYPE_SSEQUENCER_JOB_TYPE	    (BSE_TYPE_ID (BseSSequencerJobType))
-extern GType BSE_TYPE_ID (BseSSequencerJobType);
-
 /* --- bsestandardosc.h --- */
 #define BSE_TYPE_STANDARD_OSC_WAVE_TYPE	    (BSE_TYPE_ID (BseStandardOscWaveType))
 extern GType BSE_TYPE_ID (BseStandardOscWaveType);
@@ -53,12 +47,12 @@ extern GType BSE_TYPE_ID (BseMidiDecoderState);
 /* Generated data ends here */
 
 
-/**
- ** Generated data (by mktypes.pl --externs)
- **/
+/*
+ * Generated data (by mktypes.pl --externs)
+ */
 
-/* --- bsesongbus.c --- */
-extern GType bse_type_builtin_id_BseSongBus;
+/* --- bsebus.c --- */
+extern GType bse_type_builtin_id_BseBus;
 
 /* --- bsebiquadfilter.c --- */
 extern GType bse_type_builtin_id_BseBiquadFilter;
@@ -84,12 +78,18 @@ extern GType bse_type_builtin_id_BseItem;
 /* --- bsejanitor.c --- */
 extern GType bse_type_builtin_id_BseJanitor;
 
+/* --- bseladspa.c --- */
+extern GType bse_type_builtin_id_BseLadspaPlugin;
+
 /* --- bsemidicontroller.c --- */
 extern GType bse_type_builtin_id_BseMidiController;
 
 /* --- bsemididevice.c --- */
 extern GType bse_type_builtin_id_BseMidiDevice;
 
+/* --- bsedevice.c --- */
+extern GType bse_type_builtin_id_BseDevice;
+
 /* --- bsemididevice-null.c --- */
 extern GType bse_type_builtin_id_BseMidiDeviceNULL;
 
@@ -120,12 +120,12 @@ extern GType bse_type_builtin_id_BsePcmDevice;
 /* --- bsepcmdevice-oss.c --- */
 extern GType bse_type_builtin_id_BsePcmDeviceOSS;
 
+/* --- bsepcmdevice-null.c --- */
+extern GType bse_type_builtin_id_BsePcmDeviceNull;
+
 /* --- bseplugin.c --- */
 extern GType bse_type_builtin_id_BsePlugin;
 
-/* --- bseladspa.c --- */
-extern GType bse_type_builtin_id_BseLadspaPlugin;
-
 /* --- bseproject.c --- */
 extern GType bse_type_builtin_id_BseProject;
 
@@ -198,9 +198,9 @@ extern GType bse_type_builtin_id_BsePcmWriter;
 /* --- bsemidiinput.c --- */
 extern GType bse_type_builtin_id_BseMidiInput;
 
-/**
- ** Generated data ends here
- **/
+/*
+ * Generated data ends here
+ */
 
 /*-------- begin ../sfi/sfidl generated code --------*/
 
@@ -383,8 +383,177 @@ typedef enum {
   BSE_MIDI_SIGNAL_CONTROL_126 = 254,
   BSE_MIDI_SIGNAL_CONTROL_127 = 255,
 } BseMidiSignalType;
-const gchar* bse_midi_signal_type_to_choice (BseMidiSignalType value);
-BseMidiSignalType bse_midi_signal_type_from_choice (const gchar *choice);
+
+typedef enum {
+  BSE_MIDI_CONTROL_NONE = 0,
+  BSE_MIDI_CONTROL_CONTINUOUS_0 = 64,
+  BSE_MIDI_CONTROL_CONTINUOUS_1 = 65,
+  BSE_MIDI_CONTROL_CONTINUOUS_2 = 66,
+  BSE_MIDI_CONTROL_CONTINUOUS_3 = 67,
+  BSE_MIDI_CONTROL_CONTINUOUS_4 = 68,
+  BSE_MIDI_CONTROL_CONTINUOUS_5 = 69,
+  BSE_MIDI_CONTROL_CONTINUOUS_6 = 70,
+  BSE_MIDI_CONTROL_CONTINUOUS_7 = 71,
+  BSE_MIDI_CONTROL_CONTINUOUS_8 = 72,
+  BSE_MIDI_CONTROL_CONTINUOUS_9 = 73,
+  BSE_MIDI_CONTROL_CONTINUOUS_10 = 74,
+  BSE_MIDI_CONTROL_CONTINUOUS_11 = 75,
+  BSE_MIDI_CONTROL_CONTINUOUS_12 = 76,
+  BSE_MIDI_CONTROL_CONTINUOUS_13 = 77,
+  BSE_MIDI_CONTROL_CONTINUOUS_14 = 78,
+  BSE_MIDI_CONTROL_CONTINUOUS_15 = 79,
+  BSE_MIDI_CONTROL_CONTINUOUS_16 = 80,
+  BSE_MIDI_CONTROL_CONTINUOUS_17 = 81,
+  BSE_MIDI_CONTROL_CONTINUOUS_18 = 82,
+  BSE_MIDI_CONTROL_CONTINUOUS_19 = 83,
+  BSE_MIDI_CONTROL_CONTINUOUS_20 = 84,
+  BSE_MIDI_CONTROL_CONTINUOUS_21 = 85,
+  BSE_MIDI_CONTROL_CONTINUOUS_22 = 86,
+  BSE_MIDI_CONTROL_CONTINUOUS_23 = 87,
+  BSE_MIDI_CONTROL_CONTINUOUS_24 = 88,
+  BSE_MIDI_CONTROL_CONTINUOUS_25 = 89,
+  BSE_MIDI_CONTROL_CONTINUOUS_26 = 90,
+  BSE_MIDI_CONTROL_CONTINUOUS_27 = 91,
+  BSE_MIDI_CONTROL_CONTINUOUS_28 = 92,
+  BSE_MIDI_CONTROL_CONTINUOUS_29 = 93,
+  BSE_MIDI_CONTROL_CONTINUOUS_30 = 94,
+  BSE_MIDI_CONTROL_CONTINUOUS_31 = 95,
+  BSE_MIDI_CONTROL_0 = 128,
+  BSE_MIDI_CONTROL_1 = 129,
+  BSE_MIDI_CONTROL_2 = 130,
+  BSE_MIDI_CONTROL_3 = 131,
+  BSE_MIDI_CONTROL_4 = 132,
+  BSE_MIDI_CONTROL_5 = 133,
+  BSE_MIDI_CONTROL_6 = 134,
+  BSE_MIDI_CONTROL_7 = 135,
+  BSE_MIDI_CONTROL_8 = 136,
+  BSE_MIDI_CONTROL_9 = 137,
+  BSE_MIDI_CONTROL_10 = 138,
+  BSE_MIDI_CONTROL_11 = 139,
+  BSE_MIDI_CONTROL_12 = 140,
+  BSE_MIDI_CONTROL_13 = 141,
+  BSE_MIDI_CONTROL_14 = 142,
+  BSE_MIDI_CONTROL_15 = 143,
+  BSE_MIDI_CONTROL_16 = 144,
+  BSE_MIDI_CONTROL_17 = 145,
+  BSE_MIDI_CONTROL_18 = 146,
+  BSE_MIDI_CONTROL_19 = 147,
+  BSE_MIDI_CONTROL_20 = 148,
+  BSE_MIDI_CONTROL_21 = 149,
+  BSE_MIDI_CONTROL_22 = 150,
+  BSE_MIDI_CONTROL_23 = 151,
+  BSE_MIDI_CONTROL_24 = 152,
+  BSE_MIDI_CONTROL_25 = 153,
+  BSE_MIDI_CONTROL_26 = 154,
+  BSE_MIDI_CONTROL_27 = 155,
+  BSE_MIDI_CONTROL_28 = 156,
+  BSE_MIDI_CONTROL_29 = 157,
+  BSE_MIDI_CONTROL_30 = 158,
+  BSE_MIDI_CONTROL_31 = 159,
+  BSE_MIDI_CONTROL_32 = 160,
+  BSE_MIDI_CONTROL_33 = 161,
+  BSE_MIDI_CONTROL_34 = 162,
+  BSE_MIDI_CONTROL_35 = 163,
+  BSE_MIDI_CONTROL_36 = 164,
+  BSE_MIDI_CONTROL_37 = 165,
+  BSE_MIDI_CONTROL_38 = 166,
+  BSE_MIDI_CONTROL_39 = 167,
+  BSE_MIDI_CONTROL_40 = 168,
+  BSE_MIDI_CONTROL_41 = 169,
+  BSE_MIDI_CONTROL_42 = 170,
+  BSE_MIDI_CONTROL_43 = 171,
+  BSE_MIDI_CONTROL_44 = 172,
+  BSE_MIDI_CONTROL_45 = 173,
+  BSE_MIDI_CONTROL_46 = 174,
+  BSE_MIDI_CONTROL_47 = 175,
+  BSE_MIDI_CONTROL_48 = 176,
+  BSE_MIDI_CONTROL_49 = 177,
+  BSE_MIDI_CONTROL_50 = 178,
+  BSE_MIDI_CONTROL_51 = 179,
+  BSE_MIDI_CONTROL_52 = 180,
+  BSE_MIDI_CONTROL_53 = 181,
+  BSE_MIDI_CONTROL_54 = 182,
+  BSE_MIDI_CONTROL_55 = 183,
+  BSE_MIDI_CONTROL_56 = 184,
+  BSE_MIDI_CONTROL_57 = 185,
+  BSE_MIDI_CONTROL_58 = 186,
+  BSE_MIDI_CONTROL_59 = 187,
+  BSE_MIDI_CONTROL_60 = 188,
+  BSE_MIDI_CONTROL_61 = 189,
+  BSE_MIDI_CONTROL_62 = 190,
+  BSE_MIDI_CONTROL_63 = 191,
+  BSE_MIDI_CONTROL_64 = 192,
+  BSE_MIDI_CONTROL_65 = 193,
+  BSE_MIDI_CONTROL_66 = 194,
+  BSE_MIDI_CONTROL_67 = 195,
+  BSE_MIDI_CONTROL_68 = 196,
+  BSE_MIDI_CONTROL_69 = 197,
+  BSE_MIDI_CONTROL_70 = 198,
+  BSE_MIDI_CONTROL_71 = 199,
+  BSE_MIDI_CONTROL_72 = 200,
+  BSE_MIDI_CONTROL_73 = 201,
+  BSE_MIDI_CONTROL_74 = 202,
+  BSE_MIDI_CONTROL_75 = 203,
+  BSE_MIDI_CONTROL_76 = 204,
+  BSE_MIDI_CONTROL_77 = 205,
+  BSE_MIDI_CONTROL_78 = 206,
+  BSE_MIDI_CONTROL_79 = 207,
+  BSE_MIDI_CONTROL_80 = 208,
+  BSE_MIDI_CONTROL_81 = 209,
+  BSE_MIDI_CONTROL_82 = 210,
+  BSE_MIDI_CONTROL_83 = 211,
+  BSE_MIDI_CONTROL_84 = 212,
+  BSE_MIDI_CONTROL_85 = 213,
+  BSE_MIDI_CONTROL_86 = 214,
+  BSE_MIDI_CONTROL_87 = 215,
+  BSE_MIDI_CONTROL_88 = 216,
+  BSE_MIDI_CONTROL_89 = 217,
+  BSE_MIDI_CONTROL_90 = 218,
+  BSE_MIDI_CONTROL_91 = 219,
+  BSE_MIDI_CONTROL_92 = 220,
+  BSE_MIDI_CONTROL_93 = 221,
+  BSE_MIDI_CONTROL_94 = 222,
+  BSE_MIDI_CONTROL_95 = 223,
+  BSE_MIDI_CONTROL_96 = 224,
+  BSE_MIDI_CONTROL_97 = 225,
+  BSE_MIDI_CONTROL_98 = 226,
+  BSE_MIDI_CONTROL_99 = 227,
+  BSE_MIDI_CONTROL_100 = 228,
+  BSE_MIDI_CONTROL_101 = 229,
+  BSE_MIDI_CONTROL_102 = 230,
+  BSE_MIDI_CONTROL_103 = 231,
+  BSE_MIDI_CONTROL_104 = 232,
+  BSE_MIDI_CONTROL_105 = 233,
+  BSE_MIDI_CONTROL_106 = 234,
+  BSE_MIDI_CONTROL_107 = 235,
+  BSE_MIDI_CONTROL_108 = 236,
+  BSE_MIDI_CONTROL_109 = 237,
+  BSE_MIDI_CONTROL_110 = 238,
+  BSE_MIDI_CONTROL_111 = 239,
+  BSE_MIDI_CONTROL_112 = 240,
+  BSE_MIDI_CONTROL_113 = 241,
+  BSE_MIDI_CONTROL_114 = 242,
+  BSE_MIDI_CONTROL_115 = 243,
+  BSE_MIDI_CONTROL_116 = 244,
+  BSE_MIDI_CONTROL_117 = 245,
+  BSE_MIDI_CONTROL_118 = 246,
+  BSE_MIDI_CONTROL_119 = 247,
+  BSE_MIDI_CONTROL_120 = 248,
+  BSE_MIDI_CONTROL_121 = 249,
+  BSE_MIDI_CONTROL_122 = 250,
+  BSE_MIDI_CONTROL_123 = 251,
+  BSE_MIDI_CONTROL_124 = 252,
+  BSE_MIDI_CONTROL_125 = 253,
+  BSE_MIDI_CONTROL_126 = 254,
+  BSE_MIDI_CONTROL_127 = 255,
+} BseMidiControlType;
+
+typedef enum {
+  BSE_USER_MSG_MISC = 0,
+  BSE_USER_MSG_INFO = 1,
+  BSE_USER_MSG_WARNING = 2,
+  BSE_USER_MSG_ERROR = 3,
+} BseUserMsgType;
 
 typedef enum {
   BSE_THREAD_STATE_UNKNOWN = 0,
@@ -396,11 +565,10 @@ typedef enum {
   BSE_THREAD_STATE_ZOMBIE = 6,
   BSE_THREAD_STATE_DEAD = 7,
 } BseThreadState;
-const gchar* bse_thread_state_to_choice (BseThreadState value);
-BseThreadState bse_thread_state_from_choice (const gchar *choice);
 
 
 /* record typedefs */
+typedef struct _BsePropertyCandidates BsePropertyCandidates;
 typedef struct _BsePartNote BsePartNote;
 typedef struct _BsePartControl BsePartControl;
 typedef struct _BseNoteDescription BseNoteDescription;
@@ -411,6 +579,7 @@ typedef struct _BseDot BseDot;
 typedef struct _BseTrackPart BseTrackPart;
 typedef struct _BseSongTiming BseSongTiming;
 typedef struct _BseSampleFileInfo BseSampleFileInfo;
+typedef struct _BseUserMsg BseUserMsg;
 typedef struct _BseThreadInfo BseThreadInfo;
 typedef struct _BseThreadTotals BseThreadTotals;
 typedef struct _BseGConfig BseGConfig;
@@ -420,6 +589,7 @@ typedef struct _BseGConfig BseGConfig;
 typedef struct _BseIntSeq BseIntSeq;
 typedef struct _BseNoteSeq BseNoteSeq;
 typedef struct _BseStringSeq BseStringSeq;
+typedef struct _BseTypeSeq BseTypeSeq;
 typedef struct _BseItemSeq BseItemSeq;
 typedef struct _BsePartNoteSeq BsePartNoteSeq;
 typedef struct _BsePartControlSeq BsePartControlSeq;
@@ -442,9 +612,13 @@ struct _BseStringSeq {
   guint n_strings;
   gchar** strings;
 };
+struct _BseTypeSeq {
+  guint n_types;
+  gchar** types;
+};
 struct _BseItemSeq {
   guint n_items;
-  SfiProxy* items;
+  BseItem** items;
 };
 struct _BsePartNoteSeq {
   guint n_pnotes;
@@ -473,6 +647,12 @@ struct _BseThreadInfoSeq {
 
 
 /* records */
+struct _BsePropertyCandidates {
+  gchar* label;
+  gchar* tooltip;
+  BseItemSeq* items;
+  BseTypeSeq* partitions;
+};
 struct _BsePartNote {
   SfiInt id;
   SfiInt channel;
@@ -545,9 +725,18 @@ struct _BseSampleFileInfo {
   BseStringSeq* waves;
   SfiInt error;
 };
+struct _BseUserMsg {
+  gchar* log_domain;
+  BseUserMsgType msg_type;
+  gchar* config_blurb;
+  gchar* message;
+  SfiInt pid;
+  gchar* process;
+};
 struct _BseThreadInfo {
   gchar* name;
   BseThreadState state;
+  SfiInt thread_id;
   SfiInt priority;
   SfiInt processor;
   SfiInt utime;
@@ -577,6 +766,13 @@ struct _BseGConfig {
 
 
 /* record functions */
+SfiRecFields bse_property_candidates_get_fields (void);
+BsePropertyCandidates* bse_property_candidates_new (void);
+BsePropertyCandidates* bse_property_candidates_copy_shallow (BsePropertyCandidates* rec);
+BsePropertyCandidates* bse_property_candidates_from_rec (SfiRec *sfi_rec);
+SfiRec *bse_property_candidates_to_rec (BsePropertyCandidates* rec);
+void bse_property_candidates_free (BsePropertyCandidates* rec);
+
 SfiRecFields bse_part_note_get_fields (void);
 BsePartNote* bse_part_note_new (void);
 BsePartNote* bse_part_note_copy_shallow (BsePartNote* rec);
@@ -647,6 +843,13 @@ BseSampleFileInfo* bse_sample_file_info_from_rec (SfiRec *sfi_rec);
 SfiRec *bse_sample_file_info_to_rec (BseSampleFileInfo* rec);
 void bse_sample_file_info_free (BseSampleFileInfo* rec);
 
+SfiRecFields bse_user_msg_get_fields (void);
+BseUserMsg* bse_user_msg_new (void);
+BseUserMsg* bse_user_msg_copy_shallow (BseUserMsg* rec);
+BseUserMsg* bse_user_msg_from_rec (SfiRec *sfi_rec);
+SfiRec *bse_user_msg_to_rec (BseUserMsg* rec);
+void bse_user_msg_free (BseUserMsg* rec);
+
 SfiRecFields bse_thread_info_get_fields (void);
 BseThreadInfo* bse_thread_info_new (void);
 BseThreadInfo* bse_thread_info_copy_shallow (BseThreadInfo* rec);
@@ -698,6 +901,15 @@ SfiSeq *bse_string_seq_to_seq (BseStringSeq* seq);
 void bse_string_seq_resize (BseStringSeq* seq, guint new_size);
 void bse_string_seq_free (BseStringSeq* seq);
 
+GParamSpec* bse_type_seq_get_element (void);
+BseTypeSeq* bse_type_seq_new (void);
+void bse_type_seq_append (BseTypeSeq* seq, const char* element);
+BseTypeSeq* bse_type_seq_copy_shallow (BseTypeSeq* seq);
+BseTypeSeq* bse_type_seq_from_seq (SfiSeq *sfi_seq);
+SfiSeq *bse_type_seq_to_seq (BseTypeSeq* seq);
+void bse_type_seq_resize (BseTypeSeq* seq, guint new_size);
+void bse_type_seq_free (BseTypeSeq* seq);
+
 GParamSpec* bse_item_seq_get_element (void);
 BseItemSeq* bse_item_seq_new (void);
 void bse_item_seq_append (BseItemSeq* seq, BseItem* element);
@@ -765,15 +977,20 @@ void bse_thread_info_seq_free (BseThreadInfoSeq* seq);
 
 /* enum functions */
 const SfiChoiceValues bse_midi_signal_type_get_values (void);
+const SfiChoiceValues bse_midi_control_type_get_values (void);
+const SfiChoiceValues bse_user_msg_type_get_values (void);
 const SfiChoiceValues bse_thread_state_get_values (void);
 
 
 /* enum type ids */
 extern GType bse__type_id__midi_signal_type;
+extern GType bse__type_id__midi_control_type;
+extern GType bse__type_id__user_msg_type;
 extern GType bse__type_id__thread_state;
 
 
 /* record type ids */
+extern GType bse__type_id__property_candidates;
 extern GType bse__type_id__part_note;
 extern GType bse__type_id__part_control;
 extern GType bse__type_id__note_description;
@@ -784,6 +1001,7 @@ extern GType bse__type_id__dot;
 extern GType bse__type_id__track_part;
 extern GType bse__type_id__song_timing;
 extern GType bse__type_id__sample_file_info;
+extern GType bse__type_id__user_msg;
 extern GType bse__type_id__thread_info;
 extern GType bse__type_id__thread_totals;
 extern GType bse__type_id__gconfig;
@@ -793,6 +1011,7 @@ extern GType bse__type_id__gconfig;
 extern GType bse__type_id__int_seq;
 extern GType bse__type_id__note_seq;
 extern GType bse__type_id__string_seq;
+extern GType bse__type_id__type_seq;
 extern GType bse__type_id__item_seq;
 extern GType bse__type_id__part_note_seq;
 extern GType bse__type_id__part_control_seq;
@@ -806,10 +1025,13 @@ extern GType bse__type_id__thread_info_seq;
 
 /* enum type macros */
 #define BSE_TYPE_MIDI_SIGNAL_TYPE		(bse__type_id__midi_signal_type)
+#define BSE_TYPE_MIDI_CONTROL_TYPE		(bse__type_id__midi_control_type)
+#define BSE_TYPE_USER_MSG_TYPE		(bse__type_id__user_msg_type)
 #define BSE_TYPE_THREAD_STATE		(bse__type_id__thread_state)
 
 
 /* record type macros */
+#define BSE_TYPE_PROPERTY_CANDIDATES		(bse__type_id__property_candidates)
 #define BSE_TYPE_PART_NOTE		(bse__type_id__part_note)
 #define BSE_TYPE_PART_CONTROL		(bse__type_id__part_control)
 #define BSE_TYPE_NOTE_DESCRIPTION		(bse__type_id__note_description)
@@ -820,6 +1042,7 @@ extern GType bse__type_id__thread_info_seq;
 #define BSE_TYPE_TRACK_PART		(bse__type_id__track_part)
 #define BSE_TYPE_SONG_TIMING		(bse__type_id__song_timing)
 #define BSE_TYPE_SAMPLE_FILE_INFO		(bse__type_id__sample_file_info)
+#define BSE_TYPE_USER_MSG		(bse__type_id__user_msg)
 #define BSE_TYPE_THREAD_INFO		(bse__type_id__thread_info)
 #define BSE_TYPE_THREAD_TOTALS		(bse__type_id__thread_totals)
 #define BSE_TYPE_GCONFIG		(bse__type_id__gconfig)
@@ -829,6 +1052,7 @@ extern GType bse__type_id__thread_info_seq;
 #define BSE_TYPE_INT_SEQ		(bse__type_id__int_seq)
 #define BSE_TYPE_NOTE_SEQ		(bse__type_id__note_seq)
 #define BSE_TYPE_STRING_SEQ		(bse__type_id__string_seq)
+#define BSE_TYPE_TYPE_SEQ		(bse__type_id__type_seq)
 #define BSE_TYPE_ITEM_SEQ		(bse__type_id__item_seq)
 #define BSE_TYPE_PART_NOTE_SEQ		(bse__type_id__part_note_seq)
 #define BSE_TYPE_PART_CONTROL_SEQ		(bse__type_id__part_control_seq)
