@@ -25,12 +25,7 @@
 #include	<bse/bsenote.h>
 #include	<bse/bseconstvalues.h>
 
-
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
-
+G_BEGIN_DECLS
 
 /* --- signal ranges --- */
 /* min..max sample value: -1.0 .. 1.0
@@ -64,16 +59,8 @@ glong	bse_time_range_to_ms		(BseTimeRangeType	time_range);
 #define BSE_DFL_BIN_DATA_BITS		(16)	// FIXME
 
 
-/* --- hard defaults (used for saving) --- */
-#define	BSE_DFL_MASTER_VOLUME_dB	(0.0)
-#define	BSE_DFL_INSTRUMENT_VOLUME_dB	(- 0.969)
-#define BSE_DFL_INSTRUMENT_BALANCE	(0)
-#define BSE_DFL_INSTRUMENT_TRANSPOSE	(0)
-#define BSE_DFL_INSTRUMENT_FINE_TUNE	(0)
-
-
 /* --- Convenience --- */
-#define	BSE_MIX_FREQ	(gsl_engine_sample_freq ())
+#define	BSE_MIX_FREQ	(bse_engine_sample_freq ())
 #define	BSE_MIX_FREQ_f	((gfloat) BSE_MIX_FREQ)
 #define	BSE_MIX_FREQ_d	((gdouble) BSE_MIX_FREQ)
 
@@ -133,17 +120,13 @@ extern const gdouble* _bse_fine_tune_factor_table;
 /* --- prototypes --- */
 void		bse_globals_init	(void);
 
-/* conversion */
-gdouble		bse_dB_to_factor	(gfloat		dB);
-gfloat		bse_dB_from_factor	(gdouble	factor,
-					 gfloat		min_dB);
+/* --- decibel conversion --- */
+gdouble	bse_db_to_factor	(gdouble	dB);
+gdouble	bse_db_from_factor	(gdouble	factor,
+                                 gdouble	min_dB);
+#define	BSE_MINDB               (-96)   /* 32bit:-192 24bit:-144 16bit:-96 */
 
 
-
-
-
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* __BSE_GLOBALS_H__ */
