@@ -17,6 +17,7 @@
  */
 #include "bseitem.h"
 #include "bsesuper.h"
+#include "bsesnet.h"
 #include "bsestorage.h"
 #include "bseprocedure.h"
 #include "bsemain.h"
@@ -48,9 +49,6 @@ static void             bse_item_get_property_internal  (GObject
 static void             bse_item_update_state           (BseItem                *self);
 static gboolean         bse_item_real_needs_storage     (BseItem                *self,
                                                          BseStorage             *storage);
-static BseItemSeq*      bse_item_list_no_items          (BseItem                *item,
-                                                         guint                   param_id,
-                                                         GParamSpec             *pspec);
 static void             bse_item_do_dispose             (GObject                *object);
 static void             bse_item_do_finalize            (GObject                *object);
 static void             bse_item_do_set_uname           (BseObject              *object,
@@ -94,7 +92,7 @@ BSE_BUILTIN_TYPE (BseItem)
 static void
 bse_item_class_init_base (BseItemClass *class)
 {
-  class->list_items = bse_item_list_no_items;
+  class->get_candidates = NULL;
 }
 
 static void
@@ -197,14 +195,6 @@ bse_item_do_finalize (GObject *object)
   g_return_if_fail (item->use_count == 0);
 }
 
-static BseItemSeq*
-bse_item_list_no_items (BseItem    *item,
-                        guint       param_id,
-                        GParamSpec *pspec)
-{
-  return bse_item_seq_new ();
-}
-
 static void
 bse_item_do_set_uname (BseObject   *object,
                        const gchar *uname)
@@ -404,12 +394,12 @@ gather_typed_acheck (BseItem  *proxy,
  * @items:          sequence of items to append to
  * @proxy_type:     base type of the items to gather
  * @container_type: base type of the containers to check for items
- * @allow_ancestor: if %FALSE, direct ancestors of @item are omitted
+ * @allow_ancestor: if %FALSE, ancestors of @item are omitted
  * @RETURNS:   returns @items
  *
  * Variant of bse_item_gather_items(), the containers and items
  * are simply filtered by checking derivation from @container_type
- * and @proxy_type respectively. Items may not be direct ancestors
+ * and @proxy_type respectively. Gathered items may not be ancestors
  * of @item if @allow_ancestor is %FALSE.
  */
 BseItemSeq*
@@ -426,21 +416,29 @@ bse_item_gather_items_typed (BseItem              *item,
                                   gather_typed_ccheck, gather_typed_acheck, (gpointer) container_type);
 }
 
-BseItemSeq*
-bse_item_list_items (BseItem     *item,
-                     const gchar *property)
+gboolean
+bse_item_get_candidates (BseItem                *item,
+                         const gchar            *property,
+                         BsePropertyCandidates  *pc)
 {
   BseItemClass *class;
   GParamSpec *pspec;
   
-  g_return_val_if_fail (BSE_IS_ITEM (item), NULL);
-  g_return_val_if_fail (property != NULL, NULL);
-  
+  g_return_val_if_fail (BSE_IS_ITEM (item), FALSE);
+  g_return_val_if_fail (property != NULL, FALSE);
+  g_return_val_if_fail (pc != NULL, FALSE);
+
   pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (item), property);
   if (!pspec)
-    return NULL;
+    return FALSE;
+  if (!pc->items)
+    pc->items = bse_item_seq_new();
+  if (!pc->partitions)
+    pc->partitions = bse_type_seq_new();
   class = g_type_class_peek (pspec->owner_type);
-  return class->list_items (item, pspec->param_id, pspec);
+  if (class && class->get_candidates)
+    class->get_candidates (item, pspec->param_id, pc, pspec);
+  return TRUE;
 }
 
 BseItem*
@@ -667,6 +665,17 @@ bse_item_get_super (BseItem *item)
   return item ? BSE_SUPER (item) : NULL;
 }
 
+BseSNet*
+bse_item_get_snet (BseItem *item)
+{
+  g_return_val_if_fail (BSE_IS_ITEM (item), NULL);
+  
+  while (!BSE_IS_SNET (item) && item)
+    item = item->parent;
+  
+  return item ? BSE_SNET (item) : NULL;
+}
+
 BseItem*
 bse_item_get_toplevel (BseItem *item)
 {
