@@ -17,7 +17,7 @@
  */
 #include "bsemididecoder.h"
 #include "bsemidireceiver.h"
-#include "gslengine.h"
+#include "bseengine.h"
 #include <string.h>
 
 #define DEBUG(...)      sfi_debug ("midi-decoder", __VA_ARGS__)
@@ -313,7 +313,7 @@ bse_midi_decoder_push_data (BseMidiDecoder *self,
   if (n_bytes)
     g_return_if_fail (bytes != NULL);
   
-  data.delta_time = gsl_engine_tick_stamp_from_systime (usec_systime);
+  data.delta_time = bse_engine_tick_stamp_from_systime (usec_systime);
   data.bytes = bytes;
   data.bound = bytes + n_bytes;
   while (data.bytes < data.bound || self->state_changed)
