@@ -22,64 +22,15 @@
 #include <errno.h>
 
 
-/* --- prototypes --- */
-static void	bse_midi_device_init		(BseMidiDevice		*self);
-static void	bse_midi_device_class_init	(BseMidiDeviceClass	*class);
-static void	bse_midi_device_dispose		(GObject		*object);
-static void	bse_midi_device_finalize	(GObject		*object);
-
-
 /* --- variables --- */
 static gpointer parent_class = NULL;
 
 
 /* --- functions --- */
-BSE_BUILTIN_TYPE (BseMidiDevice)
-{
-  static const GTypeInfo midi_device_info = {
-    sizeof (BseMidiDeviceClass),
-    
-    (GBaseInitFunc) NULL,
-    (GBaseFinalizeFunc) NULL,
-    (GClassInitFunc) bse_midi_device_class_init,
-    (GClassFinalizeFunc) NULL,
-    NULL /* class_data */,
-    
-    sizeof (BseMidiDevice),
-    0 /* n_preallocs */,
-    (GInstanceInitFunc) bse_midi_device_init,
-  };
-  
-  g_assert (BSE_MIDI_FLAGS_USHIFT < BSE_OBJECT_FLAGS_MAX_SHIFT);
-  
-  return bse_type_register_abstract (BSE_TYPE_OBJECT,
-                                     "BseMidiDevice",
-                                     "MIDI device base type",
-                                     &midi_device_info);
-}
-
-static void
-bse_midi_device_class_init (BseMidiDeviceClass *class)
-{
-  GObjectClass *gobject_class = G_OBJECT_CLASS (class);
-  
-  parent_class = g_type_class_peek_parent (class);
-  
-  gobject_class->dispose = bse_midi_device_dispose;
-  gobject_class->finalize = bse_midi_device_finalize;
-  
-  class->driver_rating = 0;
-  class->open = NULL;
-  class->suspend = NULL;
-}
-
 static void
 bse_midi_device_init (BseMidiDevice *self)
 {
-  BSE_OBJECT_UNSET_FLAGS (self, (BSE_MIDI_FLAG_OPEN |
-				 BSE_MIDI_FLAG_READABLE |
-				 BSE_MIDI_FLAG_WRITABLE));
-  self->midi_decoder = bse_midi_decoder_new (TRUE);
+  self->midi_decoder = bse_midi_decoder_new (TRUE, FALSE);
   self->handle = NULL;
 }
 
@@ -88,10 +39,10 @@ bse_midi_device_dispose (GObject *object)
 {
   BseMidiDevice *self = BSE_MIDI_DEVICE (object);
   
-  if (BSE_MIDI_DEVICE_OPEN (self))
+  if (BSE_DEVICE_OPEN (self))
     {
       g_warning ("%s: midi device still opened", G_STRLOC);
-      bse_midi_device_suspend (self);
+      bse_device_close (BSE_DEVICE (self));
     }
   if (self->handle)
     g_warning (G_STRLOC ": midi device with stale midi handle");
@@ -112,36 +63,35 @@ bse_midi_device_finalize (GObject *object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-BseErrorType
-bse_midi_device_open (BseMidiDevice  *mdev)
+static void
+bse_midi_device_class_init (BseMidiDeviceClass *class)
 {
-  BseErrorType error;
+  GObjectClass *gobject_class = G_OBJECT_CLASS (class);
   
-  g_return_val_if_fail (BSE_IS_MIDI_DEVICE (mdev), BSE_ERROR_INTERNAL);
-  g_return_val_if_fail (!BSE_MIDI_DEVICE_OPEN (mdev), BSE_ERROR_INTERNAL);
+  parent_class = g_type_class_peek_parent (class);
   
-  error = BSE_MIDI_DEVICE_GET_CLASS (mdev)->open (mdev);
-  
-  if (!error)
-    g_return_val_if_fail (BSE_MIDI_DEVICE_OPEN (mdev) && mdev->handle, BSE_ERROR_INTERNAL);
-  else
-    g_return_val_if_fail (!BSE_MIDI_DEVICE_OPEN (mdev), BSE_ERROR_INTERNAL);
-  
-  errno = 0;
-  
-  return error;
+  gobject_class->dispose = bse_midi_device_dispose;
+  gobject_class->finalize = bse_midi_device_finalize;
 }
 
-void
-bse_midi_device_suspend (BseMidiDevice *mdev)
+BSE_BUILTIN_TYPE (BseMidiDevice)
 {
-  g_return_if_fail (BSE_IS_MIDI_DEVICE (mdev));
-  g_return_if_fail (BSE_MIDI_DEVICE_OPEN (mdev));
+  static const GTypeInfo midi_device_info = {
+    sizeof (BseMidiDeviceClass),
+    
+    (GBaseInitFunc) NULL,
+    (GBaseFinalizeFunc) NULL,
+    (GClassInitFunc) bse_midi_device_class_init,
+    (GClassFinalizeFunc) NULL,
+    NULL /* class_data */,
+    
+    sizeof (BseMidiDevice),
+    0 /* n_preallocs */,
+    (GInstanceInitFunc) bse_midi_device_init,
+  };
   
-  BSE_MIDI_DEVICE_GET_CLASS (mdev)->suspend (mdev);
-  
-  BSE_OBJECT_UNSET_FLAGS (mdev, (BSE_MIDI_FLAG_OPEN |
-				 BSE_MIDI_FLAG_READABLE |
-				 BSE_MIDI_FLAG_WRITABLE));
-  errno = 0;
+  return bse_type_register_abstract (BSE_TYPE_DEVICE,
+                                     "BseMidiDevice",
+                                     "MIDI device base type",
+                                     &midi_device_info);
 }
