@@ -75,7 +75,12 @@ struct _BseObjectClass
 {
   GObjectClass		 parent_class;
 
+  gboolean              (*editable_property)    (BseObject      *object, /* for set_property/get_property implementations */
+                                                 guint           param_id,
+                                                 GParamSpec     *pspec);
   /* custom methods for specific object needs, most of them require chaining */
+  gboolean              (*check_pspec_editable) (BseObject      *object, /* for containers */
+                                                 GParamSpec     *pspec);
   void			(*set_uname)		(BseObject	*object,
 						 const gchar	*uname);
   void			(*store_private)	(BseObject	*object,
@@ -117,6 +122,8 @@ guint		bse_object_class_add_dsignal	(BseObjectClass	*oclass,
 /* --- object prototypes --- */
 void		bse_object_lock			(gpointer	 object);
 void		bse_object_unlock		(gpointer	 object);
+gboolean        bse_object_editable_property	(gpointer	 object,
+                                                 const gchar    *property);
 BseIcon*	bse_object_get_icon		(BseObject	*object);
 void		bse_object_notify_icon_changed	(BseObject	*object);
 gpointer	bse_object_from_id		(guint		 unique_id);
@@ -125,6 +132,7 @@ GList*		bse_objects_list_by_uname	(GType		 type,
 						 const gchar	*uname);
 void		bse_object_debug_leaks		(void);
 const gchar*	bse_object_debug_name		(gpointer	 object);
+gchar*	        bse_object_strdup_debug_handle 	(gpointer	 object);
 void		bse_object_reemit_signal	(gpointer	 src_object,
 						 const gchar	*src_signal,
 						 gpointer	 dest_obejct,
@@ -153,6 +161,12 @@ bse_object_unproxy_notifies	(gpointer	 src_object,
 
 /* --- implementation details --- */
 extern GQuark bse_quark_uname;
+void          bse_object_marshal_signal (GClosure       *closure,
+                                         GValue /*out*/ *return_value,
+                                         guint           n_param_values,
+                                         const GValue   *param_values,
+                                         gpointer        invocation_hint,
+                                         gpointer        marshal_data);
 
 G_END_DECLS
 
