@@ -15,17 +15,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
-#include	"bseplugin.h"
+#include "bseplugin.h"
 
-#include	"bsecategories.h"
-#include	"bseprocedure.h"
-#include	"bseobject.h"
-#include	"bseenums.h"
-#include	<gmodule.h>
-#include	<string.h>
-#include	<fcntl.h>
-#include	<errno.h>
-#include	<unistd.h>
+#include "bsecategories.h"
+#include "bseprocedure.h"
+#include "bseobject.h"
+#include "bseenums.h"
+#include "bsemain.h"
+#include <gmodule.h>
+#include <string.h>
+#include <fcntl.h>
+#include <errno.h>
+#include <unistd.h>
 
 
 #define DEBUG(...)      sfi_debug ("plugins", __VA_ARGS__)
@@ -294,7 +295,7 @@ bse_plugin_complete_info (GTypePlugin     *gplugin,
             break;
           case BSE_EXPORT_NODE_ENUM:
             enode = (BseExportNodeEnum*) node;
-            g_enum_complete_type_info (type, type_info, enode->values);
+            g_enum_complete_type_info (type, type_info, enode->get_values());
             break;
           case BSE_EXPORT_NODE_RECORD:
           case BSE_EXPORT_NODE_SEQUENCE:
@@ -616,21 +617,40 @@ bse_plugin_lookup (const gchar *name)
 #include "topconfig.h"
 
 SfiRing*
-bse_plugin_path_list_files (void)
+bse_plugin_path_list_files (gboolean include_drivers,
+                            gboolean include_plugins)
 {
-  SfiRing *ring1, *ring2 = NULL, *ring3 = NULL;
-
-  ring1 = sfi_file_crawler_list_files (BSE_PATH_PLUGINS, "*.so", G_FILE_TEST_IS_REGULAR);
-  ring1 = sfi_ring_sort (ring1, (GCompareFunc) strcmp);
-
-  if (BSE_GCONFIG (plugin_path) && BSE_GCONFIG (plugin_path)[0])
-    ring2 = sfi_file_crawler_list_files (BSE_GCONFIG (plugin_path), "*.so", G_FILE_TEST_IS_REGULAR);
-  ring2 = sfi_ring_sort (ring2, (GCompareFunc) strcmp);
-
-  /* allow file names in plugin_path */
-  if (BSE_GCONFIG (plugin_path) && BSE_GCONFIG (plugin_path)[0])
-    ring3 = sfi_file_crawler_list_files (BSE_GCONFIG (plugin_path), NULL, G_FILE_TEST_IS_REGULAR);
-  ring3 = sfi_ring_sort (ring3, (GCompareFunc) strcmp);
-
-  return sfi_ring_concat (ring1, sfi_ring_concat (ring2, ring3));
+  SfiRing *files, *ring = NULL;
+  if (bse_main_args->override_plugin_globs)
+    {
+      /* expect filename globs */
+      files = sfi_file_crawler_list_files (bse_main_args->override_plugin_globs, NULL, G_FILE_TEST_IS_REGULAR);
+      ring = sfi_ring_concat (ring, sfi_ring_sort (files, (SfiCompareFunc) strcmp, NULL));
+      return ring;      /* override */
+    }
+  if (include_drivers)
+    {
+      files = sfi_file_crawler_list_files (BSE_PATH_DRIVERS, "*.so", G_FILE_TEST_IS_REGULAR);
+      ring = sfi_ring_concat (ring, sfi_ring_sort (files, (SfiCompareFunc) strcmp, NULL));
+      files = sfi_file_crawler_list_files (BSE_PATH_DRIVERS, "*.o", G_FILE_TEST_IS_REGULAR);
+      ring = sfi_ring_concat (ring, sfi_ring_sort (files, (SfiCompareFunc) strcmp, NULL));
+    }
+  if (include_plugins)
+    {
+      files = sfi_file_crawler_list_files (BSE_PATH_PLUGINS, "*.so", G_FILE_TEST_IS_REGULAR);
+      ring = sfi_ring_concat (ring, sfi_ring_sort (files, (SfiCompareFunc) strcmp, NULL));
+      files = sfi_file_crawler_list_files (BSE_PATH_PLUGINS, "*.o", G_FILE_TEST_IS_REGULAR);
+      ring = sfi_ring_concat (ring, sfi_ring_sort (files, (SfiCompareFunc) strcmp, NULL));
+    }
+  if (include_plugins && BSE_GCONFIG (plugin_path) && BSE_GCONFIG (plugin_path)[0])
+    {
+      files = sfi_file_crawler_list_files (BSE_GCONFIG (plugin_path), "*.so", G_FILE_TEST_IS_REGULAR);
+      ring = sfi_ring_concat (ring, sfi_ring_sort (files, (SfiCompareFunc) strcmp, NULL));
+      files = sfi_file_crawler_list_files (BSE_GCONFIG (plugin_path), "*.o", G_FILE_TEST_IS_REGULAR);
+      ring = sfi_ring_concat (ring, sfi_ring_sort (files, (SfiCompareFunc) strcmp, NULL));
+      /* allow file names in plugin_path */
+      files = sfi_file_crawler_list_files (BSE_GCONFIG (plugin_path), NULL, G_FILE_TEST_IS_REGULAR);
+      ring = sfi_ring_concat (ring, sfi_ring_sort (files, (SfiCompareFunc) strcmp, NULL));
+    }
+  return ring;
 }
