@@ -1,7 +1,7 @@
 
-/**
- ** Generated data (by mkcproc.pl)
- **/
+/*
+ * Generated data (by mkcproc.pl)
+ */
 #line 1 "bseproject.proc"
 
 #include <bse/bseplugin.h>
@@ -804,7 +804,7 @@ undo_do_deactivate (BseUndoStep  *ustep,
                     BseUndoStack *ustack)
 {
   BseProject *self = bse_undo_pointer_unpack (ustep->data[0].v_pointer, ustack);
-  bse_project_stop_playback (self);
+  bse_project_deactivate (self);
 }
 
 static void
@@ -816,42 +816,53 @@ undo_do_deactivate_free (BseUndoStep *ustep)
 static void
 project_push_undo_silent_deactivate (BseProject *self)
 {
+#line 614 "bseproject.proc"
+  
   BseUndoStack *ustack = bse_item_undo_open (self, "deactivate-project");
   BseUndoStep *ustep = bse_undo_step_new (undo_do_deactivate, undo_do_deactivate_free, 1);
-#line 616 "bseproject.proc"
-  
   ustep->data[0].v_pointer = bse_undo_pointer_pack (self, ustack);
   bse_undo_stack_push_add_on (ustack, ustep);
   bse_item_undo_close (ustack);
+
+  gboolean in_undo = self->in_undo;
+#line 628 "bseproject.proc"
+  self->in_undo = !in_undo;             
+  ustack = bse_item_undo_open (self, "deactivate-project");
+  ustep = bse_undo_step_new (undo_do_deactivate, undo_do_deactivate_free, 1);
+  ustep->data[0].v_pointer = bse_undo_pointer_pack (self, ustack);
+  bse_undo_stack_push_add_on (ustack, ustep);
+  bse_item_undo_close (ustack);
+#line 634 "bseproject.proc"
+  self->in_undo = in_undo;              
 }
 
 /* --- activate --- */
 static void
 activate_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 628 "bseproject.proc"
+#line 637 "bseproject.proc"
  {
-#line 630 "bseproject.proc"
+#line 639 "bseproject.proc"
   *(in_pspecs++)    = bse_param_spec_object ("project", "Project", "The project",
                                  BSE_TYPE_PROJECT, SFI_PARAM_STANDARD);
   *(out_pspecs++)   = bse_param_spec_genum ("error", "Error", NULL,
                                 BSE_TYPE_ERROR_TYPE, BSE_ERROR_NONE,
                                 SFI_PARAM_STANDARD);
-#line 635 "bseproject.proc"
+#line 644 "bseproject.proc"
 }  }
 static BseErrorType
-#line 635 "bseproject.proc"
+#line 644 "bseproject.proc"
 activate_exec (BseProcedureClass *proc,
         const GValue      *in_values,
         GValue            *out_values)
-#line 638 "bseproject.proc"
+#line 647 "bseproject.proc"
 {
-#line 639 "bseproject.proc"
+#line 648 "bseproject.proc"
   
   BseProject *self = bse_value_get_object (in_values++);
   BseProjectState state_before;
   BseErrorType error;
 
-#line 644 "bseproject.proc"
+#line 653 "bseproject.proc"
   
   if (!BSE_IS_PROJECT (self))
     return BSE_ERROR_PROC_PARAM_INVAL;
@@ -860,7 +871,7 @@ activate_exec (BseProcedureClass *proc,
   error = bse_project_activate (self);
   if (state_before == BSE_PROJECT_INACTIVE && self->state != BSE_PROJECT_INACTIVE)
     {
-#line 652 "bseproject.proc"
+#line 661 "bseproject.proc"
       
       project_push_undo_silent_deactivate (self);
     }
@@ -873,26 +884,26 @@ activate_exec (BseProcedureClass *proc,
 /* --- start-playback --- */
 static void
 start_playback_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 661 "bseproject.proc"
+#line 670 "bseproject.proc"
  {
-#line 663 "bseproject.proc"
+#line 672 "bseproject.proc"
   *(in_pspecs++)    = bse_param_spec_object ("project", "Project", "The project",
                                  BSE_TYPE_PROJECT, SFI_PARAM_STANDARD);
-#line 665 "bseproject.proc"
+#line 674 "bseproject.proc"
 }  }
 static BseErrorType
-#line 665 "bseproject.proc"
+#line 674 "bseproject.proc"
 start_playback_exec (BseProcedureClass *proc,
         const GValue      *in_values,
         GValue            *out_values)
-#line 668 "bseproject.proc"
+#line 677 "bseproject.proc"
 {
-#line 669 "bseproject.proc"
+#line 678 "bseproject.proc"
   
   BseProject *self = bse_value_get_object (in_values++);
   BseProjectState state_before;
 
-#line 673 "bseproject.proc"
+#line 682 "bseproject.proc"
   
   if (!BSE_IS_PROJECT (self))
     return BSE_ERROR_PROC_PARAM_INVAL;
@@ -901,7 +912,7 @@ start_playback_exec (BseProcedureClass *proc,
   bse_project_start_playback (self);
   if (state_before == BSE_PROJECT_INACTIVE && self->state != BSE_PROJECT_INACTIVE)
     {
-#line 681 "bseproject.proc"
+#line 690 "bseproject.proc"
       
       project_push_undo_silent_deactivate (self);
     }
@@ -912,30 +923,30 @@ start_playback_exec (BseProcedureClass *proc,
 /* --- play --- */
 static void
 play_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 688 "bseproject.proc"
+#line 697 "bseproject.proc"
  {
-#line 690 "bseproject.proc"
+#line 699 "bseproject.proc"
   *(in_pspecs++)    = bse_param_spec_object ("project", "Project", "The project",
                                  BSE_TYPE_PROJECT, SFI_PARAM_STANDARD);
   *(out_pspecs++)   = bse_param_spec_genum ("error", "Error", NULL,
                                 BSE_TYPE_ERROR_TYPE, BSE_ERROR_NONE,
                                 SFI_PARAM_STANDARD);
-#line 695 "bseproject.proc"
+#line 704 "bseproject.proc"
 }  }
 static BseErrorType
-#line 695 "bseproject.proc"
+#line 704 "bseproject.proc"
 play_exec (BseProcedureClass *proc,
         const GValue      *in_values,
         GValue            *out_values)
-#line 698 "bseproject.proc"
+#line 707 "bseproject.proc"
 {
-#line 699 "bseproject.proc"
+#line 708 "bseproject.proc"
   
   BseProject *self = bse_value_get_object (in_values++);
   BseErrorType error;
   BseProjectState state_before;
   
-#line 704 "bseproject.proc"
+#line 713 "bseproject.proc"
   
   if (!BSE_IS_PROJECT (self))
     return BSE_ERROR_PROC_PARAM_INVAL;
@@ -950,7 +961,7 @@ play_exec (BseProcedureClass *proc,
     }
   if (state_before == BSE_PROJECT_INACTIVE && self->state != BSE_PROJECT_INACTIVE)
     {
-#line 718 "bseproject.proc"
+#line 727 "bseproject.proc"
       
       project_push_undo_silent_deactivate (self);
     }
@@ -963,25 +974,25 @@ play_exec (BseProcedureClass *proc,
 /* --- deactivate --- */
 static void
 deactivate_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 727 "bseproject.proc"
+#line 736 "bseproject.proc"
  {
-#line 729 "bseproject.proc"
+#line 738 "bseproject.proc"
   *(in_pspecs++)    = bse_param_spec_object ("project", "Project", "The project",
                                  BSE_TYPE_PROJECT, SFI_PARAM_STANDARD);
-#line 731 "bseproject.proc"
+#line 740 "bseproject.proc"
 }  }
 static BseErrorType
-#line 731 "bseproject.proc"
+#line 740 "bseproject.proc"
 deactivate_exec (BseProcedureClass *proc,
         const GValue      *in_values,
         GValue            *out_values)
-#line 734 "bseproject.proc"
+#line 743 "bseproject.proc"
 {
-#line 735 "bseproject.proc"
+#line 744 "bseproject.proc"
   
   BseProject *self = bse_value_get_object (in_values++);
 
-#line 738 "bseproject.proc"
+#line 747 "bseproject.proc"
   
   if (!BSE_IS_PROJECT (self))
     return BSE_ERROR_PROC_PARAM_INVAL;
@@ -994,25 +1005,25 @@ deactivate_exec (BseProcedureClass *proc,
 /* --- stop-playback --- */
 static void
 stop_playback_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 747 "bseproject.proc"
+#line 756 "bseproject.proc"
  {
-#line 749 "bseproject.proc"
+#line 758 "bseproject.proc"
   *(in_pspecs++)    = bse_param_spec_object ("project", "Project", "The project",
                                  BSE_TYPE_PROJECT, SFI_PARAM_STANDARD);
-#line 751 "bseproject.proc"
+#line 760 "bseproject.proc"
 }  }
 static BseErrorType
-#line 751 "bseproject.proc"
+#line 760 "bseproject.proc"
 stop_playback_exec (BseProcedureClass *proc,
         const GValue      *in_values,
         GValue            *out_values)
-#line 754 "bseproject.proc"
+#line 763 "bseproject.proc"
 {
-#line 755 "bseproject.proc"
+#line 764 "bseproject.proc"
   
   BseProject *self = bse_value_get_object (in_values++);
 
-#line 758 "bseproject.proc"
+#line 767 "bseproject.proc"
   
   if (!BSE_IS_PROJECT (self))
     return BSE_ERROR_PROC_PARAM_INVAL;
@@ -1025,25 +1036,25 @@ stop_playback_exec (BseProcedureClass *proc,
 /* --- stop --- */
 static void
 stop_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 767 "bseproject.proc"
+#line 776 "bseproject.proc"
  {
-#line 769 "bseproject.proc"
+#line 778 "bseproject.proc"
   *(in_pspecs++)    = bse_param_spec_object ("project", "Project", "The project",
                                  BSE_TYPE_PROJECT, SFI_PARAM_STANDARD);
-#line 771 "bseproject.proc"
+#line 780 "bseproject.proc"
 }  }
 static BseErrorType
-#line 771 "bseproject.proc"
+#line 780 "bseproject.proc"
 stop_exec (BseProcedureClass *proc,
         const GValue      *in_values,
         GValue            *out_values)
-#line 774 "bseproject.proc"
+#line 783 "bseproject.proc"
 {
-#line 775 "bseproject.proc"
+#line 784 "bseproject.proc"
   
   BseProject *self = bse_value_get_object (in_values++);
 
-#line 778 "bseproject.proc"
+#line 787 "bseproject.proc"
   
   if (!BSE_IS_PROJECT (self))
     return BSE_ERROR_PROC_PARAM_INVAL;
@@ -1056,28 +1067,28 @@ stop_exec (BseProcedureClass *proc,
 /* --- auto-deactivate --- */
 static void
 auto_deactivate_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 787 "bseproject.proc"
+#line 796 "bseproject.proc"
  {
-#line 789 "bseproject.proc"
+#line 798 "bseproject.proc"
   *(in_pspecs++)    = bse_param_spec_object ("project", NULL, "The project",
                                  BSE_TYPE_PROJECT, SFI_PARAM_STANDARD);
   *(in_pspecs++)    = sfi_pspec_int ("msec_delay", NULL, "Delay for deactivation in milliseconds",
                          0, -1, G_MAXINT, 1000, SFI_PARAM_STANDARD);
-#line 793 "bseproject.proc"
+#line 802 "bseproject.proc"
 }  }
 static BseErrorType
-#line 793 "bseproject.proc"
+#line 802 "bseproject.proc"
 auto_deactivate_exec (BseProcedureClass *proc,
         const GValue      *in_values,
         GValue            *out_values)
-#line 796 "bseproject.proc"
+#line 805 "bseproject.proc"
 {
-#line 797 "bseproject.proc"
+#line 806 "bseproject.proc"
   
   BseProject *self  = bse_value_get_object (in_values++);
   gint64      msecs = sfi_value_get_int (in_values++);
 
-#line 801 "bseproject.proc"
+#line 810 "bseproject.proc"
   
   if (!BSE_IS_PROJECT (self))
     return BSE_ERROR_PROC_PARAM_INVAL;
@@ -1090,32 +1101,32 @@ auto_deactivate_exec (BseProcedureClass *proc,
 /* --- undo-depth --- */
 static void
 undo_depth_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 810 "bseproject.proc"
+#line 819 "bseproject.proc"
  {
-#line 812 "bseproject.proc"
+#line 821 "bseproject.proc"
   *(in_pspecs++)    = bse_param_spec_object ("project", "Project", "The project",
                                  BSE_TYPE_PROJECT, SFI_PARAM_STANDARD);
   *(out_pspecs++)   = sfi_pspec_int ("undo_depth", NULL, "Number of times undo can be called on the project",
                          0, 0, G_MAXINT, 1, SFI_PARAM_STANDARD);
-#line 816 "bseproject.proc"
+#line 825 "bseproject.proc"
 }  }
 static BseErrorType
-#line 816 "bseproject.proc"
+#line 825 "bseproject.proc"
 undo_depth_exec (BseProcedureClass *proc,
         const GValue      *in_values,
         GValue            *out_values)
-#line 819 "bseproject.proc"
+#line 828 "bseproject.proc"
 {
-#line 820 "bseproject.proc"
+#line 829 "bseproject.proc"
   
   BseProject *project = bse_value_get_object (in_values++);
   
-#line 823 "bseproject.proc"
+#line 832 "bseproject.proc"
   
   if (!BSE_IS_PROJECT (project))
     return BSE_ERROR_PROC_PARAM_INVAL;
   
-#line 827 "bseproject.proc"
+#line 836 "bseproject.proc"
   
   sfi_value_set_int (out_values++, bse_undo_stack_depth (project->undo_stack));
 
@@ -1125,41 +1136,43 @@ undo_depth_exec (BseProcedureClass *proc,
 /* --- undo --- */
 static void
 undo_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 833 "bseproject.proc"
+#line 842 "bseproject.proc"
  {
-#line 835 "bseproject.proc"
+#line 844 "bseproject.proc"
   *(in_pspecs++)    = bse_param_spec_object ("project", "Project", "The project",
                                  BSE_TYPE_PROJECT, SFI_PARAM_STANDARD);
-#line 837 "bseproject.proc"
+#line 846 "bseproject.proc"
 }  }
 static BseErrorType
-#line 837 "bseproject.proc"
+#line 846 "bseproject.proc"
 undo_exec (BseProcedureClass *proc,
         const GValue      *in_values,
         GValue            *out_values)
-#line 840 "bseproject.proc"
+#line 849 "bseproject.proc"
 {
-#line 841 "bseproject.proc"
+#line 850 "bseproject.proc"
   
   BseProject *project = bse_value_get_object (in_values++);
 
-#line 844 "bseproject.proc"
+#line 853 "bseproject.proc"
   
   if (!BSE_IS_PROJECT (project))
     return BSE_ERROR_PROC_PARAM_INVAL;
 
-#line 848 "bseproject.proc"
+#line 857 "bseproject.proc"
   
   if (!project->in_undo && !project->in_redo)
     {
       const gchar *name = bse_undo_stack_peek (project->undo_stack);
       if (name)
         {
-          project->in_undo = TRUE;
+#line 863 "bseproject.proc"
+          project->in_undo = TRUE;      
           bse_undo_group_open (project->redo_stack, name);
           bse_undo_stack_undo (project->undo_stack);
           bse_undo_group_close (project->redo_stack);
-          project->in_undo = FALSE;
+#line 867 "bseproject.proc"
+          project->in_undo = FALSE;     
         }
     }
 
@@ -1169,32 +1182,32 @@ undo_exec (BseProcedureClass *proc,
 /* --- redo-depth --- */
 static void
 redo_depth_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 865 "bseproject.proc"
+#line 874 "bseproject.proc"
  {
-#line 867 "bseproject.proc"
+#line 876 "bseproject.proc"
   *(in_pspecs++)    = bse_param_spec_object ("project", "Project", "The project",
                                  BSE_TYPE_PROJECT, SFI_PARAM_STANDARD);
   *(out_pspecs++)   = sfi_pspec_int ("redo_depth", NULL, "Number of times redo can be called on the project",
                          0, 0, G_MAXINT, 1, SFI_PARAM_STANDARD);
-#line 871 "bseproject.proc"
+#line 880 "bseproject.proc"
 }  }
 static BseErrorType
-#line 871 "bseproject.proc"
+#line 880 "bseproject.proc"
 redo_depth_exec (BseProcedureClass *proc,
         const GValue      *in_values,
         GValue            *out_values)
-#line 874 "bseproject.proc"
+#line 883 "bseproject.proc"
 {
-#line 875 "bseproject.proc"
+#line 884 "bseproject.proc"
   
   BseProject *project = bse_value_get_object (in_values++);
   
-#line 878 "bseproject.proc"
+#line 887 "bseproject.proc"
   
   if (!BSE_IS_PROJECT (project))
     return BSE_ERROR_PROC_PARAM_INVAL;
   
-#line 882 "bseproject.proc"
+#line 891 "bseproject.proc"
   
   sfi_value_set_int (out_values++, bse_undo_stack_depth (project->redo_stack));
 
@@ -1204,41 +1217,43 @@ redo_depth_exec (BseProcedureClass *proc,
 /* --- redo --- */
 static void
 redo_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 888 "bseproject.proc"
+#line 897 "bseproject.proc"
  {
-#line 890 "bseproject.proc"
+#line 899 "bseproject.proc"
   *(in_pspecs++)    = bse_param_spec_object ("project", "Project", "The project",
                                  BSE_TYPE_PROJECT, SFI_PARAM_STANDARD);
-#line 892 "bseproject.proc"
+#line 901 "bseproject.proc"
 }  }
 static BseErrorType
-#line 892 "bseproject.proc"
+#line 901 "bseproject.proc"
 redo_exec (BseProcedureClass *proc,
         const GValue      *in_values,
         GValue            *out_values)
-#line 895 "bseproject.proc"
+#line 904 "bseproject.proc"
 {
-#line 896 "bseproject.proc"
+#line 905 "bseproject.proc"
   
   BseProject *project = bse_value_get_object (in_values++);
   
-#line 899 "bseproject.proc"
+#line 908 "bseproject.proc"
   
   if (!BSE_IS_PROJECT (project))
     return BSE_ERROR_PROC_PARAM_INVAL;
 
-#line 903 "bseproject.proc"
+#line 912 "bseproject.proc"
   
   if (!project->in_undo && !project->in_redo)
     {
       const gchar *name = bse_undo_stack_peek (project->redo_stack);
       if (name)
         {
-          project->in_redo = TRUE;
+#line 918 "bseproject.proc"
+          project->in_redo = TRUE;      
           bse_undo_group_open (project->undo_stack, name);
           bse_undo_stack_undo (project->redo_stack);
           bse_undo_group_close (project->undo_stack);
-          project->in_redo = FALSE;
+#line 922 "bseproject.proc"
+          project->in_redo = FALSE;     
         }
     }
 
@@ -1248,37 +1263,32 @@ redo_exec (BseProcedureClass *proc,
 /* --- clear-undo --- */
 static void
 clear_undo_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 920 "bseproject.proc"
+#line 929 "bseproject.proc"
  {
-#line 922 "bseproject.proc"
+#line 931 "bseproject.proc"
   *(in_pspecs++)    = bse_param_spec_object ("project", "Project", NULL,
                                  BSE_TYPE_PROJECT, SFI_PARAM_STANDARD);
-#line 924 "bseproject.proc"
+#line 933 "bseproject.proc"
 }  }
 static BseErrorType
-#line 924 "bseproject.proc"
+#line 933 "bseproject.proc"
 clear_undo_exec (BseProcedureClass *proc,
         const GValue      *in_values,
         GValue            *out_values)
-#line 927 "bseproject.proc"
+#line 936 "bseproject.proc"
 {
-#line 928 "bseproject.proc"
+#line 937 "bseproject.proc"
   
   BseProject *project = bse_value_get_object (in_values++);
   
-#line 931 "bseproject.proc"
+#line 940 "bseproject.proc"
   
   if (!BSE_IS_PROJECT (project))
     return BSE_ERROR_PROC_PARAM_INVAL;
 
-#line 935 "bseproject.proc"
+#line 944 "bseproject.proc"
   
-  if (!project->in_undo && !project->in_redo)
-    {
-      bse_undo_stack_clear (project->undo_stack);
-      bse_undo_stack_clear (project->redo_stack);
-      g_object_notify (project, "dirty");
-    }
+  bse_project_clear_undo (project);
 
   return BSE_ERROR_NONE;
 }
@@ -1785,6 +1795,6 @@ BseExportNode* bse__builtin_init_bseproject_gen_proc_c (void)
   return (BseExportNode*) &__enode_clear_undo;
 }
 
-/**
- ** Generated data ends here
- **/
+/*
+ * Generated data ends here
+ */
