@@ -68,7 +68,6 @@ BseErrorType	bse_project_activate		(BseProject	*project);
 void		bse_project_start_playback	(BseProject	*project);
 void		bse_project_stop_playback	(BseProject	*project);
 void		bse_project_check_auto_stop	(BseProject	*project);
-void		bse_project_queue_auto_stop_SL	(BseProject	*project);
 void		bse_project_deactivate		(BseProject	*project);
 void		bse_project_set_auto_deactivate	(BseProject	*project,
 						 gint64		 usecs);
@@ -76,6 +75,7 @@ void		bse_project_keep_activated	(BseProject	*project,
 						 guint64	 min_tick);
 void		bse_project_state_changed	(BseProject	*project,
 						 BseProjectState state);
+void            bse_project_clear_undo          (BseProject     *project);
 BseStringSeq*	bse_project_list_upaths		(BseProject	*project,
 						 GType  	 item_type);
 BseItem*	bse_project_item_from_upath	(BseProject	*project,
@@ -97,6 +97,8 @@ BseWaveRepo*	bse_project_get_wave_repo	(BseProject	*project);
 gpointer	bse_project_create_intern_synth	(BseProject	*project,
 						 const gchar	*synth_name,
 						 GType           check_type);
+BseCSynth*      bse_project_create_intern_csynth(BseProject     *project,
+                                                 const char     *base_name);
 
 
 G_END_DECLS
