@@ -1,7 +1,7 @@
 
-/**
- ** Generated data (by mkcproc.pl)
- **/
+/*
+ * Generated data (by mkcproc.pl)
+ */
 #line 1 "bseserver.proc"
 
 #include <bse/bseplugin.h>
@@ -579,7 +579,7 @@ register_core_plugins_exec (BseProcedureClass *proc,
     }
   registration_done = TRUE;
   
-  ring = bse_plugin_path_list_files ();
+  ring = bse_plugin_path_list_files (!bse_main_args->load_drivers_early, TRUE);
   g_object_set_data (server, "plugin-registration-queue", ring);
   
   bse_idle_normal (register_core_plugins, server);
@@ -991,267 +991,411 @@ get_mp3_version_exec (BseProcedureClass *proc,
 }
 
 /* --- Export to BSE --- */
+static void
+__enode_use_new_project__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Create a new project, owned by the caller (name is modified to be unique if necessary)";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_use_new_project = {
   { NULL, BSE_EXPORT_NODE_PROC,
     "BseServer+use-new-project", 
     NULL,
     "/Methods/BseServer/General/Use New Project",
-    "Create a new project, owned by the caller (name is modified to be unique if necessary)",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_use_new_project__fill_strings,
   },
   0, use_new_project_setup, use_new_project_exec, 
 };
+static void
+__enode_get_midi_notifier__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Retrieve the global midi notifier object.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_get_midi_notifier = {
   { (BseExportNode*) &__enode_use_new_project, BSE_EXPORT_NODE_PROC,
     "BseServer+get-midi-notifier", 
     NULL,
     "/Methods/BseServer/General/Get Midi Notifier",
-    "Retrieve the global midi notifier object.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_get_midi_notifier__fill_strings,
   },
   0, get_midi_notifier_setup, get_midi_notifier_exec, 
 };
+static void
+__enode_can_load__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Check whether a loader can be found for a wave file";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_can_load = {
   { (BseExportNode*) &__enode_get_midi_notifier, BSE_EXPORT_NODE_PROC,
     "BseServer+can-load", 
     NULL,
     "/Methods/BseServer/General/Can Load",
-    "Check whether a loader can be found for a wave file",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_can_load__fill_strings,
   },
   0, can_load_setup, can_load_exec, 
 };
+static void
+__enode_describe_note__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Describe a note, providing information about its octave, semitone, frequency, etc.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_describe_note = {
   { (BseExportNode*) &__enode_can_load, BSE_EXPORT_NODE_PROC,
     "BseServer+describe-note", 
     NULL,
     "/Methods/BseServer/General/Describe Note",
-    "Describe a note, providing information about its octave, semitone, frequency, etc.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_describe_note__fill_strings,
   },
   0, describe_note_setup, describe_note_exec, 
 };
+static void
+__enode_note_from_freq__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Describe a note, given its frequency.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_note_from_freq = {
   { (BseExportNode*) &__enode_describe_note, BSE_EXPORT_NODE_PROC,
     "BseServer+note-from-freq", 
     NULL,
     "/Methods/BseServer/General/Note From Freq",
-    "Describe a note, given its frequency.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_note_from_freq__fill_strings,
   },
   0, note_from_freq_setup, note_from_freq_exec, 
 };
+static void
+__enode_note_from_string__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Describe a note, given its name and octave offset.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_note_from_string = {
   { (BseExportNode*) &__enode_note_from_freq, BSE_EXPORT_NODE_PROC,
     "BseServer+note-from-string", 
     NULL,
     "/Methods/BseServer/General/Note From String",
-    "Describe a note, given its name and octave offset.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_note_from_string__fill_strings,
   },
   0, note_from_string_setup, note_from_string_exec, 
 };
+static void
+__enode_construct_note__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Describe a note, given its semitone, octave and fine tune.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_construct_note = {
   { (BseExportNode*) &__enode_note_from_string, BSE_EXPORT_NODE_PROC,
     "BseServer+construct-note", 
     NULL,
     "/Methods/BseServer/General/Construct Note",
-    "Describe a note, given its semitone, octave and fine tune.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_construct_note__fill_strings,
   },
   0, construct_note_setup, construct_note_exec, 
 };
+static void
+__enode_n_scripts__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Return the number of scripts currently running on this server.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_n_scripts = {
   { (BseExportNode*) &__enode_construct_note, BSE_EXPORT_NODE_PROC,
     "BseServer+n-scripts", 
     NULL,
     "/Methods/BseServer/General/N Scripts",
-    "Return the number of scripts currently running on this server.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_n_scripts__fill_strings,
   },
   0, n_scripts_setup, n_scripts_exec, 
 };
+static void
+__enode_preferences_locked__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Returns whether the bse-preferences property is currently locked against modifications or not.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_preferences_locked = {
   { (BseExportNode*) &__enode_n_scripts, BSE_EXPORT_NODE_PROC,
     "BseServer+preferences-locked", 
     NULL,
     "/Methods/BseServer/General/Preferences Locked",
-    "Returns whether the bse-preferences property is currently locked against modifications or not.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_preferences_locked__fill_strings,
   },
   0, preferences_locked_setup, preferences_locked_exec, 
 };
+static void
+__enode_register_scripts__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Register external scripts.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_register_scripts = {
   { (BseExportNode*) &__enode_preferences_locked, BSE_EXPORT_NODE_PROC,
     "BseServer+register-scripts", 
     NULL,
     "/Methods/BseServer/General/Register Scripts",
-    "Register external scripts.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_register_scripts__fill_strings,
   },
   0, register_scripts_setup, register_scripts_exec, 
 };
+static void
+__enode_register_core_plugins__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Register core plugins.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_register_core_plugins = {
   { (BseExportNode*) &__enode_register_scripts, BSE_EXPORT_NODE_PROC,
     "BseServer+register-core-plugins", 
     NULL,
     "/Methods/BseServer/General/Register Core Plugins",
-    "Register core plugins.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_register_core_plugins__fill_strings,
   },
   0, register_core_plugins_setup, register_core_plugins_exec, 
 };
+static void
+__enode_register_ladspa_plugins__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Register LADSPA (Linux Audio Developer's Simple Plugin API) plugins.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_register_ladspa_plugins = {
   { (BseExportNode*) &__enode_register_core_plugins, BSE_EXPORT_NODE_PROC,
     "BseServer+register-ladspa-plugins", 
     NULL,
     "/Methods/BseServer/General/Register Ladspa Plugins",
-    "Register LADSPA (Linux Audio Developer's Simple Plugin API) plugins.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_register_ladspa_plugins__fill_strings,
   },
   0, register_ladspa_plugins_setup, register_ladspa_plugins_exec, 
 };
+static void
+__enode_save_preferences__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Request the bse-preferences property to be saved into BSE's configuration file.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_save_preferences = {
   { (BseExportNode*) &__enode_register_ladspa_plugins, BSE_EXPORT_NODE_PROC,
     "BseServer+save-preferences", 
     NULL,
     "/Methods/BseServer/General/Save Preferences",
-    "Request the bse-preferences property to be saved into BSE's configuration file.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_save_preferences__fill_strings,
   },
   0, save_preferences_setup, save_preferences_exec, 
 };
+static void
+__enode_get_version__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Retrieve BSE version.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_get_version = {
   { (BseExportNode*) &__enode_save_preferences, BSE_EXPORT_NODE_PROC,
     "BseServer+get-version", 
     NULL,
     "/Methods/BseServer/General/Get Version",
-    "Retrieve BSE version.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_get_version__fill_strings,
   },
   0, get_version_setup, get_version_exec, 
 };
+static void
+__enode_get_demo_path__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Retrieve demo search path.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_get_demo_path = {
   { (BseExportNode*) &__enode_get_version, BSE_EXPORT_NODE_PROC,
     "BseServer+get-demo-path", 
     NULL,
     "/Methods/BseServer/General/Get Demo Path",
-    "Retrieve demo search path.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_get_demo_path__fill_strings,
   },
   0, get_demo_path_setup, get_demo_path_exec, 
 };
+static void
+__enode_get_sample_path__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Retrieve sample search path.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_get_sample_path = {
   { (BseExportNode*) &__enode_get_demo_path, BSE_EXPORT_NODE_PROC,
     "BseServer+get-sample-path", 
     NULL,
     "/Methods/BseServer/General/Get Sample Path",
-    "Retrieve sample search path.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_get_sample_path__fill_strings,
   },
   0, get_sample_path_setup, get_sample_path_exec, 
 };
+static void
+__enode_get_custom_effect_dir__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Retrieve user specific effects directory.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_get_custom_effect_dir = {
   { (BseExportNode*) &__enode_get_sample_path, BSE_EXPORT_NODE_PROC,
     "BseServer+get-custom-effect-dir", 
     NULL,
     "/Methods/BseServer/General/Get Custom Effect Dir",
-    "Retrieve user specific effects directory.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_get_custom_effect_dir__fill_strings,
   },
   0, get_custom_effect_dir_setup, get_custom_effect_dir_exec, 
 };
+static void
+__enode_get_effect_path__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Retrieve effect search path.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_get_effect_path = {
   { (BseExportNode*) &__enode_get_custom_effect_dir, BSE_EXPORT_NODE_PROC,
     "BseServer+get-effect-path", 
     NULL,
     "/Methods/BseServer/General/Get Effect Path",
-    "Retrieve effect search path.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_get_effect_path__fill_strings,
   },
   0, get_effect_path_setup, get_effect_path_exec, 
 };
+static void
+__enode_get_custom_instrument_dir__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Retrieve user specific instruments directory.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_get_custom_instrument_dir = {
   { (BseExportNode*) &__enode_get_effect_path, BSE_EXPORT_NODE_PROC,
     "BseServer+get-custom-instrument-dir", 
     NULL,
     "/Methods/BseServer/General/Get Custom Instrument Dir",
-    "Retrieve user specific instruments directory.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_get_custom_instrument_dir__fill_strings,
   },
   0, get_custom_instrument_dir_setup, get_custom_instrument_dir_exec, 
 };
+static void
+__enode_get_instrument_path__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Retrieve instrument search path.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_get_instrument_path = {
   { (BseExportNode*) &__enode_get_custom_instrument_dir, BSE_EXPORT_NODE_PROC,
     "BseServer+get-instrument-path", 
     NULL,
     "/Methods/BseServer/General/Get Instrument Path",
-    "Retrieve instrument search path.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_get_instrument_path__fill_strings,
   },
   0, get_instrument_path_setup, get_instrument_path_exec, 
 };
+static void
+__enode_get_script_path__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Retrieve script search path.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_get_script_path = {
   { (BseExportNode*) &__enode_get_instrument_path, BSE_EXPORT_NODE_PROC,
     "BseServer+get-script-path", 
     NULL,
     "/Methods/BseServer/General/Get Script Path",
-    "Retrieve script search path.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_get_script_path__fill_strings,
   },
   0, get_script_path_setup, get_script_path_exec, 
 };
+static void
+__enode_get_plugin_path__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Retrieve plugin search path.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_get_plugin_path = {
   { (BseExportNode*) &__enode_get_script_path, BSE_EXPORT_NODE_PROC,
     "BseServer+get-plugin-path", 
     NULL,
     "/Methods/BseServer/General/Get Plugin Path",
-    "Retrieve plugin search path.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_get_plugin_path__fill_strings,
   },
   0, get_plugin_path_setup, get_plugin_path_exec, 
 };
+static void
+__enode_get_ladspa_path__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Retrieve ladspa search path.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_get_ladspa_path = {
   { (BseExportNode*) &__enode_get_plugin_path, BSE_EXPORT_NODE_PROC,
     "BseServer+get-ladspa-path", 
     NULL,
     "/Methods/BseServer/General/Get Ladspa Path",
-    "Retrieve ladspa search path.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_get_ladspa_path__fill_strings,
   },
   0, get_ladspa_path_setup, get_ladspa_path_exec, 
 };
+static void
+__enode_get_mp3_version__fill_strings (BseExportStrings *es)
+{
+  es->blurb = "Retrieve BSE MP3 handler version.";
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU General Public License";
+}
 static BseExportNodeProc __enode_get_mp3_version = {
   { (BseExportNode*) &__enode_get_ladspa_path, BSE_EXPORT_NODE_PROC,
     "BseServer+get-mp3-version", 
     NULL,
     "/Methods/BseServer/General/Get Mp3 Version",
-    "Retrieve BSE MP3 handler version.",
-    "Tim Janik <timj@gtk.org>",
-    "GNU General Public License",
+    NULL,
+    __enode_get_mp3_version__fill_strings,
   },
   0, get_mp3_version_setup, get_mp3_version_exec, 
 };
@@ -1261,6 +1405,6 @@ BseExportNode* bse__builtin_init_bseserver_gen_proc_c (void)
   return (BseExportNode*) &__enode_get_mp3_version;
 }
 
-/**
- ** Generated data ends here
- **/
+/*
+ * Generated data ends here
+ */
