@@ -1,23 +1,23 @@
 
-/**
- ** Generated data (by mkcproc.pl)
- **/
+/*
+ * Generated data (by mkcproc.pl)
+ */
 #line 1 "bsesong.proc"
 
 #include <bse/bseplugin.h>
 #include <bse/bseprocedure.h>
 #include <bse/bsesong.h>
-#include <bse/bsesongbus.h>
+#include <bse/bsebus.h>
 #include <bse/bsepart.h>
 #include <bse/bsetrack.h>
 #include "bsemidireceiver.h"
 #include "bseundostack.h"
 #include "bseproject.h"
 #include "gslcommon.h"
-#include "gslengine.h"
-
-#line 33 "bsesong.proc"
+#include "bseengine.h"
+#include <string.h>
 
+#line 34 "bsesong.proc"
 
 /* --- create-bus --- */
 static void
@@ -28,7 +28,7 @@ create_bus_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **
   *(in_pspecs++)    = bse_param_spec_object ("song", NULL, NULL,
 				 BSE_TYPE_SONG, SFI_PARAM_STANDARD);
   *(out_pspecs++)   = bse_param_spec_object ("bus", NULL, NULL,
-				 BSE_TYPE_SONG_BUS, SFI_PARAM_STANDARD);
+				 BSE_TYPE_BUS, SFI_PARAM_STANDARD);
 #line 41 "bsesong.proc"
 }  }
 static BseErrorType
@@ -41,75 +41,85 @@ create_bus_exec (BseProcedureClass *proc,
 #line 45 "bsesong.proc"
   
   BseSong *self = bse_value_get_object (in_values++);
+  BseErrorType error = BSE_ERROR_NONE;
   BseUndoStack *ustack;
-  BseItem *child;
+  BseItem *child = NULL;
 
-#line 50 "bsesong.proc"
+#line 51 "bsesong.proc"
   
   if (!BSE_IS_SONG (self))
     return BSE_ERROR_PROC_PARAM_INVAL;
   
-#line 54 "bsesong.proc"
+#line 55 "bsesong.proc"
   
-  ustack = bse_item_undo_open (self, "create-bus");
-  child = bse_container_new_child (BSE_CONTAINER (self), BSE_TYPE_SONG_BUS, NULL);
-  bse_item_push_undo_proc (self, "remove-bus", child);
-  bse_item_undo_close (ustack);
-  
-#line 60 "bsesong.proc"
+  if (BSE_SOURCE_PREPARED (self))
+    error = BSE_ERROR_SOURCE_BUSY;
+  else
+    {
+      ustack = bse_item_undo_open (self, "create-bus");
+      child = bse_container_new_child (BSE_CONTAINER (self), BSE_TYPE_BUS, NULL);
+      bse_item_push_undo_proc (self, "remove-bus", child);
+      bse_item_undo_close (ustack);
+    }
+
+#line 66 "bsesong.proc"
   
   bse_value_set_object (out_values++, child);
   
-  return BSE_ERROR_NONE;
+  return error;
 }
 
 /* --- remove-bus --- */
 static void
 remove_bus_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 66 "bsesong.proc"
+#line 72 "bsesong.proc"
  {
-#line 68 "bsesong.proc"
+#line 74 "bsesong.proc"
   *(in_pspecs++)    = bse_param_spec_object ("song", NULL, NULL,
 				 BSE_TYPE_SONG, SFI_PARAM_STANDARD);
   *(in_pspecs++)    = bse_param_spec_object ("bus", NULL, NULL,
-				 BSE_TYPE_SONG_BUS, SFI_PARAM_STANDARD);
-#line 72 "bsesong.proc"
+				 BSE_TYPE_BUS, SFI_PARAM_STANDARD);
+#line 78 "bsesong.proc"
 }  }
 static BseErrorType
-#line 72 "bsesong.proc"
+#line 78 "bsesong.proc"
 remove_bus_exec (BseProcedureClass *proc,
         const GValue      *in_values,
         GValue            *out_values)
-#line 75 "bsesong.proc"
+#line 81 "bsesong.proc"
 {
-#line 76 "bsesong.proc"
+#line 82 "bsesong.proc"
   
   BseSong *self = bse_value_get_object (in_values++);
   BseItem *child = bse_value_get_object (in_values++);
   BseErrorType error = BSE_ERROR_NONE;
   
-#line 81 "bsesong.proc"
+#line 87 "bsesong.proc"
   
-  if (!BSE_IS_SONG (self) || !BSE_IS_SONG_BUS (child) || BSE_ITEM (child)->parent != BSE_ITEM (self))
+  if (!BSE_IS_SONG (self) || !BSE_IS_BUS (child) || BSE_ITEM (child)->parent != BSE_ITEM (self))
     return BSE_ERROR_PROC_PARAM_INVAL;
   
-#line 85 "bsesong.proc"
+#line 91 "bsesong.proc"
   
   if (BSE_SOURCE_PREPARED (self))
     error = BSE_ERROR_SOURCE_BUSY;
   else
     {
       BseUndoStack *ustack = bse_item_undo_open (self, "remove-child %s", bse_object_debug_name (child));
-#line 91 "bsesong.proc"
+#line 97 "bsesong.proc"
       
-      bse_container_uncross_undoable (BSE_CONTAINER (self), child);
-#line 93 "bsesong.proc"
+      bse_item_set (child, "master-output", FALSE, NULL);
+#line 99 "bsesong.proc"
+      
+#line 100 "bsesong.proc"
+      bse_container_uncross_undoable (BSE_CONTAINER (self), child);     
+#line 101 "bsesong.proc"
       
       bse_item_push_redo_proc (self, "remove-bus", child);
-#line 95 "bsesong.proc"
+#line 103 "bsesong.proc"
       
       bse_container_remove_backedup (BSE_CONTAINER (self), child, ustack);
-#line 97 "bsesong.proc"
+#line 105 "bsesong.proc"
       
       bse_item_undo_close (ustack);
     }
@@ -117,44 +127,216 @@ remove_bus_exec (BseProcedureClass *proc,
   return error;
 }
 
+/* --- ensure-master-bus --- */
+static void
+ensure_master_bus_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
+#line 112 "bsesong.proc"
+ {
+#line 114 "bsesong.proc"
+  *(in_pspecs++)    = bse_param_spec_object ("song", NULL, NULL,
+				 BSE_TYPE_SONG, SFI_PARAM_STANDARD);
+  *(out_pspecs++)   = bse_param_spec_object ("bus", NULL, NULL,
+				 BSE_TYPE_BUS, SFI_PARAM_STANDARD);
+#line 118 "bsesong.proc"
+}  }
+static BseErrorType
+#line 118 "bsesong.proc"
+ensure_master_bus_exec (BseProcedureClass *proc,
+        const GValue      *in_values,
+        GValue            *out_values)
+#line 121 "bsesong.proc"
+{
+#line 122 "bsesong.proc"
+  
+  BseSong *self = bse_value_get_object (in_values++);
+  BseSource *child;
+
+#line 126 "bsesong.proc"
+  
+  if (!BSE_IS_SONG (self))
+    return BSE_ERROR_PROC_PARAM_INVAL;
+  
+#line 130 "bsesong.proc"
+  
+  child = bse_song_ensure_master (self);
+
+#line 133 "bsesong.proc"
+  
+  bse_value_set_object (out_values++, child);
+  
+  return BSE_ERROR_NONE;
+}
+
+/* --- get-master-bus --- */
+static void
+get_master_bus_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
+#line 139 "bsesong.proc"
+ {
+#line 141 "bsesong.proc"
+  *(in_pspecs++)    = bse_param_spec_object ("song", NULL, NULL,
+				 BSE_TYPE_SONG, SFI_PARAM_STANDARD);
+  *(out_pspecs++)   = bse_param_spec_object ("bus", NULL, NULL,
+				 BSE_TYPE_BUS, SFI_PARAM_STANDARD);
+#line 145 "bsesong.proc"
+}  }
+static BseErrorType
+#line 145 "bsesong.proc"
+get_master_bus_exec (BseProcedureClass *proc,
+        const GValue      *in_values,
+        GValue            *out_values)
+#line 148 "bsesong.proc"
+{
+#line 149 "bsesong.proc"
+  
+  BseSong *self = bse_value_get_object (in_values++);
+
+#line 152 "bsesong.proc"
+  
+  if (!BSE_IS_SONG (self))
+    return BSE_ERROR_PROC_PARAM_INVAL;
+  
+#line 156 "bsesong.proc"
+  
+  BseBus *bus = bse_song_find_master (self);
+
+#line 159 "bsesong.proc"
+  
+  bse_value_set_object (out_values++, bus);
+  
+  return BSE_ERROR_NONE;
+}
+
+static const gchar*
+orphans_track_name (void)
+{
+#line 168 "bsesong.proc"
+  
+  return _("Orphan Parts");
+}
+
+static BseTrack*
+bse_song_ensure_orphans_track_noundo (BseSong *self)
+{
+  SfiRing *ring;
+  for (ring = self->tracks_SL; ring; ring = sfi_ring_walk (ring, self->tracks_SL))
+    {
+      BseTrack *track = ring->data;
+      gboolean muted = FALSE;
+      g_object_get (track, "muted", &muted, NULL);
+#line 183 "bsesong.proc"
+      if (muted && g_object_get_data (track, "BseSong-orphan-track") == bse_song_ensure_orphans_track_noundo) 
+        return track;
+    }
+  gpointer child = bse_container_new_child_bname (BSE_CONTAINER (self), BSE_TYPE_TRACK, orphans_track_name(), NULL);
+#line 187 "bsesong.proc"
+  g_object_set (child, "muted", TRUE, NULL); 
+#line 188 "bsesong.proc"
+  g_object_set_data (child, "BseSong-orphan-track", bse_song_ensure_orphans_track_noundo); 
+  return child;
+}
+
+static BseTrack*
+bse_song_find_first_track (BseSong *self,
+                           BsePart *part)
+{
+  SfiRing *ring;
+#line 197 "bsesong.proc"
+  
+  for (ring = self->tracks_SL; ring; ring = sfi_ring_walk (ring, self->tracks_SL))
+    {
+      BseTrack *track = ring->data;
+      guint start;
+      if (bse_track_find_part (track, part, &start))
+        return track;
+    }
+  return NULL;
+}
+
+/* --- ensure-track-links --- */
+static void
+ensure_track_links_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
+#line 208 "bsesong.proc"
+ {
+#line 210 "bsesong.proc"
+  *(in_pspecs++)    = bse_param_spec_object ("song", NULL, NULL,
+				 BSE_TYPE_SONG, SFI_PARAM_STANDARD);
+#line 212 "bsesong.proc"
+}  }
+static BseErrorType
+#line 212 "bsesong.proc"
+ensure_track_links_exec (BseProcedureClass *proc,
+        const GValue      *in_values,
+        GValue            *out_values)
+#line 215 "bsesong.proc"
+{
+#line 216 "bsesong.proc"
+  
+  BseSong *self = bse_value_get_object (in_values++);
+
+#line 219 "bsesong.proc"
+  
+  if (!BSE_IS_SONG (self))
+    return BSE_ERROR_PROC_PARAM_INVAL;
+  
+#line 223 "bsesong.proc"
+  
+  gboolean clear_undo = FALSE;
+  SfiRing *ring;
+  for (ring = self->parts; ring; ring = sfi_ring_walk (ring, self->parts))
+    if (!bse_song_find_first_track (self, ring->data))
+      {
+        BseTrack *track = bse_song_ensure_orphans_track_noundo (self);
+        bse_item_exec_void (track, "insert-part", bse_track_get_last_tick (track), ring->data);
+        clear_undo = TRUE;
+      }
+  if (clear_undo)
+    {
+      BseProject *project = bse_item_get_project (BSE_ITEM (self));
+      if (project)
+        bse_project_clear_undo (project);
+    }
+  
+  return BSE_ERROR_NONE;
+}
+
 /* --- create-part --- */
 static void
 create_part_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 104 "bsesong.proc"
+#line 243 "bsesong.proc"
  {
-#line 106 "bsesong.proc"
-  *(in_pspecs++)    = bse_param_spec_object ("song", "Song", NULL,
+#line 245 "bsesong.proc"
+  *(in_pspecs++)    = bse_param_spec_object ("song", NULL, NULL,
 				 BSE_TYPE_SONG, SFI_PARAM_STANDARD);
-  *(out_pspecs++)   = bse_param_spec_object ("part", "Part", NULL,
+  *(out_pspecs++)   = bse_param_spec_object ("part", NULL, NULL,
 				 BSE_TYPE_PART, SFI_PARAM_STANDARD);
-#line 110 "bsesong.proc"
+#line 249 "bsesong.proc"
 }  }
 static BseErrorType
-#line 110 "bsesong.proc"
+#line 249 "bsesong.proc"
 create_part_exec (BseProcedureClass *proc,
         const GValue      *in_values,
         GValue            *out_values)
-#line 113 "bsesong.proc"
+#line 252 "bsesong.proc"
 {
-#line 114 "bsesong.proc"
+#line 253 "bsesong.proc"
   
   BseSong *self = bse_value_get_object (in_values++);
   BseUndoStack *ustack;
   BseItem *child;
 
-#line 119 "bsesong.proc"
+#line 258 "bsesong.proc"
   
   if (!BSE_IS_SONG (self))
     return BSE_ERROR_PROC_PARAM_INVAL;
   
-#line 123 "bsesong.proc"
+#line 262 "bsesong.proc"
   
   ustack = bse_item_undo_open (self, "create-part");
   child = bse_container_new_child (BSE_CONTAINER (self), BSE_TYPE_PART, NULL);
   bse_item_push_undo_proc (self, "remove-part", child);
   bse_item_undo_close (ustack);
   
-#line 129 "bsesong.proc"
+#line 268 "bsesong.proc"
   
   bse_value_set_object (out_values++, child);
   
@@ -164,50 +346,50 @@ create_part_exec (BseProcedureClass *proc,
 /* --- remove-part --- */
 static void
 remove_part_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 135 "bsesong.proc"
+#line 274 "bsesong.proc"
  {
-#line 137 "bsesong.proc"
+#line 276 "bsesong.proc"
   *(in_pspecs++)    = bse_param_spec_object ("song", "Song", NULL,
 				 BSE_TYPE_SONG, SFI_PARAM_STANDARD);
   *(in_pspecs++)    = bse_param_spec_object ("part", "Part", NULL,
 				 BSE_TYPE_PART, SFI_PARAM_STANDARD);
-#line 141 "bsesong.proc"
+#line 280 "bsesong.proc"
 }  }
 static BseErrorType
-#line 141 "bsesong.proc"
+#line 280 "bsesong.proc"
 remove_part_exec (BseProcedureClass *proc,
         const GValue      *in_values,
         GValue            *out_values)
-#line 144 "bsesong.proc"
+#line 283 "bsesong.proc"
 {
-#line 145 "bsesong.proc"
+#line 284 "bsesong.proc"
   
   BseSong *self = bse_value_get_object (in_values++);
   BseItem *child = bse_value_get_object (in_values++);
   BseErrorType error = BSE_ERROR_NONE;
   
-#line 150 "bsesong.proc"
+#line 289 "bsesong.proc"
   
   if (!BSE_IS_SONG (self) || !BSE_IS_PART (child) || BSE_ITEM (child)->parent != BSE_ITEM (self))
     return BSE_ERROR_PROC_PARAM_INVAL;
   
-#line 154 "bsesong.proc"
+#line 293 "bsesong.proc"
   
   if (BSE_SOURCE_PREPARED (self))
     error = BSE_ERROR_SOURCE_BUSY;
   else
     {
       BseUndoStack *ustack = bse_item_undo_open (self, "remove-child %s", bse_object_debug_name (child));
-#line 160 "bsesong.proc"
+#line 299 "bsesong.proc"
       
       bse_container_uncross_undoable (BSE_CONTAINER (self), child);
-#line 162 "bsesong.proc"
+#line 301 "bsesong.proc"
       
       bse_item_push_redo_proc (self, "remove-part", child);
-#line 164 "bsesong.proc"
+#line 303 "bsesong.proc"
       
       bse_container_remove_backedup (BSE_CONTAINER (self), child, ustack);
-#line 166 "bsesong.proc"
+#line 305 "bsesong.proc"
       
       bse_item_undo_close (ustack);
     }
@@ -218,34 +400,34 @@ remove_part_exec (BseProcedureClass *proc,
 /* --- create-track --- */
 static void
 create_track_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 173 "bsesong.proc"
+#line 312 "bsesong.proc"
  {
-#line 175 "bsesong.proc"
+#line 314 "bsesong.proc"
   *(in_pspecs++)    = bse_param_spec_object ("song", "Song", NULL,
 				 BSE_TYPE_SONG, SFI_PARAM_STANDARD);
   *(out_pspecs++)   = bse_param_spec_object ("track", "Track", NULL,
 				 BSE_TYPE_TRACK, SFI_PARAM_STANDARD);
-#line 179 "bsesong.proc"
+#line 318 "bsesong.proc"
 }  }
 static BseErrorType
-#line 179 "bsesong.proc"
+#line 318 "bsesong.proc"
 create_track_exec (BseProcedureClass *proc,
         const GValue      *in_values,
         GValue            *out_values)
-#line 182 "bsesong.proc"
+#line 321 "bsesong.proc"
 {
-#line 183 "bsesong.proc"
+#line 322 "bsesong.proc"
   
   BseSong *self = bse_value_get_object (in_values++);
   BseErrorType error = BSE_ERROR_NONE;
   BseItem *child = NULL;
 
-#line 188 "bsesong.proc"
+#line 327 "bsesong.proc"
   
   if (!BSE_IS_SONG (self))
     return BSE_ERROR_PROC_PARAM_INVAL;
   
-#line 192 "bsesong.proc"
+#line 331 "bsesong.proc"
   
   if (BSE_SOURCE_PREPARED (self))
     error = BSE_ERROR_SOURCE_BUSY;
@@ -257,7 +439,7 @@ create_track_exec (BseProcedureClass *proc,
       bse_item_undo_close (ustack);
     }
 
-#line 203 "bsesong.proc"
+#line 342 "bsesong.proc"
   
   bse_value_set_object (out_values++, child);
   
@@ -267,51 +449,51 @@ create_track_exec (BseProcedureClass *proc,
 /* --- remove-track --- */
 static void
 remove_track_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 209 "bsesong.proc"
+#line 348 "bsesong.proc"
  {
-#line 211 "bsesong.proc"
+#line 350 "bsesong.proc"
   *(in_pspecs++)    = bse_param_spec_object ("song", "Song", NULL,
 				 BSE_TYPE_SONG, SFI_PARAM_STANDARD);
   *(in_pspecs++)    = bse_param_spec_object ("track", "Track", NULL,
 				 BSE_TYPE_TRACK, SFI_PARAM_STANDARD);
-#line 215 "bsesong.proc"
+#line 354 "bsesong.proc"
 }  }
 static BseErrorType
-#line 215 "bsesong.proc"
+#line 354 "bsesong.proc"
 remove_track_exec (BseProcedureClass *proc,
         const GValue      *in_values,
         GValue            *out_values)
-#line 218 "bsesong.proc"
+#line 357 "bsesong.proc"
 {
-#line 219 "bsesong.proc"
+#line 358 "bsesong.proc"
   
   BseSong *self  = bse_value_get_object (in_values++);
   BseItem *child = bse_value_get_object (in_values++);
   BseErrorType error = BSE_ERROR_NONE;
   
-#line 224 "bsesong.proc"
+#line 363 "bsesong.proc"
   
   if (!BSE_IS_SONG (self) || !BSE_IS_TRACK (child) ||
       BSE_ITEM (child)->parent != BSE_ITEM (self))
     return BSE_ERROR_PROC_PARAM_INVAL;
   
-#line 229 "bsesong.proc"
+#line 368 "bsesong.proc"
   
   if (BSE_SOURCE_PREPARED (self))
     error = BSE_ERROR_SOURCE_BUSY;
   else
     {
       BseUndoStack *ustack = bse_item_undo_open (self, "remove-child %s", bse_object_debug_name (child));
-#line 235 "bsesong.proc"
+#line 374 "bsesong.proc"
       
       bse_container_uncross_undoable (BSE_CONTAINER (self), child);
-#line 237 "bsesong.proc"
+#line 376 "bsesong.proc"
       
       bse_item_push_redo_proc (self, "remove-track", child);
-#line 239 "bsesong.proc"
+#line 378 "bsesong.proc"
       
       bse_container_remove_backedup (BSE_CONTAINER (self), child, ustack);
-#line 241 "bsesong.proc"
+#line 380 "bsesong.proc"
       
       bse_item_undo_close (ustack);
     }
@@ -322,25 +504,25 @@ remove_track_exec (BseProcedureClass *proc,
 /* --- find-track-for-part --- */
 static void
 find_track_for_part_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 248 "bsesong.proc"
+#line 387 "bsesong.proc"
  {
-#line 250 "bsesong.proc"
+#line 389 "bsesong.proc"
   *(in_pspecs++)    = bse_param_spec_object ("song", "Song", NULL,
 				 BSE_TYPE_SONG, SFI_PARAM_STANDARD);
   *(in_pspecs++)    = bse_param_spec_object ("part", "Part", NULL,
 				 BSE_TYPE_PART, SFI_PARAM_STANDARD);
   *(out_pspecs++)   = bse_param_spec_object ("track", "Track", NULL,
 				 BSE_TYPE_TRACK, SFI_PARAM_STANDARD);
-#line 256 "bsesong.proc"
+#line 395 "bsesong.proc"
 }  }
 static BseErrorType
-#line 256 "bsesong.proc"
+#line 395 "bsesong.proc"
 find_track_for_part_exec (BseProcedureClass *proc,
         const GValue      *in_values,
         GValue            *out_values)
-#line 259 "bsesong.proc"
+#line 398 "bsesong.proc"
 {
-#line 260 "bsesong.proc"
+#line 399 "bsesong.proc"
   
   BseSong *self = bse_value_get_object (in_values++);
   BsePart *part = bse_value_get_object (in_values++);
@@ -348,13 +530,13 @@ find_track_for_part_exec (BseProcedureClass *proc,
   guint tick = 0;
   SfiRing *ring;
 
-#line 267 "bsesong.proc"
+#line 406 "bsesong.proc"
   
   if (!BSE_IS_SONG (self) || !BSE_IS_PART (part) ||
       BSE_ITEM (part)->parent != BSE_ITEM (self))
     return BSE_ERROR_PROC_PARAM_INVAL;
 
-#line 272 "bsesong.proc"
+#line 411 "bsesong.proc"
   
   for (ring = self->tracks_SL; ring; ring = sfi_ring_walk (ring, self->tracks_SL))
     {
@@ -367,22 +549,59 @@ find_track_for_part_exec (BseProcedureClass *proc,
 	  tick = start;
 	}
     }
-  if (!track)
-    track = self->tracks_SL ? self->tracks_SL->data : NULL;
 
-#line 287 "bsesong.proc"
+#line 424 "bsesong.proc"
   
   bse_value_set_object (out_values++, track);
   
   return BSE_ERROR_NONE;
 }
 
+/* --- find-any-track-for-part --- */
+static void
+find_any_track_for_part_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
+#line 430 "bsesong.proc"
+ {
+#line 432 "bsesong.proc"
+  *(in_pspecs++)    = bse_param_spec_object ("song", "Song", NULL,
+				 BSE_TYPE_SONG, SFI_PARAM_STANDARD);
+  *(in_pspecs++)    = bse_param_spec_object ("part", "Part", NULL,
+				 BSE_TYPE_PART, SFI_PARAM_STANDARD);
+  *(out_pspecs++)   = bse_param_spec_object ("track", "Track", NULL,
+				 BSE_TYPE_TRACK, SFI_PARAM_STANDARD);
+#line 438 "bsesong.proc"
+}  }
+static BseErrorType
+#line 438 "bsesong.proc"
+find_any_track_for_part_exec (BseProcedureClass *proc,
+        const GValue      *in_values,
+        GValue            *out_values)
+#line 441 "bsesong.proc"
+{
+#line 442 "bsesong.proc"
+  
+  BseSong *self = bse_value_get_object (in_values++);
+  BsePart *part = bse_value_get_object (in_values++);
+
+#line 446 "bsesong.proc"
+  
+  if (!BSE_IS_SONG (self) || !BSE_IS_PART (part) ||
+      BSE_ITEM (part)->parent != BSE_ITEM (self))
+    return BSE_ERROR_PROC_PARAM_INVAL;
+
+#line 451 "bsesong.proc"
+  
+  bse_value_set_object (out_values++, bse_song_find_first_track (self, part));
+  
+  return BSE_ERROR_NONE;
+}
+
 /* --- synthesize-note --- */
 static void
 synthesize_note_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 293 "bsesong.proc"
+#line 457 "bsesong.proc"
  {
-#line 295 "bsesong.proc"
+#line 459 "bsesong.proc"
   *(in_pspecs++)    = bse_param_spec_object ("song", "Song", NULL,
 				 BSE_TYPE_SONG, SFI_PARAM_STANDARD);
   *(in_pspecs++)    = bse_param_spec_object ("track", "Track", NULL,
@@ -395,16 +614,16 @@ synthesize_note_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSp
   *(in_pspecs++)    = sfi_pspec_real ("velocity", "Velocity", NULL,
 			  1.0, 0, 1.0, 0.01,
 			  SFI_PARAM_STANDARD ":scale");
-#line 307 "bsesong.proc"
+#line 471 "bsesong.proc"
 }  }
 static BseErrorType
-#line 307 "bsesong.proc"
+#line 471 "bsesong.proc"
 synthesize_note_exec (BseProcedureClass *proc,
         const GValue      *in_values,
         GValue            *out_values)
-#line 310 "bsesong.proc"
+#line 474 "bsesong.proc"
 {
-#line 311 "bsesong.proc"
+#line 475 "bsesong.proc"
   
   BseSong *self      = bse_value_get_object (in_values++);
   BseTrack *track    = bse_value_get_object (in_values++);
@@ -413,20 +632,20 @@ synthesize_note_exec (BseProcedureClass *proc,
   gint     fine_tune = sfi_value_get_int (in_values++);
   gfloat   velocity  = sfi_value_get_real (in_values++);
   
-#line 319 "bsesong.proc"
+#line 483 "bsesong.proc"
   
   if (!BSE_IS_SONG (self) || !BSE_IS_TRACK (track) ||
       BSE_ITEM (track)->parent != BSE_ITEM (self))
     return BSE_ERROR_PROC_PARAM_INVAL;
   
-#line 324 "bsesong.proc"
+#line 488 "bsesong.proc"
   
   if (BSE_SOURCE_PREPARED (self) && self->midi_receiver_SL)
     {
       gfloat freq = BSE_KAMMER_FREQUENCY_f *
 		    BSE_SEMITONE_FACTOR (note) *
 		    BSE_FINE_TUNE_FACTOR (fine_tune);
-      SfiTime tstamp = gsl_tick_stamp () + gsl_engine_block_size () * 2;
+      SfiTime tstamp = gsl_tick_stamp () + bse_engine_block_size () * 2;
       BseMidiEvent *eon, *eoff;
       eon  = bse_midi_event_note_on (track->midi_channel_SL, tstamp, freq, velocity);
       eoff = bse_midi_event_note_off (track->midi_channel_SL, tstamp + duration, freq);
@@ -442,37 +661,37 @@ synthesize_note_exec (BseProcedureClass *proc,
 /* --- get-timing --- */
 static void
 get_timing_setup (BseProcedureClass *proc, GParamSpec **in_pspecs, GParamSpec **out_pspecs) {
-#line 343 "bsesong.proc"
+#line 507 "bsesong.proc"
  {
-#line 345 "bsesong.proc"
+#line 509 "bsesong.proc"
   *(in_pspecs++)    = bse_param_spec_object ("song", "Song", NULL, BSE_TYPE_SONG, SFI_PARAM_STANDARD);
   *(in_pspecs++)    = sfi_pspec_int ("tick", "Tick", "Tick to retrieve timing info about", 0, 0, G_MAXINT, 384, SFI_PARAM_STANDARD);
   *(out_pspecs++)   = bse_param_spec_boxed ("timing", "Timing", "Song Timing", BSE_TYPE_SONG_TIMING, SFI_PARAM_STANDARD);
-#line 348 "bsesong.proc"
+#line 512 "bsesong.proc"
 }  }
 static BseErrorType
-#line 348 "bsesong.proc"
+#line 512 "bsesong.proc"
 get_timing_exec (BseProcedureClass *proc,
         const GValue      *in_values,
         GValue            *out_values)
-#line 351 "bsesong.proc"
+#line 515 "bsesong.proc"
 {
-#line 352 "bsesong.proc"
+#line 516 "bsesong.proc"
   
   BseSong *self = bse_value_get_object (in_values++);
   SfiInt   tick = sfi_value_get_int (in_values++);
   BseSongTiming timing = { 0, };
 
-#line 357 "bsesong.proc"
+#line 521 "bsesong.proc"
   
   if (!BSE_IS_SONG (self))
     return BSE_ERROR_PROC_PARAM_INVAL;
 
-#line 361 "bsesong.proc"
+#line 525 "bsesong.proc"
   
   bse_song_get_timing (self, tick, &timing);
 
-#line 364 "bsesong.proc"
+#line 528 "bsesong.proc"
   
   bse_value_set_boxed (out_values++, &timing);
   
@@ -483,7 +702,7 @@ get_timing_exec (BseProcedureClass *proc,
 static void
 __enode_create_bus__fill_strings (BseExportStrings *es)
 {
-  es->blurb = "Create a new Bus in a Song";
+  es->blurb = _("Create a new mixer bus for a Song.");
   es->authors = "Tim Janik <timj@gtk.org>";
   es->license = "GNU Lesser General Public License";
 }
@@ -500,7 +719,7 @@ static BseExportNodeProc __enode_create_bus = {
 static void
 __enode_remove_bus__fill_strings (BseExportStrings *es)
 {
-  es->blurb = "Delete a Bus from a Song";
+  es->blurb = _("Delete a mixer bus from a Song.");
   es->authors = "Tim Janik <timj@gtk.org>";
   es->license = "GNU Lesser General Public License";
 }
@@ -515,14 +734,65 @@ static BseExportNodeProc __enode_remove_bus = {
   0, remove_bus_setup, remove_bus_exec, 
 };
 static void
+__enode_ensure_master_bus__fill_strings (BseExportStrings *es)
+{
+  es->blurb = _("Retrieve master output bus of a song, will create one if it doesn't exist.");
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU Lesser General Public License";
+}
+static BseExportNodeProc __enode_ensure_master_bus = {
+  { (BseExportNode*) &__enode_remove_bus, BSE_EXPORT_NODE_PROC,
+    "BseSong+ensure-master-bus", 
+    NULL,
+    "/Methods/BseSong/General/Ensure Master Bus",
+    NULL,
+    __enode_ensure_master_bus__fill_strings,
+  },
+  0, ensure_master_bus_setup, ensure_master_bus_exec, 
+};
+static void
+__enode_get_master_bus__fill_strings (BseExportStrings *es)
+{
+  es->blurb = _("Retrieve master output bus of a song if it exists.");
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU Lesser General Public License";
+}
+static BseExportNodeProc __enode_get_master_bus = {
+  { (BseExportNode*) &__enode_ensure_master_bus, BSE_EXPORT_NODE_PROC,
+    "BseSong+get-master-bus", 
+    NULL,
+    "/Methods/BseSong/General/Get Master Bus",
+    NULL,
+    __enode_get_master_bus__fill_strings,
+  },
+  0, get_master_bus_setup, get_master_bus_exec, 
+};
+static void
+__enode_ensure_track_links__fill_strings (BseExportStrings *es)
+{
+  es->blurb = _("Ensure that each part in a song is inserted into at least one track.");
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU Lesser General Public License";
+}
+static BseExportNodeProc __enode_ensure_track_links = {
+  { (BseExportNode*) &__enode_get_master_bus, BSE_EXPORT_NODE_PROC,
+    "BseSong+ensure-track-links", 
+    NULL,
+    "/Methods/BseSong/General/Ensure Track Links",
+    NULL,
+    __enode_ensure_track_links__fill_strings,
+  },
+  0, ensure_track_links_setup, ensure_track_links_exec, 
+};
+static void
 __enode_create_part__fill_strings (BseExportStrings *es)
 {
-  es->blurb = "Create a new Part in a Song";
+  es->blurb = _("Create a new Part in a Song.");
   es->authors = "Tim Janik <timj@gtk.org>";
   es->license = "GNU Lesser General Public License";
 }
 static BseExportNodeProc __enode_create_part = {
-  { (BseExportNode*) &__enode_remove_bus, BSE_EXPORT_NODE_PROC,
+  { (BseExportNode*) &__enode_ensure_track_links, BSE_EXPORT_NODE_PROC,
     "BseSong+create-part", 
     NULL,
     "/Methods/BseSong/General/Create Part",
@@ -534,7 +804,7 @@ static BseExportNodeProc __enode_create_part = {
 static void
 __enode_remove_part__fill_strings (BseExportStrings *es)
 {
-  es->blurb = "Delete a Part from a Song";
+  es->blurb = _("Delete a Part from a Song.");
   es->authors = "Tim Janik <timj@gtk.org>";
   es->license = "GNU Lesser General Public License";
 }
@@ -551,7 +821,7 @@ static BseExportNodeProc __enode_remove_part = {
 static void
 __enode_create_track__fill_strings (BseExportStrings *es)
 {
-  es->blurb = "Create a new Track for a Song";
+  es->blurb = _("Create a new Track for a Song.");
   es->authors = "Tim Janik <timj@gtk.org>";
   es->license = "GNU Lesser General Public License";
 }
@@ -568,7 +838,7 @@ static BseExportNodeProc __enode_create_track = {
 static void
 __enode_remove_track__fill_strings (BseExportStrings *es)
 {
-  es->blurb = "Delete a Track from a Song";
+  es->blurb = _("Delete a Track from a Song.");
   es->authors = "Tim Janik <timj@gtk.org>";
   es->license = "GNU Lesser General Public License";
 }
@@ -585,7 +855,7 @@ static BseExportNodeProc __enode_remove_track = {
 static void
 __enode_find_track_for_part__fill_strings (BseExportStrings *es)
 {
-  es->blurb = "Find a track suitable for playing notes of a given part.";
+  es->blurb = _("Find a track suitable for playing notes of a given part.");
   es->authors = "Tim Janik <timj@gtk.org>";
   es->license = "GNU Lesser General Public License";
 }
@@ -600,14 +870,31 @@ static BseExportNodeProc __enode_find_track_for_part = {
   0, find_track_for_part_setup, find_track_for_part_exec, 
 };
 static void
+__enode_find_any_track_for_part__fill_strings (BseExportStrings *es)
+{
+  es->blurb = _("Find the first track that contains part, suitable to check for orphan parts.");
+  es->authors = "Tim Janik <timj@gtk.org>";
+  es->license = "GNU Lesser General Public License";
+}
+static BseExportNodeProc __enode_find_any_track_for_part = {
+  { (BseExportNode*) &__enode_find_track_for_part, BSE_EXPORT_NODE_PROC,
+    "BseSong+find-any-track-for-part", 
+    NULL,
+    "/Methods/BseSong/General/Find Any Track For Part",
+    NULL,
+    __enode_find_any_track_for_part__fill_strings,
+  },
+  0, find_any_track_for_part_setup, find_any_track_for_part_exec, 
+};
+static void
 __enode_synthesize_note__fill_strings (BseExportStrings *es)
 {
-  es->blurb = "Synthesize a note on a song of an active project.";
+  es->blurb = _("Synthesize a note on a song of an active project.");
   es->authors = "Tim Janik <timj@gtk.org>";
   es->license = "GNU Lesser General Public License";
 }
 static BseExportNodeProc __enode_synthesize_note = {
-  { (BseExportNode*) &__enode_find_track_for_part, BSE_EXPORT_NODE_PROC,
+  { (BseExportNode*) &__enode_find_any_track_for_part, BSE_EXPORT_NODE_PROC,
     "BseSong+synthesize-note", 
     NULL,
     "/Methods/BseSong/General/Synthesize Note",
@@ -619,7 +906,7 @@ static BseExportNodeProc __enode_synthesize_note = {
 static void
 __enode_get_timing__fill_strings (BseExportStrings *es)
 {
-  es->blurb = "Retrieve song timing information at a specific tick.";
+  es->blurb = _("Retrieve song timing information at a specific tick.");
   es->authors = "Tim Janik <timj@gtk.org>";
   es->license = "GNU Lesser General Public License";
 }
@@ -639,6 +926,6 @@ BseExportNode* bse__builtin_init_bsesong_gen_proc_c (void)
   return (BseExportNode*) &__enode_get_timing;
 }
 
-/**
- ** Generated data ends here
- **/
+/*
+ * Generated data ends here
+ */
