@@ -45,7 +45,6 @@ struct _BseStorageDBlock
 };
 struct _BseStorageItemLink
 {
-  BseStorageItemLink   *next;
   BseItem              *from_item;
   BseStorageRestoreLink restore_link;
   gpointer              data;
@@ -326,7 +325,7 @@ storage_parse_bse_version (BseStorage *self)
         vminor = strtol (pminor, &ep, 10);
       if (!ep || *ep == 0)
         vmicro = strtol (pmicro, &ep, 10);
-      if ((!ep || *ep == 0) && vmajor >= 0 && vminor >= 0 && vmicro >= 0 &&
+      if ((!ep || *ep == 0 || ep > pmicro) && vmajor >= 0 && vminor >= 0 && vmicro >= 0 &&
           BSE_VERSION_CMP (vmajor, vminor, vmicro, 0, 0, 0) > 0)
         {
           parsed_version = TRUE;
@@ -360,9 +359,7 @@ storage_add_item_link (BseStorage           *self,
                        gchar                *error)
 {
   BseStorageItemLink *ilink = g_new0 (BseStorageItemLink, 1);
-
-  ilink->next = self->item_links;
-  self->item_links = ilink;
+  self->item_links = sfi_ring_append (self->item_links, ilink);
   ilink->from_item = g_object_ref (from_item);
   ilink->restore_link = restore_link;
   ilink->data = data;
@@ -379,9 +376,7 @@ bse_storage_resolve_item_links (BseStorage *self)
 
   while (self->item_links)
     {
-      BseStorageItemLink *ilink = self->item_links;
-
-      self->item_links = ilink->next;
+      BseStorageItemLink *ilink = sfi_ring_pop_head (&self->item_links);
 
       if (ilink->error)
         {
@@ -438,6 +433,15 @@ bse_storage_resolve_item_links (BseStorage *self)
     }
 }
 
+const gchar*
+bse_storage_item_get_compat_type (BseItem *item)
+{
+  const gchar *type = g_object_get_data (item, "BseStorage-compat-type");
+  if (!type)
+    type = G_OBJECT_TYPE_NAME (item);
+  return type;
+}
+
 typedef struct {
   BseContainer *container;
   gchar        *uname;
@@ -624,6 +628,40 @@ restore_item_property (BseItem    *item,
   return G_TOKEN_NONE;
 }
 
+static GTokenType
+restore_source_automation (BseItem    *item,
+                           BseStorage *self)
+{
+  GScanner *scanner = bse_storage_get_scanner (self);
+  /* check identifier */
+  if (g_scanner_peek_next_token (scanner) != G_TOKEN_IDENTIFIER ||
+      !bse_string_equals ("source-automate", scanner->next_value.v_identifier))
+    return SFI_TOKEN_UNMATCHED;
+  /* check object type */
+  if (!BSE_IS_SOURCE (item))
+    return SFI_TOKEN_UNMATCHED;
+  /* eat source-automate */
+  parse_or_return (scanner, G_TOKEN_IDENTIFIER);
+  /* read pspec name */
+  parse_or_return (scanner, G_TOKEN_STRING);
+  /* find pspec */
+  GParamSpec *pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (item), scanner->value.v_string);
+  if (!pspec || !sfi_pspec_check_option (pspec, "automate"))
+    return bse_storage_warn_skip (self, "not an automatable property: \"%s\"", pspec->name);
+  /* parse midi channel */
+  parse_or_return (scanner, G_TOKEN_INT);
+  gint midi_channel = scanner->value.v_int64;
+  /* parse control type */
+  parse_or_return (scanner, G_TOKEN_IDENTIFIER);
+  BseMidiControlType control_type = sfi_choice2enum (scanner->value.v_identifier, BSE_TYPE_MIDI_CONTROL_TYPE);
+  /* close statement */
+  parse_or_return (scanner, ')');
+  BseErrorType error = bse_source_set_automation_property (BSE_SOURCE (item), pspec->name, midi_channel, control_type);
+  if (error)
+    bse_storage_warn (self, "failed to automate property \"%s\": %s", pspec->name, bse_error_blurb (error));
+  return G_TOKEN_NONE;
+}
+
 static GTokenType
 restore_container_child (BseContainer *container,
                          BseStorage   *self)
@@ -632,7 +670,7 @@ restore_container_child (BseContainer *container,
   GTokenType expected_token;
   BseItem *item;
   const gchar *uname;
-  gchar *type_name, *tmp;
+  gchar *type_name, *tmp, *compat_type = NULL;
 
   /* check identifier */
   if (g_scanner_peek_next_token (scanner) != G_TOKEN_IDENTIFIER ||
@@ -652,10 +690,10 @@ restore_container_child (BseContainer *container,
   uname += 2;
 
   /* handle different versions */
-  tmp = bse_compat_rewrite_type_name (self->major_version, self->minor_version, self->micro_version, type_name);
+  tmp = bse_compat_rewrite_type_name (self, type_name);
   if (tmp)
     {
-      g_free (type_name);
+      compat_type = type_name;
       type_name = tmp;
     }
   
@@ -663,6 +701,7 @@ restore_container_child (BseContainer *container,
   if (!bse_container_check_restore (container, type_name))
     {
       g_free (type_name);
+      g_free (compat_type);
       return bse_storage_warn_skip (self, "ignoring child: \"%s\"", scanner->value.v_string);
     }
 
@@ -670,6 +709,10 @@ restore_container_child (BseContainer *container,
   tmp = g_strconcat (type_name, "::", uname, NULL);
   g_free (type_name);
   item = bse_container_retrieve_child (container, tmp);
+  if (item)
+    g_object_set_data_full (item, "BseStorage-compat-type", compat_type, g_free);
+  else
+    g_free (compat_type);
   g_free (tmp);
   if (!item)
     return bse_storage_warn_skip (self, "failed to create object from (invalid?) handle: \"%s\"",
@@ -713,6 +756,9 @@ item_restore_try_statement (gpointer    item,
   if (expected_token == SFI_TOKEN_UNMATCHED)
     expected_token = restore_item_property (item, self);
 
+  if (expected_token == SFI_TOKEN_UNMATCHED)
+    expected_token = restore_source_automation (item, self);
+
   if (expected_token == SFI_TOKEN_UNMATCHED)
     expected_token = BSE_OBJECT_GET_CLASS (item)->restore_private (item, self, scanner);
 
@@ -813,7 +859,7 @@ bse_storage_put_item_link (BseStorage *self,
     {
       bse_storage_puts (self, SFI_SERIAL_NULL_TOKEN);
     }
-  else          /* ordiniary object link within a project or other container */
+  else          /* ordinary object link within a project or other container */
     {
       BseItem *tmp, *common_ancestor;
       guint pbackup = 0;
@@ -1024,6 +1070,22 @@ bse_item_store_property (BseItem    *item,
     bse_storage_put_param (storage, value, pspec);
 }
 
+static void
+bse_source_store_automation (BseSource  *source,
+                             BseStorage *storage,
+                             GParamSpec *pspec)
+{
+  guint midi_channel = 0;
+  BseMidiControlType control_type = 0;
+  bse_source_get_automation_property (source, pspec->name, &midi_channel, &control_type);
+  if (control_type)
+    {
+      bse_storage_break (storage);
+      bse_storage_printf (storage, "(source-automate \"%s\" %u %s)", pspec->name,
+                          midi_channel, sfi_enum2choice (control_type, BSE_TYPE_MIDI_CONTROL_TYPE));
+    }
+}
+
 static void
 store_item_properties (BseItem    *item,
                        BseStorage *storage)
@@ -1047,6 +1109,8 @@ store_item_properties (BseItem    *item,
               !sfi_pspec_check_option (pspec, "skip-default"))
             bse_item_store_property (item, storage, &value, pspec);
           g_value_unset (&value);
+          if (sfi_pspec_check_option (pspec, "automate") && BSE_IS_SOURCE (item))
+            bse_source_store_automation (BSE_SOURCE (item), storage, pspec);
         }
     }
   g_free (pspecs);
