@@ -76,7 +76,7 @@ struct _BseStorage
   guint               minor_version;
   guint               micro_version;
   GHashTable         *path_table;
-  BseStorageItemLink *item_links;
+  SfiRing            *item_links;
   /* internal data */
   guint               n_dblocks;
   BseStorageDBlock   *dblocks;
@@ -119,6 +119,7 @@ void         bse_storage_store_item             (BseStorage             *self,
                                                  gpointer                item);
 void         bse_storage_store_child            (BseStorage             *self,
                                                  gpointer                item);
+const gchar* bse_storage_item_get_compat_type   (BseItem                *item);
 
 
 /* --- writing --- */
