@@ -20,12 +20,9 @@
 #define __BSE_SUB_OPORT_H__
 
 #include <bse/bsesource.h>
-#include <bse/gslengine.h>
-
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+#include <bse/bseengine.h>
 
+G_BEGIN_DECLS
 
 /* --- object type macros --- */
 #define BSE_TYPE_SUB_OPORT		(BSE_TYPE_ID (BseSubOPort))
@@ -43,20 +40,13 @@ typedef struct _BseSubOPortClass BseSubOPortClass;
 struct _BseSubOPort
 {
   BseSource	 parent_object;
-  
   gchar	       **output_ports;
 };
 struct _BseSubOPortClass
 {
   BseSourceClass parent_class;
-
-  guint          n_output_ports;
-  GslClass       gsl_class;
 };
 
-
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* __BSE_SUB_OPORT_H__ */
