@@ -22,13 +22,13 @@
 #include "bsecsynth.h"
 #include "bsewave.h"
 #include "bsepart.h"
+#include "bsebus.h"
 #include "bsemain.h"
 #include "gslcommon.h"
 #include "bsesubsynth.h"
 #include "bseproject.h"
 #include "bsesong.h"
 #include "bsemidivoice.h"
-#include "bsecontextmerger.h"
 #include "bsemidireceiver.h"
 #include "bsewaverepo.h"
 #include <string.h>
@@ -43,11 +43,12 @@
 enum {
   PROP_0,
   PROP_MUTED,
-  PROP_SYNTH_NET,
+  PROP_SNET,
   PROP_WAVE,
   PROP_MIDI_CHANNEL,
   PROP_N_VOICES,
-  PROP_POST_NET
+  PROP_PNET,
+  PROP_OUTPUTS
 };
 
 /* --- prototypes --- */
@@ -63,9 +64,6 @@ static void         bse_track_get_property        (GObject       *object,
                                                    guint          param_id,
                                                    GValue        *value,
                                                    GParamSpec    *pspec);
-static BseItemSeq*  bse_track_list_items          (BseItem       *item,
-                                                   guint          param_id,
-                                                   GParamSpec    *pspec);
 static void         bse_track_store_private       (BseObject     *object,
                                                    BseStorage    *storage);
 static SfiTokenType bse_track_restore_private     (BseObject     *object,
@@ -96,65 +94,12 @@ BSE_BUILTIN_TYPE (BseTrack)
     (GInstanceInitFunc) bse_track_init,
   };
   
-  return bse_type_register_static (BSE_TYPE_ITEM,
+  return bse_type_register_static (BSE_TYPE_CONTEXT_MERGER,
 				   "BseTrack",
 				   "BSE track type",
 				   &track_info);
 }
 
-static void
-bse_track_class_init (BseTrackClass *class)
-{
-  GObjectClass *gobject_class = G_OBJECT_CLASS (class);
-  BseObjectClass *object_class = BSE_OBJECT_CLASS (class);
-  BseItemClass *item_class = BSE_ITEM_CLASS (class);
-  
-  parent_class = g_type_class_peek_parent (class);
-  
-  gobject_class->set_property = bse_track_set_property;
-  gobject_class->get_property = bse_track_get_property;
-  gobject_class->dispose = bse_track_dispose;
-  gobject_class->finalize = bse_track_finalize;
-
-  object_class->store_private = bse_track_store_private;
-  object_class->restore_private = bse_track_restore_private;
-  
-  item_class->list_items = bse_track_list_items;
-  
-  bse_object_class_add_param (object_class, "Adjustments",
-			      PROP_MUTED,
-			      sfi_pspec_bool ("muted", "Muted", NULL,
-					      FALSE, SFI_PARAM_STANDARD ":skip-default"));
-  bse_object_class_add_param (object_class, "Synth Input",
-			      PROP_SYNTH_NET,
-			      bse_param_spec_object ("snet", "Custom Synth Net", "Synthesis network to be used as instrument",
-						     BSE_TYPE_CSYNTH,
-						     SFI_PARAM_STANDARD));
-  bse_object_class_add_param (object_class, "Synth Input",
-			      PROP_WAVE,
-			      bse_param_spec_object ("wave", "Custom Wave", "Wave to be used as instrument",
-						     BSE_TYPE_WAVE,
-						     SFI_PARAM_STANDARD));
-  bse_object_class_add_param (object_class, "Synth Input",
-			      PROP_N_VOICES,
-			      sfi_pspec_int ("n_voices", "Max Voixes", "Maximum number of voices for simultaneous playback",
-					     16, 1, 256, 1,
-					     SFI_PARAM_GUI SFI_PARAM_STORAGE ":scale"));
-  bse_object_class_add_param (object_class, "MIDI Instrument",
-                              PROP_MIDI_CHANNEL,
-                              sfi_pspec_int ("midi_channel", "MIDI Channel",
-                                             "Midi channel assigned to this track, 0 uses private per-track channel",
-                                             0, 0, BSE_MIDI_MAX_CHANNELS, 1,
-                                             SFI_PARAM_GUI SFI_PARAM_STORAGE ":scale:skip-default"));
-  bse_object_class_add_param (object_class, "Synth Postprocess",
-			      PROP_POST_NET,
-			      bse_param_spec_object ("pnet", "Custom Postprocess Net",
-                                                     "Synthesis network to postprocess track sound",
-						     BSE_TYPE_CSYNTH,
-						     SFI_PARAM_STANDARD));
-  signal_changed = bse_object_class_add_asignal (object_class, "changed", G_TYPE_NONE, 0);
-}
-
 static gulong
 alloc_id_above (guint n)
 {
@@ -170,6 +115,7 @@ alloc_id_above (guint n)
 static void
 bse_track_init (BseTrack *self)
 {
+  BSE_OBJECT_SET_FLAGS (self, BSE_SOURCE_FLAG_PRIVATE_INPUTS);
   self->snet = NULL;
   self->pnet = NULL;
   self->max_voices = 16;
@@ -198,6 +144,8 @@ bse_track_dispose (GObject *object)
   
   /* chain parent class' handler */
   G_OBJECT_CLASS (parent_class)->dispose (object);
+
+  g_assert (self->bus_outputs == NULL);
 }
 
 static void
@@ -205,6 +153,8 @@ bse_track_finalize (GObject *object)
 {
   BseTrack *self = BSE_TRACK (object);
 
+  g_assert (self->bus_outputs == NULL);
+
   g_assert (self->n_entries_SL == 0);
   g_free (self->entries_SL);
   bse_id_free (self->channel_id);
@@ -317,35 +267,45 @@ track_uncross_part (BseItem *owner,
       }
 }
 
-static BseItemSeq*
-bse_track_list_items (BseItem    *item,
-                      guint       param_id,
-                      GParamSpec *pspec)
+static void
+bse_track_get_candidates (BseItem               *item,
+                          guint                  param_id,
+                          BsePropertyCandidates *pc,
+                          GParamSpec            *pspec)
 {
   BseTrack *self = BSE_TRACK (item);
-  BseItemSeq *iseq = bse_item_seq_new ();
   switch (param_id)
     {
       BseProject *project;
-    case PROP_SYNTH_NET:
-      bse_item_gather_items_typed (item, iseq, BSE_TYPE_CSYNTH, BSE_TYPE_PROJECT, FALSE);
-      break;
-    case PROP_POST_NET:
-      bse_item_gather_items_typed (item, iseq, BSE_TYPE_CSYNTH, BSE_TYPE_PROJECT, FALSE);
-      break;
+      SfiRing *ring;
     case PROP_WAVE:
+      bse_property_candidate_relabel (pc, _("Available Waves"), _("List of available waves to choose as track instrument"));
       project = bse_item_get_project (item);
       if (project)
 	{
 	  BseWaveRepo *wrepo = bse_project_get_wave_repo (project);
-	  bse_item_gather_items_typed (BSE_ITEM (wrepo), iseq, BSE_TYPE_WAVE, BSE_TYPE_WAVE_REPO, FALSE);
+	  bse_item_gather_items_typed (BSE_ITEM (wrepo), pc->items, BSE_TYPE_WAVE, BSE_TYPE_WAVE_REPO, FALSE);
 	}
       break;
+    case PROP_SNET:
+      bse_property_candidate_relabel (pc, _("Available Synthesizers"), _("List of available synthesis networks to choose a track instrument from"));
+      bse_item_gather_items_typed (item, pc->items, BSE_TYPE_CSYNTH, BSE_TYPE_PROJECT, FALSE);
+      break;
+    case PROP_PNET:
+      bse_property_candidate_relabel (pc, _("Available Postprocessors"), _("List of available synthesis networks to choose a postprocessor from"));
+      bse_item_gather_items_typed (item, pc->items, BSE_TYPE_CSYNTH, BSE_TYPE_PROJECT, FALSE);
+      break;
+    case PROP_OUTPUTS:
+      bse_property_candidate_relabel (pc, _("Available Outputs"), _("List of available mixer busses to be used as track output"));
+      bse_bus_or_track_list_output_candidates (BSE_ITEM (self), pc->items);
+      /* remove existing outputs */
+      for (ring = self->bus_outputs; ring; ring = sfi_ring_walk (ring, self->bus_outputs))
+        bse_item_seq_remove (pc->items, ring->data);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (self, param_id, pspec);
       break;
     }
-  return iseq;
 }
 
 static void
@@ -438,8 +398,8 @@ bse_track_set_property (GObject      *object,
       self->muted_SL = sfi_value_get_bool (value);
       BSE_SEQUENCER_UNLOCK ();
       break;
-    case PROP_SYNTH_NET:
-      if (!self->sub_synth || !BSE_SOURCE_PREPARED (self->sub_synth))
+    case PROP_SNET:
+      if (!self->sub_synth || !BSE_SOURCE_PREPARED (self))
 	{
 	  BseSNet *snet = bse_value_get_object (value);
 	  if (snet || self->snet)
@@ -483,15 +443,14 @@ bse_track_set_property (GObject      *object,
       break;
     case PROP_MIDI_CHANNEL:
       i = sfi_value_get_int (value);
-      if (i != self->midi_channel_SL &&
-          (!self->postprocess || !BSE_SOURCE_PREPARED (self->postprocess)))
+      if (i != self->midi_channel_SL && !BSE_SOURCE_PREPARED (self))
         {
           self->midi_channel_SL = i > 0 ? i : self->channel_id;
           bse_track_update_midi_channel (self);
         }
       break;
-    case PROP_POST_NET:
-      if (!self->postprocess || !BSE_SOURCE_PREPARED (self->postprocess))
+    case PROP_PNET:
+      if (!self->postprocess || !BSE_SOURCE_PREPARED (self))
 	{
           if (self->pnet)
             {
@@ -511,6 +470,9 @@ bse_track_set_property (GObject      *object,
                           NULL);
 	}
       break;
+    case PROP_OUTPUTS:
+      bse_bus_or_track_set_outputs (BSE_ITEM (self), g_value_get_boxed (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (self, param_id, pspec);
       break;
@@ -527,15 +489,23 @@ bse_track_get_property (GObject    *object,
   
   switch (param_id)
     {
+      BseItemSeq *iseq;
+      SfiRing *ring;
     case PROP_MUTED:
       sfi_value_set_bool (value, self->muted_SL);
       break;
-    case PROP_SYNTH_NET:
+    case PROP_SNET:
       bse_value_set_object (value, self->snet);
       break;
-    case PROP_POST_NET:
+    case PROP_PNET:
       bse_value_set_object (value, self->pnet);
       break;
+    case PROP_OUTPUTS:
+      iseq = bse_item_seq_new();
+      for (ring = self->bus_outputs; ring; ring = sfi_ring_walk (ring, self->bus_outputs))
+        bse_item_seq_append (iseq, ring->data);
+      g_value_take_boxed (value, iseq);
+      break;
     case PROP_WAVE:
       bse_value_set_object (value, self->wave);
       break;
@@ -705,12 +675,10 @@ bse_track_get_part_SL (BseTrack *self,
 void
 bse_track_add_modules (BseTrack        *self,
 		       BseContainer    *container,
-                       BseMidiReceiver *midi_receiver,
-		       BseSource       *merger)
+                       BseMidiReceiver *midi_receiver)
 {
   g_return_if_fail (BSE_IS_TRACK (self));
   g_return_if_fail (BSE_IS_CONTAINER (container));
-  g_return_if_fail (BSE_IS_CONTEXT_MERGER (merger));
   g_return_if_fail (self->sub_synth == NULL);
   g_return_if_fail (midi_receiver != NULL);
 
@@ -719,17 +687,17 @@ bse_track_add_modules (BseTrack        *self,
   bse_item_set_internal (self->voice_input, TRUE);
   
   /* sub synth */
-  self->sub_synth = bse_container_new_child (container, BSE_TYPE_SUB_SYNTH,
-                                             "in_port_1", "frequency",
-                                             "in_port_2", "gate",
-                                             "in_port_3", "velocity",
-                                             "in_port_4", "aftertouch",
-                                             "out_port_1", "left-audio",
-                                             "out_port_2", "right-audio",
-                                             "out_port_3", "unused",
-                                             "out_port_4", "synth-done",
-                                             "snet", self->snet,
-                                             NULL);
+  self->sub_synth = bse_container_new_child_bname (container, BSE_TYPE_SUB_SYNTH, "Track-Instrument",
+                                                   "in_port_1", "frequency",
+                                                   "in_port_2", "gate",
+                                                   "in_port_3", "velocity",
+                                                   "in_port_4", "aftertouch",
+                                                   "out_port_1", "left-audio",
+                                                   "out_port_2", "right-audio",
+                                                   "out_port_3", "unused",
+                                                   "out_port_4", "synth-done",
+                                                   "snet", self->snet,
+                                                   NULL);
   bse_item_set_internal (self->sub_synth, TRUE);
   
   /* voice input <-> sub-synth */
@@ -755,39 +723,61 @@ bse_track_add_modules (BseTrack        *self,
   bse_source_must_set_input (self->voice_switch, BSE_MIDI_VOICE_SWITCH_ICHANNEL_DISCONNECT,
 			     self->sub_synth, 3);
   
-  /* context merger */
-  self->context_merger = bse_container_new_child (container, BSE_TYPE_CONTEXT_MERGER, NULL);
-  bse_item_set_internal (self->context_merger, TRUE);
-  
   /* midi voice switch <-> context merger */
-  bse_source_must_set_input (self->context_merger, 0,
+  bse_source_must_set_input (BSE_SOURCE (self), 0,
 			     self->voice_switch, BSE_MIDI_VOICE_SWITCH_OCHANNEL_LEFT);
-  bse_source_must_set_input (self->context_merger, 1,
+  bse_source_must_set_input (BSE_SOURCE (self), 1,
 			     self->voice_switch, BSE_MIDI_VOICE_SWITCH_OCHANNEL_RIGHT);
   
   /* postprocess */
-  self->postprocess = bse_container_new_child (container, BSE_TYPE_SUB_SYNTH, NULL);
+  self->postprocess = bse_container_new_child_bname (container, BSE_TYPE_SUB_SYNTH, "Track-Postprocess", NULL);
   bse_item_set_internal (self->postprocess, TRUE);
   bse_sub_synth_set_null_shortcut (BSE_SUB_SYNTH (self->postprocess), TRUE);
   
   /* context merger <-> postprocess */
-  bse_source_must_set_input (self->postprocess, 0, self->context_merger, 0);
-  bse_source_must_set_input (self->postprocess, 1, self->context_merger, 1);
-
-  /* postprocess <-> container's merger */
-  bse_source_must_set_input (merger, 0, self->postprocess, 0);
-  bse_source_must_set_input (merger, 1, self->postprocess, 1);
+  bse_source_must_set_input (self->postprocess, 0, BSE_SOURCE (self), 0);
+  bse_source_must_set_input (self->postprocess, 1, BSE_SOURCE (self), 1);
 
   /* propagate midi channel to modules */
   bse_track_update_midi_channel (self);
 }
 
-void
-bse_track_connect_sniffer (BseTrack               *self,
-                           BseSource              *source)
+BseSource*
+bse_track_get_output (BseTrack *self)
 {
-  bse_source_must_set_input (source, 0, self->postprocess, 0);
-  bse_source_must_set_input (source, 1, self->postprocess, 1);
+  return self->postprocess;
+}
+
+guint
+bse_track_get_last_tick (BseTrack *self)
+{
+  int last_tick = 0;
+
+  /* find last part */
+  BsePart *part = NULL;
+  guint i, offset = 0;
+  for (i = 0; i < self->n_entries_SL; i++)
+    if (self->entries_SL[i].part)
+      {
+        part = self->entries_SL[i].part;
+        offset = self->entries_SL[i].tick;
+      }
+  if (part)
+    {
+      BseItem *item = BSE_ITEM (self);
+      BseSongTiming timing;
+      g_object_get (part, "last-tick", &last_tick, NULL);
+      if (BSE_IS_SONG (item->parent))
+        bse_song_get_timing (BSE_SONG (item->parent), offset, &timing);
+      else
+        bse_song_timing_get_default (&timing);
+      last_tick = MAX (last_tick, timing.tpt);  /* MAX duration with tact size, see bse_track_list_parts() */
+      last_tick += offset;
+    }
+  else
+    last_tick += 1;     /* always return one after */
+  
+  return last_tick;
 }
 
 static void
@@ -801,6 +791,32 @@ bse_track_update_midi_channel (BseTrack *self)
     }
 }
 
+static void
+bse_track_context_create (BseSource      *source,
+                          guint           context_handle,
+                          BseTrans       *trans)
+{
+  BseTrack *self = BSE_TRACK (source);
+  BseMidiContext mcontext = bse_snet_get_midi_context (bse_item_get_snet (BSE_ITEM (self)), context_handle);
+  if (self->snet || self->wave)
+    bse_midi_receiver_channel_enable_poly (mcontext.midi_receiver, self->midi_channel_SL);
+  /* chain parent class' handler */
+  BSE_SOURCE_CLASS (parent_class)->context_create (source, context_handle, trans);
+}
+
+static void
+bse_track_context_dismiss (BseSource      *source,
+                           guint           context_handle,
+                           BseTrans       *trans)
+{
+  BseTrack *self = BSE_TRACK (source);
+  BseMidiContext mcontext = bse_snet_get_midi_context (bse_item_get_snet (BSE_ITEM (self)), context_handle);
+  if (self->snet || self->wave)
+    bse_midi_receiver_channel_disable_poly (mcontext.midi_receiver, self->midi_channel_SL);
+  /* chain parent class' handler */
+  BSE_SOURCE_CLASS (parent_class)->context_dismiss (source, context_handle, trans);
+}
+
 void
 bse_track_remove_modules (BseTrack     *self,
 			  BseContainer *container)
@@ -815,8 +831,6 @@ bse_track_remove_modules (BseTrack     *self,
   self->voice_input = NULL;
   bse_container_remove_item (container, BSE_ITEM (self->voice_switch));
   self->voice_switch = NULL;
-  bse_container_remove_item (container, BSE_ITEM (self->context_merger));
-  self->context_merger = NULL;
   bse_container_remove_item (container, BSE_ITEM (self->postprocess));
   self->postprocess = NULL;
 }
@@ -826,7 +840,7 @@ bse_track_clone_voices (BseTrack       *self,
 			BseSNet        *snet,
 			guint           context,
                         BseMidiContext  mcontext,
-			GslTrans       *trans)
+			BseTrans       *trans)
 {
   guint i;
   
@@ -835,7 +849,7 @@ bse_track_clone_voices (BseTrack       *self,
   g_return_if_fail (trans != NULL);
 
   for (i = 0; i < self->max_voices - 1; i++)
-    bse_snet_context_clone_branch (snet, context, self->context_merger, mcontext, trans);
+    bse_snet_context_clone_branch (snet, context, BSE_SOURCE (self), mcontext, trans);
 }
 
 static void
@@ -914,3 +928,65 @@ bse_track_restore_private (BseObject  *object,
   else /* chain parent class' handler */
     return BSE_OBJECT_CLASS (parent_class)->restore_private (object, storage, scanner);
 }
+
+static void
+bse_track_class_init (BseTrackClass *class)
+{
+  GObjectClass *gobject_class = G_OBJECT_CLASS (class);
+  BseObjectClass *object_class = BSE_OBJECT_CLASS (class);
+  BseItemClass *item_class = BSE_ITEM_CLASS (class);
+  BseSourceClass *source_class = BSE_SOURCE_CLASS (class);
+  
+  parent_class = g_type_class_peek_parent (class);
+  
+  gobject_class->set_property = bse_track_set_property;
+  gobject_class->get_property = bse_track_get_property;
+  gobject_class->dispose = bse_track_dispose;
+  gobject_class->finalize = bse_track_finalize;
+
+  object_class->store_private = bse_track_store_private;
+  object_class->restore_private = bse_track_restore_private;
+  
+  item_class->get_candidates = bse_track_get_candidates;
+  
+  source_class->context_create = bse_track_context_create;
+  source_class->context_dismiss = bse_track_context_dismiss;
+
+  bse_source_class_inherit_channels (BSE_SOURCE_CLASS (class));
+
+  bse_object_class_add_param (object_class, _("Adjustments"),
+			      PROP_MUTED,
+			      sfi_pspec_bool ("muted", _("Muted"), NULL,
+					      FALSE, SFI_PARAM_STANDARD ":skip-default"));
+  bse_object_class_add_param (object_class, _("Synth Input"),
+			      PROP_SNET,
+			      bse_param_spec_object ("snet", _("Synthesizer"), _("Synthesis network to be used as instrument"),
+						     BSE_TYPE_CSYNTH,
+						     SFI_PARAM_STANDARD ":unprepared"));
+  bse_object_class_add_param (object_class, _("Synth Input"),
+			      PROP_WAVE,
+			      bse_param_spec_object ("wave", _("Wave"), _("Wave to be used as instrument"),
+						     BSE_TYPE_WAVE,
+						     SFI_PARAM_STANDARD ":unprepared"));
+  bse_object_class_add_param (object_class, _("Synth Input"),
+			      PROP_N_VOICES,
+			      sfi_pspec_int ("n_voices", _("Max Voixes"), _("Maximum number of voices for simultaneous playback"),
+					     16, 1, 256, 1,
+					     SFI_PARAM_GUI SFI_PARAM_STORAGE ":scale:unprepared"));
+  bse_object_class_add_param (object_class, _("MIDI Instrument"),
+                              PROP_MIDI_CHANNEL,
+                              sfi_pspec_int ("midi_channel", _("MIDI Channel"),
+                                             _("Midi channel assigned to this track, 0 uses private per-track channel"),
+                                             0, 0, BSE_MIDI_MAX_CHANNELS, 1,
+                                             SFI_PARAM_GUI SFI_PARAM_STORAGE ":scale:skip-default:unprepared"));
+  bse_object_class_add_param (object_class, _("MIDI Instrument"),
+			      PROP_PNET,
+			      bse_param_spec_object ("pnet", _("Postprocessor"), _("Synthesis network to be used as postprocessor"),
+						     BSE_TYPE_CSYNTH, SFI_PARAM_STANDARD ":unprepared"));
+  bse_object_class_add_param (object_class, _("Signal Outputs"),
+                              PROP_OUTPUTS,
+                              bse_param_spec_boxed ("outputs", _("Output Signals"),
+                                                    _("Mixer busses used as output for this track"),
+                                                    BSE_TYPE_ITEM_SEQ, SFI_PARAM_GUI ":item-sequence"));
+  signal_changed = bse_object_class_add_asignal (object_class, "changed", G_TYPE_NONE, 0);
+}
