@@ -18,8 +18,9 @@
 #ifndef __BSE_TRACK_H__
 #define __BSE_TRACK_H__
 
-#include	<bse/bseitem.h>
-#include	<bse/bsesnet.h>
+#include <bse/bseitem.h>
+#include <bse/bsesnet.h>
+#include <bse/bsecontextmerger.h>
 
 G_BEGIN_DECLS
 
@@ -40,7 +41,7 @@ typedef struct {
 } BseTrackEntry;
 struct _BseTrack
 {
-  BseItem	   parent_instance;
+  BseContextMerger parent_instance;
 
   guint            channel_id;
   guint		   max_voices;
@@ -55,9 +56,10 @@ struct _BseTrack
   BseSource       *sub_synth;
   BseSource       *voice_input;
   BseSource       *voice_switch;
-  BseSource       *context_merger;
   BseSource       *postprocess;
 
+  SfiRing         *bus_outputs; /* maintained by bsebus.[hc] */
+
   /* fields protected by sequencer mutex */
   guint		   n_entries_SL : 30;
   guint		   muted_SL : 1;
@@ -67,24 +69,23 @@ struct _BseTrack
 };
 struct _BseTrackClass
 {
-  BseItemClass parent_class;
+  BseContextMergerClass parent_class;
 };
 
 
 /* --- prototypes -- */
 void	bse_track_add_modules		(BseTrack		*self,
 					 BseContainer		*container,
-                                         BseMidiReceiver        *midi_receiver,
-					 BseSource		*merger);
-void    bse_track_connect_sniffer       (BseTrack               *self,
-                                         BseSource              *source);
+                                         BseMidiReceiver        *midi_receiver);
 void	bse_track_remove_modules	(BseTrack		*self,
 					 BseContainer		*container);
 void	bse_track_clone_voices		(BseTrack		*self,
 					 BseSNet		*snet,
 					 guint			 context,
                                          BseMidiContext          mcontext,
-					 GslTrans		*trans);
+					 BseTrans		*trans);
+BseSource*       bse_track_get_output   (BseTrack               *self);
+guint        	 bse_track_get_last_tick(BseTrack		*self);
 guint        	 bse_track_insert_part	(BseTrack		*self,
 					 guint			 tick,
 					 BsePart		*part);
