@@ -21,7 +21,7 @@
 #include <bse/bseeditablesample.h>
 #include <bse/bsewaverepo.h>
 #include <bse/bseproject.h>
-#include <bse/gslengine.h>
+#include <bse/bseengine.h>
 #include <bse/gslwavechunk.h>
 #include <bse/gslfilter.h>
 
@@ -32,6 +32,7 @@ enum
 {
   PARAM_0,
   PARAM_WAVE,
+  PARAM_CHANNEL,
   PARAM_FM_PERC,
   PARAM_FM_EXP,
   PARAM_FM_OCTAVES
@@ -51,12 +52,9 @@ static void     bse_wave_osc_get_property       (GObject                *object,
                                                  guint                   param_id,
                                                  GValue                 *value,
                                                  GParamSpec             *pspec);
-static BseItemSeq* bse_wave_osc_list_items      (BseItem                *item,
-                                                 guint                   param_id,
-                                                 GParamSpec             *pspec);
 static void     bse_wave_osc_context_create     (BseSource              *source,
                                                  guint                   context_handle,
-                                                 GslTrans               *trans);
+                                                 BseTrans               *trans);
 static void   bse_wave_osc_update_config_wchunk (BseWaveOsc             *self);
 static void     bse_wave_osc_update_modules     (BseWaveOsc             *self);
 
@@ -98,66 +96,6 @@ BSE_BUILTIN_TYPE (BseWaveOsc)
   return type;
 }
 
-static void
-bse_wave_osc_class_init (BseWaveOscClass *class)
-{
-  GObjectClass *gobject_class = G_OBJECT_CLASS (class);
-  BseObjectClass *object_class = BSE_OBJECT_CLASS (class);
-  BseItemClass *item_class = BSE_ITEM_CLASS (class);
-  BseSourceClass *source_class = BSE_SOURCE_CLASS (class);
-  guint ochannel, ichannel;
-  
-  parent_class = g_type_class_peek_parent (class);
-  
-  gobject_class->set_property = bse_wave_osc_set_property;
-  gobject_class->get_property = bse_wave_osc_get_property;
-  gobject_class->finalize = bse_wave_osc_finalize;
-  gobject_class->dispose = bse_wave_osc_dispose;
-  
-  item_class->list_items = bse_wave_osc_list_items;
-  
-  source_class->context_create = bse_wave_osc_context_create;
-  
-  bse_object_class_add_param (object_class, _("Wave"),
-                              PARAM_WAVE,
-                              bse_param_spec_object ("wave", _("Wave"), _("Wave to play"),
-                                                     BSE_TYPE_WAVE, SFI_PARAM_STANDARD));
-  bse_object_class_add_param (object_class, _("Modulation"),
-                              PARAM_FM_PERC,
-                              sfi_pspec_real ("fm_perc", _("Input Modulation [%]"),
-                                              _("Modulation Strength for linear frequency modulation"),
-                                              10.0, 0, 100.0,5.0,
-                                              SFI_PARAM_STANDARD ":scale"));
-  bse_object_class_add_param (object_class, _("Modulation"),
-                              PARAM_FM_EXP,
-                              sfi_pspec_bool ("exponential_fm", _("Exponential FM"),
-                                              _("Perform exponential frequency modulation "
-                                                "instead of linear"),
-                                              FALSE, SFI_PARAM_STANDARD));
-  bse_object_class_add_param (object_class, _("Modulation"),
-                              PARAM_FM_OCTAVES,
-                              sfi_pspec_real ("fm_n_octaves", _("Octaves"),
-                                              _("Number of octaves to be affected by exponential frequency modulation"),
-                                              1.0, 0, 3.0, 0.01,
-                                              SFI_PARAM_STANDARD ":scale"));
-  
-  signal_notify_pcm_position = bse_object_class_add_signal (object_class, "notify_pcm_position",
-                                                            G_TYPE_NONE, 2,
-                                                            SFI_TYPE_NUM,
-                                                            G_TYPE_INT);
-  
-  ichannel = bse_source_class_add_ichannel (source_class, "freq-in", _("Freq In"), _("Frequency Input"));
-  g_assert (ichannel == BSE_WAVE_OSC_ICHANNEL_FREQ);
-  ichannel = bse_source_class_add_ichannel (source_class, "sync-in", _("Sync In"), _("Syncronization Input"));
-  g_assert (ichannel == BSE_WAVE_OSC_ICHANNEL_SYNC);
-  ichannel = bse_source_class_add_ichannel (source_class, "mod-in", _("Mod In"), _("Modulation Input"));
-  g_assert (ichannel == BSE_WAVE_OSC_ICHANNEL_MOD);
-  ochannel = bse_source_class_add_ochannel (source_class, "audio-out", _("Audio Out"), _("Wave Output"));
-  g_assert (ochannel == BSE_WAVE_OSC_OCHANNEL_WAVE);
-  ochannel = bse_source_class_add_ochannel (source_class, "gate-out", _("Gate Out"), _("Gate Output"));
-  g_assert (ochannel == BSE_WAVE_OSC_OCHANNEL_GATE);
-}
-
 static void
 bse_wave_osc_init (BseWaveOsc *self)
 {
@@ -167,6 +105,7 @@ bse_wave_osc_init (BseWaveOsc *self)
   self->n_octaves = 1;
   self->config.start_offset = 0;
   self->config.play_dir = +1;
+  self->config.channel = 0;
   self->config.wchunk_data = NULL;
   self->config.wchunk_from_freq = NULL;
   self->config.fm_strength = self->fm_strength / 100.0;
@@ -174,30 +113,29 @@ bse_wave_osc_init (BseWaveOsc *self)
   self->config.cfreq = 440.;
 }
 
-static BseItemSeq*
-bse_wave_osc_list_items (BseItem    *item,
-                         guint       param_id,
-                         GParamSpec *pspec)
+static void
+bse_wave_osc_get_candidates (BseItem               *item,
+                             guint                  param_id,
+                             BsePropertyCandidates *pc,
+                             GParamSpec            *pspec)
 {
   BseWaveOsc *self = BSE_WAVE_OSC (item);
-  BseItemSeq *iseq = bse_item_seq_new ();
   switch (param_id)
     {
       BseProject *project;
     case PARAM_WAVE:
+      bse_property_candidate_relabel (pc, _("Available Waves"), _("List of available waves to choose as oscillator source"));
       project = bse_item_get_project (item);
       if (project)
         {
           BseWaveRepo *wrepo = bse_project_get_wave_repo (project);
-          
-          bse_item_gather_items_typed (BSE_ITEM (wrepo), iseq, BSE_TYPE_WAVE, BSE_TYPE_WAVE_REPO, FALSE);
+          bse_item_gather_items_typed (BSE_ITEM (wrepo), pc->items, BSE_TYPE_WAVE, BSE_TYPE_WAVE_REPO, FALSE);
         }
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (self, param_id, pspec);
       break;
     }
-  return iseq;
 }
 
 static void
@@ -225,7 +163,7 @@ clear_wave_and_esample (BseWaveOsc *self)
            * before we continue (since bse_wave_drop_index() will destroy
            * the wchunk index)
            */
-          gsl_engine_wait_on_trans ();
+          bse_engine_wait_on_trans ();
         }
       bse_wave_drop_index (wave);
       g_object_notify (self, "wave");
@@ -242,7 +180,7 @@ clear_wave_and_esample (BseWaveOsc *self)
           /* need to make sure our modules know about the wave chunk vanishing
            * before we return (so the wchunk update propagates)
            */
-          gsl_engine_wait_on_trans ();
+          bse_engine_wait_on_trans ();
         }
       gsl_wave_chunk_close (esample_wchunk);
     }
@@ -265,7 +203,7 @@ bse_wave_osc_set_from_esample (BseWaveOsc        *self,
           bse_wave_osc_update_config_wchunk (self);
           bse_wave_osc_update_modules (self);
           if (BSE_SOURCE_PREPARED (self))
-            gsl_engine_wait_on_trans ();
+            bse_engine_wait_on_trans ();
         }
     }
 }
@@ -299,11 +237,15 @@ bse_wave_osc_set_property (GObject      *object,
                   /* need to make sure our modules know about BseWave vanishing
                    * before we return (so the wchunk update propagates)
                    */
-                  gsl_engine_wait_on_trans ();
+                  bse_engine_wait_on_trans ();
                 }
             }
         }
       break;
+    case PARAM_CHANNEL:
+      self->config.channel = g_value_get_int (value) - 1;
+      bse_wave_osc_update_modules (self);
+      break;
     case PARAM_FM_PERC:
       self->fm_strength = sfi_value_get_real (value);
       if (!self->config.exponential_fm)
@@ -347,6 +289,9 @@ bse_wave_osc_get_property (GObject    *object,
     case PARAM_WAVE:
       bse_value_set_object (value, self->wave);
       break;
+    case PARAM_CHANNEL:
+      g_value_set_int (value, self->config.channel + 1);
+      break;
     case PARAM_FM_PERC:
       sfi_value_set_real (value, self->fm_strength);
       break;
@@ -387,7 +332,7 @@ bse_wave_osc_finalize (GObject *object)
 }
 
 static void
-wosc_access (GslModule *module,
+wosc_access (BseModule *module,
              gpointer   data)
 {
   GslWaveOscData *wosc = module->user_data;
@@ -437,7 +382,7 @@ bse_wave_osc_update_modules (BseWaveOsc *self)
 
 static void
 wosc_free (gpointer        data,
-           const GslClass *klass)
+           const BseModuleClass *klass)
 {
   GslWaveOscData *wosc = data;
   
@@ -446,7 +391,7 @@ wosc_free (gpointer        data,
 }
 
 static void
-wosc_process (GslModule *module,
+wosc_process (BseModule *module,
               guint      n_values)
 {
   GslWaveOscData *wosc = module->user_data;
@@ -454,20 +399,20 @@ wosc_process (GslModule *module,
   
   gsl_wave_osc_process (wosc,
                         n_values,
-                        (GSL_MODULE_ISTREAM (module, BSE_WAVE_OSC_ICHANNEL_FREQ).connected ?
-                         GSL_MODULE_IBUFFER (module, BSE_WAVE_OSC_ICHANNEL_FREQ) : NULL),
-                        (GSL_MODULE_ISTREAM (module, BSE_WAVE_OSC_ICHANNEL_MOD).connected ?
-                         GSL_MODULE_IBUFFER (module, BSE_WAVE_OSC_ICHANNEL_MOD) : NULL),
-                        (GSL_MODULE_ISTREAM (module, BSE_WAVE_OSC_ICHANNEL_SYNC).connected ?
-                         GSL_MODULE_IBUFFER (module, BSE_WAVE_OSC_ICHANNEL_SYNC) : NULL),
-                        GSL_MODULE_OBUFFER (module, BSE_WAVE_OSC_OCHANNEL_WAVE));
+                        (BSE_MODULE_ISTREAM (module, BSE_WAVE_OSC_ICHANNEL_FREQ).connected ?
+                         BSE_MODULE_IBUFFER (module, BSE_WAVE_OSC_ICHANNEL_FREQ) : NULL),
+                        (BSE_MODULE_ISTREAM (module, BSE_WAVE_OSC_ICHANNEL_MOD).connected ?
+                         BSE_MODULE_IBUFFER (module, BSE_WAVE_OSC_ICHANNEL_MOD) : NULL),
+                        (BSE_MODULE_ISTREAM (module, BSE_WAVE_OSC_ICHANNEL_SYNC).connected ?
+                         BSE_MODULE_IBUFFER (module, BSE_WAVE_OSC_ICHANNEL_SYNC) : NULL),
+                        BSE_MODULE_OBUFFER (module, BSE_WAVE_OSC_OCHANNEL_WAVE));
   
   gate = wosc->done ? 0.0 : 1.0;
-  module->ostreams[BSE_WAVE_OSC_OCHANNEL_GATE].values = gsl_engine_const_values (gate);
+  module->ostreams[BSE_WAVE_OSC_OCHANNEL_GATE].values = bse_engine_const_values (gate);
 }
 
 static void
-wosc_reset (GslModule *module)
+wosc_reset (BseModule *module)
 {
   GslWaveOscData *wosc = module->user_data;
   gsl_wave_osc_reset (wosc);
@@ -476,9 +421,9 @@ wosc_reset (GslModule *module)
 static void
 bse_wave_osc_context_create (BseSource *source,
                              guint      context_handle,
-                             GslTrans  *trans)
+                             BseTrans  *trans)
 {
-  static const GslClass wosc_class = {
+  static const BseModuleClass wosc_class = {
     BSE_WAVE_OSC_N_ICHANNELS,   /* n_istreams */
     0,                          /* n_jstreams */
     BSE_WAVE_OSC_N_OCHANNELS,   /* n_ostreams */
@@ -486,22 +431,22 @@ bse_wave_osc_context_create (BseSource *source,
     NULL,                       /* process_defer */
     wosc_reset,                 /* reset */
     wosc_free,                  /* free */
-    GSL_COST_NORMAL,            /* cost */
+    BSE_COST_NORMAL,            /* cost */
   };
   BseWaveOsc *self = BSE_WAVE_OSC (source);
   GslWaveOscData *wosc = g_new0 (GslWaveOscData, 1);
-  GslModule *module;
+  BseModule *module;
   
   gsl_wave_osc_init (wosc);
   gsl_wave_osc_config (wosc, &self->config);
   
-  module = gsl_module_new (&wosc_class, wosc);
+  module = bse_module_new (&wosc_class, wosc);
   
   /* setup module i/o streams with BseSource i/o channels */
   bse_source_set_context_module (source, context_handle, module);
   
   /* commit module to engine */
-  gsl_trans_add (trans, gsl_job_integrate (module));
+  bse_trans_add (trans, bse_job_integrate (module));
   
   /* chain parent class' handler */
   BSE_SOURCE_CLASS (parent_class)->context_create (source, context_handle, trans);
@@ -516,13 +461,11 @@ typedef struct {
 } PcmPos;
 
 static void
-pcm_pos_access (GslModule *module,
+pcm_pos_access (BseModule *module,      /* EngineThread */
                 gpointer   data)
 {
   GslWaveOscData *wosc = module->user_data;
   PcmPos *pos = data;
-  
-  /* this runs in the GSL engine threads */
 
   pos->stamp = GSL_TICK_STAMP;
   pos->module_pcm_position = gsl_wave_osc_cur_pos (wosc);
@@ -535,13 +478,11 @@ pcm_pos_access (GslModule *module,
 }
 
 static void
-pcm_pos_access_free (gpointer data)
+pcm_pos_access_free (gpointer data)     /* UserThread */
 {
   PcmPos *pos = data;
   BseWaveOsc *self = pos->wosc;
   
-  /* this is guaranteed by the GSL engine to be run in user thread */
-  
   if (pos->perc < 0)
     g_signal_emit (self, signal_notify_pcm_position, 0, pos->stamp, pos->module_pcm_position);
   
@@ -550,18 +491,41 @@ pcm_pos_access_free (gpointer data)
 }
 
 void
-bse_wave_osc_request_pcm_position (BseWaveOsc *self,
-                                   gfloat      perc)
+bse_wave_osc_mass_seek (guint              n_woscs,
+                        BseWaveOsc       **woscs,
+                        gfloat             perc)
+{
+  guint i;
+  g_return_if_fail (perc >= 0 && perc <= 100);
+  BseTrans *trans = bse_trans_open();
+  for (i = 0; i < n_woscs; i++)
+    {
+      BseWaveOsc *wosc = woscs[i];
+      g_return_if_fail (BSE_IS_WAVE_OSC (wosc));
+      if (BSE_SOURCE_PREPARED (wosc))
+        {
+          PcmPos *pos = g_new (PcmPos, 1);
+          pos->perc = perc;
+          pos->wosc = g_object_ref (wosc);
+          bse_source_access_modules (BSE_SOURCE (pos->wosc),
+                                     pcm_pos_access,
+                                     pos,
+                                     pcm_pos_access_free,
+                                     NULL);
+        }
+    }
+  bse_trans_commit (trans);
+}
+
+void
+bse_wave_osc_request_pcm_position (BseWaveOsc *self)
 {
   g_return_if_fail (BSE_IS_WAVE_OSC (self));
-
-  /* we seek for 0 <= perc <= 100 and we emit notification for perc < 0 */
   
   if (BSE_SOURCE_PREPARED (self))
     {
       PcmPos *pos = g_new (PcmPos, 1);
-      
-      pos->perc = perc;
+      pos->perc = -1;
       pos->wosc = g_object_ref (self);
       bse_source_access_modules (BSE_SOURCE (self),
                                  pcm_pos_access,
@@ -570,3 +534,67 @@ bse_wave_osc_request_pcm_position (BseWaveOsc *self,
                                  NULL);
     }
 }
+
+static void
+bse_wave_osc_class_init (BseWaveOscClass *class)
+{
+  GObjectClass *gobject_class = G_OBJECT_CLASS (class);
+  BseObjectClass *object_class = BSE_OBJECT_CLASS (class);
+  BseItemClass *item_class = BSE_ITEM_CLASS (class);
+  BseSourceClass *source_class = BSE_SOURCE_CLASS (class);
+  guint ochannel, ichannel;
+  
+  parent_class = g_type_class_peek_parent (class);
+  
+  gobject_class->set_property = bse_wave_osc_set_property;
+  gobject_class->get_property = bse_wave_osc_get_property;
+  gobject_class->finalize = bse_wave_osc_finalize;
+  gobject_class->dispose = bse_wave_osc_dispose;
+  
+  item_class->get_candidates = bse_wave_osc_get_candidates;
+  
+  source_class->context_create = bse_wave_osc_context_create;
+  
+  bse_object_class_add_param (object_class, _("Wave"),
+                              PARAM_WAVE,
+                              bse_param_spec_object ("wave", _("Wave"), _("Wave used as oscillator source"),
+                                                     BSE_TYPE_WAVE, SFI_PARAM_STANDARD));
+  bse_object_class_add_param (object_class, _("Wave"),
+                              PARAM_CHANNEL,
+                              sfi_pspec_int ("channel", _("Channel"), _("The audio channel to play, usually 1 is left, 2 is right"),
+                                             1, 1, 256, 2, SFI_PARAM_STANDARD));
+  bse_object_class_add_param (object_class, _("Modulation"),
+                              PARAM_FM_PERC,
+                              sfi_pspec_real ("fm_perc", _("Input Modulation [%]"),
+                                              _("Modulation Strength for linear frequency modulation"),
+                                              10.0, 0, 100.0,5.0,
+                                              SFI_PARAM_STANDARD ":scale"));
+  bse_object_class_add_param (object_class, _("Modulation"),
+                              PARAM_FM_EXP,
+                              sfi_pspec_bool ("exponential_fm", _("Exponential FM"),
+                                              _("Perform exponential frequency modulation "
+                                                "instead of linear"),
+                                              FALSE, SFI_PARAM_STANDARD));
+  bse_object_class_add_param (object_class, _("Modulation"),
+                              PARAM_FM_OCTAVES,
+                              sfi_pspec_real ("fm_n_octaves", _("Octaves"),
+                                              _("Number of octaves to be affected by exponential frequency modulation"),
+                                              1.0, 0, 3.0, 0.01,
+                                              SFI_PARAM_STANDARD ":scale"));
+  
+  signal_notify_pcm_position = bse_object_class_add_signal (object_class, "notify_pcm_position",
+                                                            G_TYPE_NONE, 2,
+                                                            SFI_TYPE_NUM,
+                                                            G_TYPE_INT);
+  
+  ichannel = bse_source_class_add_ichannel (source_class, "freq-in", _("Freq In"), _("Frequency Input"));
+  g_assert (ichannel == BSE_WAVE_OSC_ICHANNEL_FREQ);
+  ichannel = bse_source_class_add_ichannel (source_class, "sync-in", _("Sync In"), _("Syncronization Input"));
+  g_assert (ichannel == BSE_WAVE_OSC_ICHANNEL_SYNC);
+  ichannel = bse_source_class_add_ichannel (source_class, "mod-in", _("Mod In"), _("Modulation Input"));
+  g_assert (ichannel == BSE_WAVE_OSC_ICHANNEL_MOD);
+  ochannel = bse_source_class_add_ochannel (source_class, "audio-out", _("Audio Out"), _("Wave Output"));
+  g_assert (ochannel == BSE_WAVE_OSC_OCHANNEL_WAVE);
+  ochannel = bse_source_class_add_ochannel (source_class, "gate-out", _("Gate Out"), _("Gate Output"));
+  g_assert (ochannel == BSE_WAVE_OSC_OCHANNEL_GATE);
+}
