@@ -26,11 +26,14 @@ G_BEGIN_DECLS
 
 /* linear-read handle. needs buffering handle wrapper
  */
-GslDataHandle*	gsl_data_handle_new_mad		(const gchar *file_name,
-                                                 gfloat       osc_freq);
-GslErrorType	gsl_data_handle_mad_testopen	(const gchar *file_name,
-						 guint       *n_channels,
-						 gfloat      *mix_freq);
+GslDataHandle*	gsl_data_handle_new_mad		(const gchar  *file_name,
+                                                 gfloat        osc_freq);
+GslDataHandle*	gsl_data_handle_new_mad_err	(const gchar  *file_name,
+                                                 gfloat        osc_freq,
+                                                 GslErrorType *error);
+GslErrorType	gsl_data_handle_mad_testopen	(const gchar  *file_name,
+						 guint        *n_channels,
+						 gfloat       *mix_freq);
 const gchar*    gsl_data_handle_mad_version     (void);
 
 G_END_DECLS
