@@ -18,13 +18,13 @@
  */
 #include "gslosctable.h"
 #include "gslcommon.h"
-#include "gslmath.h"
+#include "bsemath.h"
 #include "gslfft.h"
 #include <string.h>
 #include <sfi/gbsearcharray.h>
 
 
-#define OSC_DEBUG		sfi_debug_keyfunc ("osc")
+#define OSC_DEBUG(...)  sfi_debug ("osc", __VA_ARGS__)
 
 #define	OSC_FREQ_EPSILON	(1e-3)	/* range within which frequencies are "equal" */
 
@@ -188,7 +188,7 @@ osc_table_entry_lookup_best (const GslOscTable *table,
   /* get exact match or a match which is one off into either direction */
   key.mfreq = mfreq;
   ep = g_bsearch_array_lookup_sibling (table->entry_array, &osc_taconfig, &k);
-  if_reject (!ep)
+  if (UNLIKELY (!ep))
     return NULL;	/* ugh, bad */
   
   if (mfreq > (*ep)->mfreq)	/* need better filter */
@@ -463,7 +463,7 @@ gsl_osc_wave_fill_buffer (GslOscWaveForm type,
       for (i = 0; i < n_values; i++)
 	{
 	  frac = ((gdouble) i) / max; /* [0..1[ */
-	  pos = frac * 2. * GSL_PI;
+	  pos = frac * 2. * PI;
 	  values[i] = sin (pos);
 	}
       break;
@@ -603,7 +603,7 @@ gsl_osc_wave_adjust_range (guint   n_values,
   max = fabs (max + center);
   if (min > max)
     max = min;
-  if (max > GSL_FLOAT_MIN_NORMAL)
+  if (max > BSE_FLOAT_MIN_NORMAL)
     max = new_max / max;
   else
     max = 0;
