@@ -22,7 +22,7 @@
 #define CHECK_FREQ		(WOSC_MIX_VARIANT & WOSC_MIX_WITH_FREQ)
 #define CHECK_MOD		(WOSC_MIX_VARIANT & WOSC_MIX_WITH_MOD)
 #define EXPONENTIAL_FM		(WOSC_MIX_VARIANT & WOSC_MIX_WITH_EXP_FM)
-#define DIRSTRIDE(b)		(b->dirstride)	 /* (1) change for n_channel stepping */
+#define DIRSTRIDE(b)		(b->dirstride)	 /* changes according to n_channels and directions */
 
 
 static void
@@ -51,37 +51,36 @@ WOSC_MIX_VARIANT_NAME (GslWaveOscData *wosc,
       if (CHECK_SYNC)
 	{
 	  gfloat sync_level = *sync_in++;
-	  if_reject (GSL_SIGNAL_RAISING_EDGE (last_sync_level, sync_level))
+	  if (UNLIKELY (BSE_SIGNAL_RAISING_EDGE (last_sync_level, sync_level)))
 	    {
 	      wosc->j = wosc_j;
-	      gsl_wave_osc_retrigger (wosc, CHECK_FREQ ? GSL_SIGNAL_TO_FREQ (*freq_in) : wosc->config.cfreq);
+	      gsl_wave_osc_retrigger (wosc, CHECK_FREQ ? BSE_SIGNAL_TO_FREQ (*freq_in) : wosc->config.cfreq);
 	      /* retrigger alters last_freq and last_mod */
 	      last_freq_level = wosc->last_freq_level;
 	      last_mod_level = wosc->last_mod_level;
 	      wosc_j = wosc->j;
 	      boundary = block->end;
-	      /* FIXME: g_assert (ABS (block->dirstride) == 1); */
 	      last_sync_level = sync_level;
 	    }
 	}
       if (CHECK_MOD && CHECK_FREQ)
 	{
 	  gfloat mod_level = *mod_in++, freq_level = *freq_in++;
-	  if_reject (GSL_SIGNAL_FREQ_CHANGED (last_freq_level, freq_level))
+	  if (UNLIKELY (BSE_SIGNAL_FREQ_CHANGED (last_freq_level, freq_level)))
 	    {
 	      last_freq_level = freq_level;
-	      if (GSL_SIGNAL_MOD_CHANGED (last_mod_level, mod_level))
+	      if (BSE_SIGNAL_MOD_CHANGED (last_mod_level, mod_level))
 		last_mod_level = mod_level;
 	      goto UPDATE_FREQ;
 	    }
-          else if_reject (GSL_SIGNAL_MOD_CHANGED (last_mod_level, mod_level))
+          else if (UNLIKELY (BSE_SIGNAL_MOD_CHANGED (last_mod_level, mod_level)))
 	    {
 	      gfloat new_freq;
 	      last_mod_level = mod_level;
 	    UPDATE_FREQ:
-	      new_freq = GSL_SIGNAL_TO_FREQ (freq_level);
+	      new_freq = BSE_SIGNAL_TO_FREQ (freq_level);
 	      if (EXPONENTIAL_FM)
-		new_freq *= gsl_signal_exp2 (wosc->config.fm_strength * mod_level);
+		new_freq *= bse_signal_exp2 (wosc->config.fm_strength * mod_level);
 	      else /* LINEAR_FM */
 		new_freq *= 1.0 + wosc->config.fm_strength * mod_level;
 	      wave_osc_transform_filter (wosc, new_freq);
@@ -90,11 +89,11 @@ WOSC_MIX_VARIANT_NAME (GslWaveOscData *wosc,
       else if (CHECK_MOD)
 	{
 	  gfloat mod_level = *mod_in++;
-	  if (GSL_SIGNAL_MOD_CHANGED (last_mod_level, mod_level))
+	  if (BSE_SIGNAL_MOD_CHANGED (last_mod_level, mod_level))
 	    {
 	      gfloat new_freq = wosc->config.cfreq;
 	      if (EXPONENTIAL_FM)
-		new_freq *= gsl_signal_exp2 (wosc->config.fm_strength * mod_level);
+		new_freq *= bse_signal_exp2 (wosc->config.fm_strength * mod_level);
 	      else /* LINEAR_FM */
 		new_freq *= 1.0 + wosc->config.fm_strength * mod_level;
 	      last_mod_level = mod_level;
@@ -104,10 +103,10 @@ WOSC_MIX_VARIANT_NAME (GslWaveOscData *wosc,
       else if (CHECK_FREQ)
 	{
 	  gfloat freq_level = *freq_in++;
-	  if (GSL_SIGNAL_FREQ_CHANGED (last_freq_level, freq_level))
+	  if (BSE_SIGNAL_FREQ_CHANGED (last_freq_level, freq_level))
 	    {
 	      last_freq_level = freq_level;
-	      wave_osc_transform_filter (wosc, GSL_SIGNAL_TO_FREQ (freq_level));
+	      wave_osc_transform_filter (wosc, BSE_SIGNAL_TO_FREQ (freq_level));
 	    }
 	}
       
@@ -118,7 +117,7 @@ WOSC_MIX_VARIANT_NAME (GslWaveOscData *wosc,
 	  gfloat d, d0, d1, d2, d3, d4, d5, d6, d7;
 	  gfloat *x;
 
-	  if_reject (wosc->x == boundary)       /* wchunk block boundary */
+	  if (UNLIKELY (wosc->x >= boundary))       /* wchunk block boundary */
 	    {
 	      GslLong next_offset = block->next_offset;
 
@@ -126,12 +125,11 @@ WOSC_MIX_VARIANT_NAME (GslWaveOscData *wosc,
 	      block->play_dir = wosc->config.play_dir;
 	      block->offset = next_offset;
 	      gsl_wave_chunk_use_block (wosc->wchunk, block);
-	      wosc->x = block->start;
+	      wosc->x = block->start + CLAMP (wosc->config.channel, 0, wosc->wchunk->n_channels - 1);
 	      boundary = block->end;
-	      /* FIXME: g_assert (ABS (block->dirstride) == 1); */
 	    }
 	  
-	  if_expect (block->dirstride > 0)
+	  if (LIKELY (block->dirstride > 0))
 	    {
 	      x = wosc->x;
 	      d0 = b[0] * y[wosc_j]; wosc_j++; wosc_j &= 0x7;
