@@ -144,44 +144,6 @@ g_strdup_lstrip (const gchar *string)
   return NULL;
 }
 
-gchar*
-g_strconcat_with_null (const gchar *string1, ...)
-{
-  gsize   l;
-  va_list args;
-  gchar   *s;
-  gchar   *concat;
-  gchar   *ptr;
-
-  if (!string1)
-    return NULL;
-
-  l = 1 + strlen (string1);
-  va_start (args, string1);
-  s = va_arg (args, gchar*);
-  while (s)
-    {
-      l += strlen (s);
-      s = va_arg (args, gchar*);
-    }
-  va_end (args);
-
-  concat = g_new (gchar, l);
-  ptr = concat;
-
-  ptr = g_stpcpy (ptr, string1);
-  va_start (args, string1);
-  s = va_arg (args, gchar*);
-  while (s)
-    {
-      ptr = g_stpcpy (ptr, s);
-      s = va_arg (args, gchar*);
-    }
-  va_end (args);
-
-  return concat;
-}
-
 const gchar*
 g_intern_string (const gchar *string)
 {
@@ -240,6 +202,25 @@ g_intern_strconcat (const gchar *first_string,
   return c;
 }
 
+const gchar*
+g_intern_printf (const gchar   *format,
+                 ...)
+{
+  if (format)
+    {
+      gchar *buffer;
+      va_list args;
+      va_start (args, format);
+      buffer = g_strdup_vprintf (format, args);
+      va_end (args);
+      const gchar *istring = g_intern_string (buffer);
+      g_free (buffer);
+      return istring;
+    }
+  else
+    return NULL;
+}
+
 gchar*
 g_path_concat (const gchar *first_path,
                ...)
@@ -252,6 +233,25 @@ g_path_concat (const gchar *first_path,
   return s;
 }
 
+GString*
+g_string_prefix_lines (GString     *gstring,
+                       const gchar *pstr)
+{
+  if (gstring->len && pstr)
+    {
+      guint l = strlen (pstr);
+      gchar *p = gstring->str - 1;
+      while (p)
+        {
+          guint pos = p - gstring->str + 1;
+          g_string_insert (gstring, pos, pstr);
+          pos += l;
+          p = strchr (gstring->str + pos, '\n');
+        }
+    }
+  return gstring;
+}
+
 
 /* --- string options --- */
 gchar*
@@ -280,15 +280,19 @@ g_option_find_value (const gchar *option_string,
 
   /* try first match */
   p = strstr (option_string, option);
-  if (p && (p == option_string || p[-1] == ':') &&
-      (p[l] == '-' || p[l] == '+' || p[l] == ':' || p[l] == 0))
+  if (p &&
+      (p == option_string || p[-1] == ':') &&
+      (p[l] == ':' || p[l] == 0 || p[l] == '=' ||
+       ((p[l] == '-' || p[l] == '+') && (p[l + 1] == ':' || p[l + 1] == 0))))
     match = p;
   /* allow later matches to override */
   while (p)
     {
       p = strstr (p + l, option);
-      if (p && p[-1] == ':' &&
-          (p[l] == '-' || p[l] == '+' || p[l] == ':' || p[l] == 0))
+      if (p &&
+          p[-1] == ':' &&
+          (p[l] == ':' || p[l] == 0 || p[l] == '=' ||
+           ((p[l] == '-' || p[l] == '+') && (p[l + 1] == ':' || p[l + 1] == 0))))
         match = p;
     }
   return match ? match + l : NULL;
