@@ -79,6 +79,32 @@ public:
 
 
 
+/* enum declarations */
+BSE_CXX_DECLARE_ENUM (FunkynessType, "NamespaceFunkynessType", 3,
+  *v++ = ::Bse::EnumValue (REALLY_FUNKY, "NAMESPACE_REALLY_FUNKY", "namespace-really-funky" );
+  *v++ = ::Bse::EnumValue (MODERATELY_FUNKY, "NAMESPACE_MODERATELY_FUNKY", "namespace-moderately-funky" );
+  *v++ = ::Bse::EnumValue (NON_FUNKY, "NAMESPACE_NON_FUNKY", "No Funk At All" );
+);
+
+
+/* sequence type declarations */
+BSE_CXX_DECLARE_SEQUENCE (TestSequence);
+
+
+
+/* record type declarations */
+BSE_CXX_DECLARE_RECORD (TestRecord);
+
+
+
+/* procedure prototypes */
+namespace Procedure {
+class test_exception;
+#define NAMESPACE_TYPE_TEST_EXCEPTION		BSE_CXX_DECLARED_PROC_TYPE (Namespace, test_exception)
+} // Procedure
+
+
+
 /* class definitions */
 BSE_CXX_DECLARE_CLASS (TestObject);
 class TestObjectBase : public ::Bse::Effect {
@@ -94,15 +120,6 @@ public:
   static inline const char* authors   () { return ""; }
   static inline const char* license   () { return ""; }
   static inline const char* type_name () { return "NamespaceTestObject"; }
-protected:
-  Sfi::Int aninteger;
-  ::Namespace::TestRecordHandle r;
-  ::Namespace::TestSequence s;
-  Sfi::BBlock bb;
-  Sfi::FBlock fb;
-  Sfi::Rec rr;
-  ::Bse::Effect* ef;
-  BseItem* it;
 protected:
   enum TestObjectPropertyID {
     PROP_ANINTEGER = 1,
@@ -114,8 +131,48 @@ protected:
     PROP_EF,
     PROP_IT,
   };
+protected:
+  typedef void AutoUpdateCategory;
+protected:
+  Sfi::Int aninteger;
+  ::Namespace::TestRecordHandle r;
+  ::Namespace::TestSequence s;
+  Sfi::BBlock bb;
+  Sfi::FBlock fb;
+  Sfi::Rec rr;
+  ::Bse::Effect* ef;
+  BseItem* it;
 public:
-  void set_property (guint prop_id, const ::Bse::Value &value, GParamSpec *pspec)
+  void get_property (TestObjectPropertyID prop_id, ::Bse::Value &value, GParamSpec *pspec)
+  {
+    switch (prop_id) {
+    case PROP_ANINTEGER:
+      sfi_value_set_int (&value, aninteger);
+    break;
+    case PROP_R:
+      ::Namespace::TestRecord::value_set_boxed (&value, r);
+    break;
+    case PROP_S:
+      ::Namespace::TestSequence::value_set_boxed (&value, s);
+    break;
+    case PROP_BB:
+      ::Sfi::BBlock::value_set_bblock (&value, bb);
+    break;
+    case PROP_FB:
+      ::Sfi::FBlock::value_set_fblock (&value, fb);
+    break;
+    case PROP_RR:
+      ::Sfi::Rec::value_set_rec (&value, rr);
+    break;
+    case PROP_EF:
+      ::Bse::CxxBase::value_set_casted< Bse::Effect, Bse::EffectBase> (&value, ef);
+    break;
+    case PROP_IT:
+      ::Bse::CxxBase::value_set_gobject (&value, it);
+    break;
+    };
+  }
+  void set_property (TestObjectPropertyID prop_id, const ::Bse::Value &value, GParamSpec *pspec)
   {
     switch (prop_id) {
     case PROP_ANINTEGER:
@@ -149,33 +206,17 @@ public:
     default: ;
     };
   }
-  void get_property (guint prop_id, ::Bse::Value &value, GParamSpec *pspec)
+  virtual bool editable_property (TestObjectPropertyID prop_id, GParamSpec *pspec)
+  {
+    return true;
+  }
+  virtual void get_candidates (TestObjectPropertyID prop_id, ::Bse::PropertyCandidatesHandle &pch, GParamSpec *pspec)
+  {
+  }
+  void property_updated (TestObjectPropertyID prop_id, guint64 tick_stamp, double prop_value, GParamSpec *pspec)
   {
     switch (prop_id) {
-    case PROP_ANINTEGER:
-      sfi_value_set_int (&value, aninteger);
-    break;
-    case PROP_R:
-      ::Namespace::TestRecord::value_set_boxed (&value, r);
-    break;
-    case PROP_S:
-      ::Namespace::TestSequence::value_set_boxed (&value, s);
-    break;
-    case PROP_BB:
-      ::Sfi::BBlock::value_set_bblock (&value, bb);
-    break;
-    case PROP_FB:
-      ::Sfi::FBlock::value_set_fblock (&value, fb);
-    break;
-    case PROP_RR:
-      ::Sfi::Rec::value_set_rec (&value, rr);
-    break;
-    case PROP_EF:
-      ::Bse::CxxBase::value_set_casted< Bse::Effect, Bse::EffectBase> (&value, ef);
-    break;
-    case PROP_IT:
-      ::Bse::CxxBase::value_set_gobject (&value, it);
-    break;
+    default: ;
     };
   }
 private:
@@ -191,32 +232,6 @@ public:
 };
 
 
-/* sequence type declarations */
-BSE_CXX_DECLARE_SEQUENCE (TestSequence);
-
-
-
-/* record type declarations */
-BSE_CXX_DECLARE_RECORD (TestRecord);
-
-
-
-/* procedure prototypes */
-namespace Procedure {
-class test_exception;
-#define NAMESPACE_TYPE_TEST_EXCEPTION		BSE_CXX_DECLARED_PROC_TYPE (Namespace, test_exception)
-} // Procedure
-
-
-
-/* enum implementations */
-BSE_CXX_DECLARE_ENUM (FunkynessType, "NamespaceFunkynessType", 3,
-  *v++ = ::Bse::EnumValue (REALLY_FUNKY, "NAMESPACE_REALLY_FUNKY", "namespace-really-funky" );
-  *v++ = ::Bse::EnumValue (MODERATELY_FUNKY, "NAMESPACE_MODERATELY_FUNKY", "namespace-moderately-funky" );
-  *v++ = ::Bse::EnumValue (NON_FUNKY, "NAMESPACE_NON_FUNKY", "No Funk At All" );
-);
-
-
 /* choice implementations */
 static inline SfiChoiceValues
 FunkynessType_choice_values()
@@ -371,14 +386,19 @@ TestSequence::get_element()
 void
 TestObjectBase::class_init (::Bse::CxxBaseClass *klass)
 {
+  klass->set_accessors (::Bse::cxx_get_property_trampoline<TestObjectBase, TestObjectPropertyID>,
+                        ::Bse::cxx_set_property_trampoline<TestObjectBase, TestObjectPropertyID>,
+                        ::Bse::cxx_editable_property_trampoline<TestObjectBase, TestObjectPropertyID>,
+                        ::Bse::cxx_get_candidates_trampoline<TestObjectBase, TestObjectPropertyID>,
+                        NULL);
   klass->add_param (PROP_ANINTEGER, sfidl_pspec_Int_default (NULL,"aninteger"));
   klass->add_param (PROP_R, sfidl_pspec_BoxedRec_default (NULL, "r", NAMESPACE_TYPE_TEST_RECORD));
   klass->add_param (PROP_S, sfidl_pspec_BoxedSeq_default (NULL, "s", NAMESPACE_TYPE_TEST_SEQUENCE));
   klass->add_param (PROP_BB, sfidl_pspec_BBlock_default (NULL,"bb"));
   klass->add_param (PROP_FB, sfidl_pspec_FBlock_default (NULL,"fb"));
   klass->add_param (PROP_RR, sfidl_pspec_Rec_default (NULL, "rr", ));
-  klass->add_param (PROP_EF, sfidl_pspec_Object_default (NULL, "ef", BSE_TYPE_EFFECT));
-  klass->add_param (PROP_IT, sfidl_pspec_Object_default (NULL, "it", BSE_TYPE_ITEM));
+  klass->add_param (PROP_EF, sfidl_pspec_TypedObject_default (NULL, "ef", BSE_TYPE_EFFECT));
+  klass->add_param (PROP_IT, sfidl_pspec_TypedObject_default (NULL, "it", BSE_TYPE_ITEM));
 }
 
 
@@ -425,7 +445,7 @@ public:
                     GParamSpec       **out_pspecs)
   {
     *(in_pspecs++) = sfidl_pspec_Int_default (NULL,"i");
-    *(in_pspecs++) = sfidl_pspec_Object_default (NULL, "o", NAMESPACE_TYPE_TEST_OBJECT);
+    *(in_pspecs++) = sfidl_pspec_TypedObject_default (NULL, "o", NAMESPACE_TYPE_TEST_OBJECT);
     *(in_pspecs++) = sfidl_pspec_Int_default (NULL,"bar");
     *(in_pspecs++) = sfidl_pspec_GEnum_default (NULL, "ft", NAMESPACE_TYPE_FUNKYNESS_TYPE);
     *(out_pspecs++) = sfidl_pspec_Int_default (NULL,"result");
